@@ -20,14 +20,20 @@
 extern "C" {
 #endif
 
-#define GLOB		0	/* Error maps */
-#define GLOBERRS	28	/* Max number of error messages in map's */
-#define EE(X)	globerrs[ X ]	/* Defines to add error to right map */
+#define GLOBERRS (EE_ERROR_LAST - EE_ERROR_FIRST + 1) /* Nr of global errors */
+#define EE(X)    (globerrs[(X) - EE_ERROR_FIRST])
 
 extern const char * NEAR globerrs[];	/* my_error_messages is here */
 
 /* Error message numbers in global map */
-#define EE_FILENOTFOUND		0
+/*
+  Do not add error numbers before EE_ERROR_FIRST.
+  If necessary to add lower numbers, change EE_ERROR_FIRST accordingly.
+
+  We start with error 1 to not confuse peoples with 'error 0'
+*/
+
+#define EE_ERROR_FIRST          1 /*Copy first error nr.*/
 #define EE_CANTCREATEFILE	1
 #define EE_READ			2
 #define EE_WRITE		3
@@ -54,6 +60,10 @@ extern const char * NEAR globerrs[];	/* my_error_messages is here */
 #define EE_CANT_SYMLINK		25
 #define EE_REALPATH		26
 #define EE_SYNC			27
+#define EE_UNKNOWN_COLLATION	28
+#define EE_FILENOTFOUND		29
+#define EE_ERROR_LAST           29 /*Copy last error nr.*/
+/* Add error numbers before EE_ERROR_LAST and change it accordingly. */
 
   /* exit codes for all MySQL programs */
 
@@ -68,6 +78,7 @@ extern const char * NEAR globerrs[];	/* my_error_messages is here */
 #define EXIT_UNKNOWN_SUFFIX		9
 #define EXIT_NO_PTR_TO_VARIABLE		10
 #define EXIT_CANNOT_CONNECT_TO_SERVICE	11
+#define EXIT_OPTION_DISABLED            12
 
 
 #ifdef	__cplusplus
