@@ -109,7 +109,7 @@ do {\
 \
 		while (struct3333->NAME != DATA) {\
 \
-			ut_ad(struct3333);\
+			ut_a(struct3333);\
 			struct3333 = struct3333->NAME;\
 		}\
 \
@@ -305,6 +305,8 @@ struct hash_cell_struct{
 
 /* The hash table structure */
 struct hash_table_struct {
+	ibool		adaptive;/* TRUE if this is the hash table of the
+				adaptive hash index */
 	ulint		n_cells;/* number of cells in the hash table */
 	hash_cell_t*	array;	/* pointer to cell array */
 	ulint		n_mutexes;/* if mutexes != NULL, then the number of
