@@ -109,9 +109,9 @@ must be reset explicitly by calling sync_os_reset_event. */
 os_event_t
 os_event_create(
 /*============*/
-			/* out: the event handle */
-	char*	name)	/* in: the name of the event, if NULL
-			the event is created without a name */
+				/* out: the event handle */
+	const char*	name)	/* in: the name of the event, if NULL
+				the event is created without a name */
 {
 #ifdef __WIN__
         os_event_t event;
@@ -121,11 +121,11 @@ os_event_create(
 	event->handle = CreateEvent(NULL,/* No security attributes */
 			TRUE,		/* Manual reset */
 			FALSE,		/* Initial state nonsignaled */
-			name);
+			(LPCTSTR) name);
 	if (!event->handle) {
 	        fprintf(stderr,
 "InnoDB: Could not create a Windows event semaphore; Windows error %lu\n",
-		  (ulint)GetLastError());
+		  (ulong) GetLastError());
 	}
 #else /* Unix */
 	os_event_t	event;
@@ -166,9 +166,9 @@ reset when a single thread is released. Works only in Windows. */
 os_event_t
 os_event_create_auto(
 /*=================*/
-			/* out: the event handle */
-	char*	name)	/* in: the name of the event, if NULL
-			the event is created without a name */
+				/* out: the event handle */
+	const char*	name)	/* in: the name of the event, if NULL
+				the event is created without a name */
 {
         os_event_t event;
 
@@ -177,12 +177,12 @@ os_event_create_auto(
 	event->handle = CreateEvent(NULL,/* No security attributes */
 			FALSE,		/* Auto-reset */
 			FALSE,		/* Initial state nonsignaled */
-			name);
+			(LPCTSTR) name);
 
 	if (!event->handle) {
 	        fprintf(stderr,
 "InnoDB: Could not create a Windows auto event semaphore; Windows error %lu\n",
-		  (ulint)GetLastError());
+		  (ulong) GetLastError());
 	}
 
         /* Put to the list of events */
@@ -317,28 +317,28 @@ os_event_wait(
 	os_fast_mutex_lock(&(event->os_mutex));
 
 	old_signal_count = event->signal_count;
-loop:
-	if (event->is_set == TRUE
-            || event->signal_count != old_signal_count) {
 
-		os_fast_mutex_unlock(&(event->os_mutex));
+	for (;;) {
+		if (event->is_set == TRUE
+			|| event->signal_count != old_signal_count) {
 
-		if (srv_shutdown_state == SRV_SHUTDOWN_EXIT_THREADS) {
+			os_fast_mutex_unlock(&(event->os_mutex));
 
-		        os_thread_exit(NULL);
+			if (srv_shutdown_state == SRV_SHUTDOWN_EXIT_THREADS) {
+
+				os_thread_exit(NULL);
+			}
+			/* Ok, we may return */
+
+			return;
 		}
-		/* Ok, we may return */
 
-		return;
+		pthread_cond_wait(&(event->cond_var), &(event->os_mutex));
+
+		/* Solaris manual said that spurious wakeups may occur: we
+		have to check if the event really has been signaled after
+		we came here to wait */
 	}
-
-	pthread_cond_wait(&(event->cond_var), &(event->os_mutex));
-
-	/* Solaris manual said that spurious wakeups may occur: we have to
-	check if the event really has been signaled after we came here to
-	wait */
-
-	goto loop;
 #endif
 }
 
@@ -361,7 +361,7 @@ os_event_wait_time(
 	ut_a(event);
 
 	if (time != OS_SYNC_INFINITE_TIME) {
-		err = WaitForSingleObject(event->handle, time / 1000);
+		err = WaitForSingleObject(event->handle, (DWORD) time / 1000);
 	} else {
 		err = WaitForSingleObject(event->handle, INFINITE);
 	}
@@ -408,11 +408,11 @@ os_event_wait_multiple(
 	ut_a(native_event_array);
 	ut_a(n > 0);
 
-	index = WaitForMultipleObjects(n, native_event_array,
+	index = WaitForMultipleObjects((DWORD) n, native_event_array,
 					FALSE,	   /* Wait for any 1 event */
 					INFINITE); /* Infinite wait time
 						   limit */
-	ut_a(index >= WAIT_OBJECT_0);
+	ut_a(index >= WAIT_OBJECT_0);	/* NOTE: Pointless comparision */
 	ut_a(index < WAIT_OBJECT_0 + n);
 
 	if (srv_shutdown_state == SRV_SHUTDOWN_EXIT_THREADS) {
@@ -430,9 +430,9 @@ mutex semaphore of InnoDB itself (mutex_t) should be used where possible. */
 os_mutex_t
 os_mutex_create(
 /*============*/
-			/* out: the mutex handle */
-	char*	name)	/* in: the name of the mutex, if NULL
-			the mutex is created without a name */
+				/* out: the mutex handle */
+	const char*	name)	/* in: the name of the mutex, if NULL
+				the mutex is created without a name */
 {
 #ifdef __WIN__
 	HANDLE		mutex;
@@ -440,7 +440,7 @@ os_mutex_create(
 
 	mutex = CreateMutex(NULL,	/* No security attributes */
 			FALSE,		/* Initial state: no owner */
-			name);
+			(LPCTSTR) name);
 	ut_a(mutex);
 #else
 	os_fast_mutex_t*	mutex;
@@ -631,7 +631,21 @@ os_fast_mutex_free(
 
 	DeleteCriticalSection((LPCRITICAL_SECTION) fast_mutex);
 #else
-	ut_a(0 == pthread_mutex_destroy(fast_mutex));
+	int	ret;
+	
+	ret = pthread_mutex_destroy(fast_mutex);
+
+	if (ret != 0) {
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+"  InnoDB: error: return value %lu when calling\n"
+"InnoDB: pthread_mutex_destroy().\n", (ulint)ret);
+		fprintf(stderr,
+"InnoDB: Byte contents of the pthread mutex at %p:\n", fast_mutex);
+		ut_print_buf(stderr, (const byte*)fast_mutex,
+						sizeof(os_fast_mutex_t));
+		fprintf(stderr, "\n");
+	}
 #endif
 	if (os_sync_mutex_inited) {
 		/* When freeing the last mutexes, we have
