@@ -37,17 +37,18 @@ row_get_rec_sys_field(
 				/* out: value of the field */
 	ulint		type,	/* in: DATA_TRX_ID or DATA_ROLL_PTR */
 	rec_t*		rec,	/* in: record */
-	dict_index_t*	index)	/* in: clustered index */
+	dict_index_t*	index,	/* in: clustered index */
+	const ulint*	offsets)/* in: rec_get_offsets(rec, index) */
 {
-	ulint	pos;
-	byte*	field;
-	ulint	len;
+	ulint		pos;
+	byte*		field;
+	ulint		len;
 
 	ut_ad(index->type & DICT_CLUSTERED);
 
 	pos = dict_index_get_sys_col_pos(index, type);
 
-	field = rec_get_nth_field(rec, pos, &len);
+	field = rec_get_nth_field(rec, offsets, pos, &len);
 
 	if (type == DATA_TRX_ID) {
 
@@ -70,6 +71,7 @@ row_set_rec_sys_field(
 	ulint		type,	/* in: DATA_TRX_ID or DATA_ROLL_PTR */
 	rec_t*		rec,	/* in: record */
 	dict_index_t*	index,	/* in: clustered index */
+	const ulint*	offsets,/* in: rec_get_offsets(rec, index) */
 	dulint		val)	/* in: value to set */
 {
 	ulint	pos;
@@ -77,10 +79,11 @@ row_set_rec_sys_field(
 	ulint	len;
 
 	ut_ad(index->type & DICT_CLUSTERED);
+	ut_ad(rec_offs_validate(rec, index, offsets));
 
 	pos = dict_index_get_sys_col_pos(index, type);
 
-	field = rec_get_nth_field(rec, pos, &len);
+	field = rec_get_nth_field(rec, offsets, pos, &len);
 
 	if (type == DATA_TRX_ID) {
 
@@ -113,6 +116,8 @@ row_build_index_entry(
 	dfield_t*	dfield2;
 	dict_col_t*	col;
 	ulint		i;
+        ulint           storage_len;
+	dtype_t*	cur_type;
 
 	ut_ad(row && index && heap);
 	ut_ad(dtuple_check_typed(row));
@@ -139,10 +144,17 @@ row_build_index_entry(
 
 		/* If a column prefix index, take only the prefix */
 		if (ind_field->prefix_len > 0
-		    && dfield_get_len(dfield2) != UNIV_SQL_NULL
-		    && dfield_get_len(dfield2) > ind_field->prefix_len) {
+		    && dfield_get_len(dfield2) != UNIV_SQL_NULL) {
 			
-			dfield_set_len(dfield, ind_field->prefix_len);
+			cur_type = dict_col_get_type(
+				dict_field_get_col(ind_field));
+
+			storage_len = dtype_get_at_most_n_mbchars(
+				cur_type,
+				ind_field->prefix_len,
+				dfield_get_len(dfield2), dfield2->data);
+
+			dfield_set_len(dfield, storage_len);
 		}
 	}
 
@@ -173,6 +185,9 @@ row_build(
 				the buffer page of this record must be
 				at least s-latched and the latch held
 				as long as the row dtuple is used! */
+	const ulint*	offsets,/* in: rec_get_offsets(rec, index)
+				or NULL, in which case this function
+				will invoke rec_get_offsets() */
 	mem_heap_t*	heap)	/* in: memory heap from which the memory
 				needed is allocated */
 {
@@ -187,14 +202,26 @@ row_build(
 	ulint		row_len;
 	byte*		buf; 
 	ulint		i;
-	
+	mem_heap_t*	tmp_heap	= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
+
 	ut_ad(index && rec && heap);
 	ut_ad(index->type & DICT_CLUSTERED);
 
+	if (!offsets) {
+		offsets = rec_get_offsets(rec, index, offsets_,
+					ULINT_UNDEFINED, &tmp_heap);
+	} else {
+		ut_ad(rec_offs_validate(rec, index, offsets));
+	}
+
 	if (type != ROW_COPY_POINTERS) {
 		/* Take a copy of rec to heap */
-		buf = mem_heap_alloc(heap, rec_get_size(rec));
-		rec = rec_copy(buf, rec);
+		buf = mem_heap_alloc(heap, rec_offs_size(offsets));
+		rec = rec_copy(buf, rec, offsets);
+		/* Avoid a debug assertion in rec_offs_validate(). */
+		rec_offs_make_valid(rec, index, (ulint*) offsets);
 	}
 
 	table = index->table;
@@ -202,11 +229,9 @@ row_build(
 
 	row = dtuple_create(heap, row_len);
 
-	dtuple_set_info_bits(row, rec_get_info_bits(rec));
-	
-	n_fields = dict_index_get_n_fields(index);
+	dtuple_set_info_bits(row, rec_get_info_bits(rec, table->comp));
 
-	ut_ad(n_fields == rec_get_n_fields(rec));
+	n_fields = rec_offs_n_fields(offsets);
 
 	dict_table_copy_types(row, table);
 
@@ -218,13 +243,13 @@ row_build(
 		        col = dict_field_get_col(ind_field);
 			dfield = dtuple_get_nth_field(row,
 						dict_col_get_no(col));
-			field = rec_get_nth_field(rec, i, &len);
+			field = rec_get_nth_field(rec, offsets, i, &len);
 
 			if (type == ROW_COPY_ALSO_EXTERNALS
-			    && rec_get_nth_field_extern_bit(rec, i)) {
+			    && rec_offs_nth_extern(offsets, i)) {
 
 			        field = btr_rec_copy_externally_stored_field(
-							rec, i, &len, heap);
+						rec, offsets, i, &len, heap);
 			}
 
 			dfield_set_data(dfield, field, len);
@@ -233,6 +258,10 @@ row_build(
 
 	ut_ad(dtuple_check_typed(row));
 
+	if (tmp_heap) {
+		mem_heap_free(tmp_heap);
+	}
+
 	return(row);
 }
 
@@ -267,16 +296,25 @@ row_rec_to_index_entry(
 	ulint		len;
 	ulint		rec_len;
 	byte*		buf;
-	
+	mem_heap_t*	tmp_heap	= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
+
 	ut_ad(rec && heap && index);
 	
+	offsets = rec_get_offsets(rec, index, offsets,
+					ULINT_UNDEFINED, &tmp_heap);
+
 	if (type == ROW_COPY_DATA) {
 		/* Take a copy of rec to heap */
-		buf = mem_heap_alloc(heap, rec_get_size(rec));
-		rec = rec_copy(buf, rec);
+		buf = mem_heap_alloc(heap, rec_offs_size(offsets));
+		rec = rec_copy(buf, rec, offsets);
+		/* Avoid a debug assertion in rec_offs_validate(). */
+		rec_offs_make_valid(rec, index, offsets);
 	}
 
-	rec_len = rec_get_n_fields(rec);
+	rec_len = rec_offs_n_fields(offsets);
 	
 	entry = dtuple_create(heap, rec_len);
 
@@ -286,17 +324,21 @@ row_rec_to_index_entry(
 
 	dict_index_copy_types(entry, index, rec_len);
 
-	dtuple_set_info_bits(entry, rec_get_info_bits(rec));
+	dtuple_set_info_bits(entry,
+			rec_get_info_bits(rec, rec_offs_comp(offsets)));
 
 	for (i = 0; i < rec_len; i++) {
 
 		dfield = dtuple_get_nth_field(entry, i);
-		field = rec_get_nth_field(rec, i, &len);
+		field = rec_get_nth_field(rec, offsets, i, &len);
 
 		dfield_set_data(dfield, field, len);
 	}
 
 	ut_ad(dtuple_check_typed(entry));
+	if (tmp_heap) {
+		mem_heap_free(tmp_heap);
+	}
 
 	return(entry);
 }
@@ -336,15 +378,24 @@ row_build_row_ref(
 	byte*		buf;
 	ulint		clust_col_prefix_len;
 	ulint		i;
-	
+	mem_heap_t*	tmp_heap	= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
+
 	ut_ad(index && rec && heap);
-	
+
+	offsets = rec_get_offsets(rec, index, offsets,
+					ULINT_UNDEFINED, &tmp_heap);
+
 	if (type == ROW_COPY_DATA) {
 		/* Take a copy of rec to heap */
 
-		buf = mem_heap_alloc(heap, rec_get_size(rec));
+		buf = mem_heap_alloc(heap, rec_offs_size(offsets));
 
-		rec = rec_copy(buf, rec);
+		rec = rec_copy(buf, rec, offsets);
+		/* Avoid a debug assertion in rec_offs_validate(). */
+		rec_offs_make_valid(rec, index, offsets);
 	}
 
 	table = index->table;
@@ -364,7 +415,7 @@ row_build_row_ref(
 
 		ut_a(pos != ULINT_UNDEFINED);
 	
-		field = rec_get_nth_field(rec, pos, &len);
+		field = rec_get_nth_field(rec, offsets, pos, &len);
 
 		dfield_set_data(dfield, field, len);
 
@@ -377,15 +428,20 @@ row_build_row_ref(
 			dict_index_get_nth_field(clust_index, i)->prefix_len;
 
 		if (clust_col_prefix_len > 0) {
-		    	if (len != UNIV_SQL_NULL
-			    && len > clust_col_prefix_len) {
+		    	if (len != UNIV_SQL_NULL) {
 
-				dfield_set_len(dfield, clust_col_prefix_len);
+				dfield_set_len(dfield,
+				  dtype_get_at_most_n_mbchars(
+					dfield_get_type(dfield),
+					clust_col_prefix_len, len, (char*) field));
 			}
 		}
 	}
 
 	ut_ad(dtuple_check_typed(ref));
+	if (tmp_heap) {
+		mem_heap_free(tmp_heap);
+	}
 
 	return(ref);
 }
@@ -400,12 +456,13 @@ row_build_row_ref_in_tuple(
 	dtuple_t*	ref,	/* in/out: row reference built; see the
 				NOTE below! */
 	dict_index_t*	index,	/* in: index */
-	rec_t*		rec)	/* in: record in the index;
+	rec_t*		rec,	/* in: record in the index;
 				NOTE: the data fields in ref will point
 				directly into this record, therefore,
 				the buffer page of this record must be
 				at least s-latched and the latch held
 				as long as the row reference is used! */
+	trx_t*		trx)	/* in: transaction */
 {
 	dict_index_t*	clust_index;
 	dfield_t*	dfield;
@@ -415,15 +472,19 @@ row_build_row_ref_in_tuple(
 	ulint		pos;
 	ulint		clust_col_prefix_len;
 	ulint		i;
-	
+	mem_heap_t*	heap		= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
+
 	ut_a(ref && index && rec);
 	
 	if (!index->table) {
 		fputs("InnoDB: table ", stderr);
 	notfound:
-		ut_print_name(stderr, index->table_name);
+		ut_print_name(stderr, trx, index->table_name);
 		fputs(" for index ", stderr);
-		ut_print_name(stderr, index->name);
+		ut_print_name(stderr, trx, index->name);
 		fputs(" not found\n", stderr);
 		ut_error;
 	}
@@ -434,7 +495,9 @@ row_build_row_ref_in_tuple(
 		fputs("InnoDB: clust index for table ", stderr);
 		goto notfound;
 	}
-	
+
+	offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
+
 	ref_len = dict_index_get_n_unique(clust_index);
 
 	ut_ad(ref_len == dtuple_get_n_fields(ref));
@@ -447,8 +510,8 @@ row_build_row_ref_in_tuple(
 		pos = dict_index_get_nth_field_pos(index, clust_index, i);
 
 		ut_a(pos != ULINT_UNDEFINED);
-			
-		field = rec_get_nth_field(rec, pos, &len);
+
+		field = rec_get_nth_field(rec, offsets, pos, &len);
 
 		dfield_set_data(dfield, field, len);
 
@@ -461,15 +524,20 @@ row_build_row_ref_in_tuple(
 			dict_index_get_nth_field(clust_index, i)->prefix_len;
 
 		if (clust_col_prefix_len > 0) {
-		    	if (len != UNIV_SQL_NULL
-			    && len > clust_col_prefix_len) {
+		    	if (len != UNIV_SQL_NULL) {
 
-				dfield_set_len(dfield, clust_col_prefix_len);
+				dfield_set_len(dfield,
+				  dtype_get_at_most_n_mbchars(
+					dfield_get_type(dfield),
+					clust_col_prefix_len, len, (char*) field));
 			}
 		}
 	}
 
 	ut_ad(dtuple_check_typed(ref));
+	if (UNIV_LIKELY_NULL(heap)) {
+		mem_heap_free(heap);
+	}
 }
 
 /***********************************************************************
@@ -494,6 +562,7 @@ row_build_row_ref_from_row(
 	dict_col_t*	col;
 	ulint		ref_len;
 	ulint		i;
+	dtype_t*	cur_type;
 	
 	ut_ad(ref && table && row);
 		
@@ -515,10 +584,15 @@ row_build_row_ref_from_row(
 		dfield_copy(dfield, dfield2);
 
 		if (field->prefix_len > 0
-		    && dfield->len != UNIV_SQL_NULL
-		    && dfield->len > field->prefix_len) {
+		    && dfield->len != UNIV_SQL_NULL) {
 
-		        dfield->len = field->prefix_len;
+			cur_type = dict_col_get_type(
+				dict_field_get_col(field));
+
+			dfield->len = dtype_get_at_most_n_mbchars(
+				cur_type,
+				field->prefix_len,
+				dfield->len, dfield->data);
 		}
 	}
 
@@ -542,7 +616,6 @@ row_search_on_row_ref(
 	ulint		low_match;	
 	rec_t*		rec;
 	dict_index_t*	index;
-	page_t*		page;	
 
 	ut_ad(dtuple_check_typed(ref));
 
@@ -555,9 +628,8 @@ row_search_on_row_ref(
 	low_match = btr_pcur_get_low_match(pcur);
 
 	rec = btr_pcur_get_rec(pcur);
-	page = buf_frame_align(rec);
 
-	if (rec == page_get_infimum_rec(page)) {
+	if (page_rec_is_infimum(rec)) {
 
 		return(FALSE);
 	}
@@ -628,7 +700,6 @@ row_search_index_entry(
 {
 	ulint	n_fields;
 	ulint	low_match;
-	page_t*	page;
 	rec_t*	rec;
 
 	ut_ad(dtuple_check_typed(entry));
@@ -637,11 +708,10 @@ row_search_index_entry(
 	low_match = btr_pcur_get_low_match(pcur);
 
 	rec = btr_pcur_get_rec(pcur);
-	page = buf_frame_align(rec);
 
 	n_fields = dtuple_get_n_fields(entry);
 
-	if (rec == page_get_infimum_rec(page)) {
+	if (page_rec_is_infimum(rec)) {
 
 		return(FALSE);
 	}
