@@ -54,16 +54,44 @@ pthread_mutex_t tina_mutex;
 static HASH tina_open_tables;
 static int tina_init= 0;
 
+handlerton tina_hton= {
+  "CSV",
+  SHOW_OPTION_YES,
+  "CSV storage engine", 
+  DB_TYPE_CSV_DB,
+  NULL,    /* One needs to be written! */
+  0,       /* slot */
+  0,       /* savepoint size. */
+  NULL,    /* close_connection */
+  NULL,    /* savepoint */
+  NULL,    /* rollback to savepoint */
+  NULL,    /* release savepoint */
+  NULL,    /* commit */
+  NULL,    /* rollback */
+  NULL,    /* prepare */
+  NULL,    /* recover */
+  NULL,    /* commit_by_xid */
+  NULL,    /* rollback_by_xid */
+  NULL,    /* create_cursor_read_view */
+  NULL,    /* set_cursor_read_view */
+  NULL,    /* close_cursor_read_view */
+  HTON_CAN_RECREATE
+};
+
 /*****************************************************************************
  ** TINA tables
  *****************************************************************************/
 
-/* 
-  Used for sorting  chains.
+/*
+  Used for sorting chains with qsort().
 */
 int sort_set (tina_set *a, tina_set *b)
 {
-  return ( a->begin > b->begin ? 1 : ( a->begin < b->begin ? -1 : 0 ) );
+  /*
+    We assume that intervals do not intersect. So, it is enought to compare
+    any two points. Here we take start of intervals for comparison.
+  */
+  return ( a->begin > b->begin ? -1 : ( a->begin < b->begin ? 1 : 0 ) );
 }
 
 static byte* tina_get_key(TINA_SHARE *share,uint *length,
@@ -162,7 +190,8 @@ static TINA_SHARE *get_share(const char *table_name, TABLE *table)
     thr_lock_init(&share->lock);
     pthread_mutex_init(&share->mutex,MY_MUTEX_INIT_FAST);
 
-    if ((share->data_file= my_open(data_file_name, O_RDWR, MYF(0))) == -1)
+    if ((share->data_file= my_open(data_file_name, O_RDWR|O_APPEND,
+                                   MYF(0))) == -1)
       goto error2;
 
     /* We only use share->data_file for writing, so we scan to the end to append */
@@ -214,6 +243,16 @@ static int free_share(TINA_SHARE *share)
   DBUG_RETURN(result_code);
 }
 
+bool tina_end()
+{
+  if (tina_init)
+  {
+    hash_free(&tina_open_tables);
+    VOID(pthread_mutex_destroy(&tina_mutex));
+  }
+  tina_init= 0;
+  return FALSE;
+}
 
 /* 
   Finds the end of a line.
@@ -228,6 +267,21 @@ byte * find_eoln(byte *data, off_t begin, off_t end)
   return 0;
 }
 
+
+ha_tina::ha_tina(TABLE *table_arg)
+  :handler(&tina_hton, table_arg),
+  /*
+    These definitions are found in hanler.h
+    These are not probably completely right.
+  */
+  current_position(0), next_position(0), chain_alloced(0),
+  chain_size(DEFAULT_CHAIN_LENGTH), records_is_known(0)
+{
+  /* Set our original buffers from pre-allocated memory */
+  buffer.set(byte_buffer, IO_SIZE, system_charset_info);
+  chain= chain_buffer;
+}
+
 /*
   Encode a buffer into the quoted format.
 */
@@ -374,7 +428,7 @@ int ha_tina::find_current_row(byte *buf)
   }
   next_position= (end_ptr - share->mapped_file)+1;
   /* Maybe use \N for null? */
-  memset(buf, 0, table->null_bytes); /* We do not implement nulls! */
+  memset(buf, 0, table->s->null_bytes); /* We do not implement nulls! */
 
   DBUG_RETURN(0);
 }
@@ -383,8 +437,15 @@ int ha_tina::find_current_row(byte *buf)
   If frm_error() is called in table.cc this is called to find out what file
   extensions exist for this handler.
 */
+static const char *ha_tina_exts[] = {
+  ".CSV",
+  NullS
+};
+
 const char **ha_tina::bas_ext() const
-{ static const char *ext[]= { ".CSV", NullS }; return ext; }
+{
+  return ha_tina_exts;
+}
 
 
 /* 
@@ -425,7 +486,7 @@ int ha_tina::write_row(byte * buf)
   int size;
   DBUG_ENTER("ha_tina::write_row");
 
-  statistic_increment(ha_write_count,&LOCK_status);
+  statistic_increment(table->in_use->status_var.ha_write_count, &LOCK_status);
 
   if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_INSERT)
     table->timestamp_field->set_time();
@@ -444,6 +505,7 @@ int ha_tina::write_row(byte * buf)
   */
   if (get_mmap(share, 0) > 0) 
     DBUG_RETURN(-1);
+  records++;
   DBUG_RETURN(0);
 }
 
@@ -461,7 +523,8 @@ int ha_tina::update_row(const byte * old_data, byte * new_data)
   int size;
   DBUG_ENTER("ha_tina::update_row");
 
-  statistic_increment(ha_update_count,&LOCK_status);
+  statistic_increment(table->in_use->status_var.ha_read_rnd_next_count,
+		      &LOCK_status);
 
   if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_UPDATE)
     table->timestamp_field->set_time();
@@ -488,7 +551,7 @@ int ha_tina::update_row(const byte * old_data, byte * new_data)
 int ha_tina::delete_row(const byte * buf)
 {
   DBUG_ENTER("ha_tina::delete_row");
-  statistic_increment(ha_delete_count,&LOCK_status);
+  statistic_increment(table->in_use->status_var.ha_delete_count,&LOCK_status);
 
   if (chain_append())
     DBUG_RETURN(-1);
@@ -607,8 +670,12 @@ int ha_tina::rnd_init(bool scan)
 
   current_position= next_position= 0;
   records= 0;
+  records_is_known= 0;
   chain_ptr= chain;
-  (void)madvise(share->mapped_file,share->file_stat.st_size,MADV_SEQUENTIAL);
+#ifdef HAVE_MADVISE
+  if (scan)
+    (void)madvise(share->mapped_file,share->file_stat.st_size,MADV_SEQUENTIAL);
+#endif
 
   DBUG_RETURN(0);
 }
@@ -628,7 +695,8 @@ int ha_tina::rnd_next(byte *buf)
 {
   DBUG_ENTER("ha_tina::rnd_next");
 
-  statistic_increment(ha_read_rnd_next_count,&LOCK_status);
+  statistic_increment(table->in_use->status_var.ha_read_rnd_next_count,
+		      &LOCK_status);
 
   current_position= next_position;
   if (!share->mapped_file) 
@@ -644,7 +712,7 @@ int ha_tina::rnd_next(byte *buf)
   In the case of an order by rows will need to be sorted.
   ::position() is called after each call to ::rnd_next(), 
   the data it stores is to a byte array. You can store this
-  data via ha_store_ptr(). ref_length is a variable defined to the 
+  data via my_store_ptr(). ref_length is a variable defined to the 
   class that is the sizeof() of position being stored. In our case  
   its just a position. Look at the bdb code if you want to see a case 
   where something other then a number is stored.
@@ -652,21 +720,22 @@ int ha_tina::rnd_next(byte *buf)
 void ha_tina::position(const byte *record)
 {
   DBUG_ENTER("ha_tina::position");
-  ha_store_ptr(ref, ref_length, current_position);
+  my_store_ptr(ref, ref_length, current_position);
   DBUG_VOID_RETURN;
 }
 
 
 /* 
   Used to fetch a row from a posiion stored with ::position(). 
-  ha_get_ptr() retrieves the data for you.
+  my_get_ptr() retrieves the data for you.
 */
 
 int ha_tina::rnd_pos(byte * buf, byte *pos)
 {
   DBUG_ENTER("ha_tina::rnd_pos");
-  statistic_increment(ha_read_rnd_count,&LOCK_status);
-  current_position= ha_get_ptr(pos,ref_length);
+  statistic_increment(table->in_use->status_var.ha_read_rnd_next_count,
+		      &LOCK_status);
+  current_position= my_get_ptr(pos,ref_length);
   DBUG_RETURN(find_current_row(buf));
 }
 
@@ -679,7 +748,7 @@ void ha_tina::info(uint flag)
 {
   DBUG_ENTER("ha_tina::info");
   /* This is a lie, but you don't want the optimizer to see zero or 1 */
-  if (records < 2) 
+  if (!records_is_known && records < 2) 
     records= 2;
   DBUG_VOID_RETURN;
 }
@@ -714,6 +783,8 @@ int ha_tina::rnd_end()
 {
   DBUG_ENTER("ha_tina::rnd_end");
 
+  records_is_known= 1;
+
   /* First position will be truncate position, second will be increment */
   if ((chain_ptr - chain)  > 0)
   {
@@ -736,13 +807,8 @@ int ha_tina::rnd_end()
     qsort(chain, (size_t)(chain_ptr - chain), sizeof(tina_set), (qsort_cmp)sort_set);
     for (ptr= chain; ptr < chain_ptr; ptr++)
     {
-      /* We peek a head to see if this is the last chain */
-      if (ptr+1 == chain_ptr)
-        memmove(share->mapped_file + ptr->begin, share->mapped_file + ptr->end,
-                length - (size_t)ptr->end);
-      else
-        memmove((caddr_t)share->mapped_file + ptr->begin, (caddr_t)share->mapped_file + ptr->end,
-                (size_t)((ptr++)->begin - ptr->end));
+      memmove(share->mapped_file + ptr->begin, share->mapped_file + ptr->end,
+              length - (size_t)ptr->end);
       length= length - (size_t)(ptr->end - ptr->begin);
     }
 
@@ -763,17 +829,21 @@ int ha_tina::rnd_end()
 }
 
 /* 
-  Truncate table and others of its ilk call this. 
+  DELETE without WHERE calls it
 */
 int ha_tina::delete_all_rows()
 {
   DBUG_ENTER("ha_tina::delete_all_rows");
 
+  if (!records_is_known)
+    return (my_errno=HA_ERR_WRONG_COMMAND);
+
   int rc= my_chsize(share->data_file, 0, 0, MYF(MY_WME));
 
   if (get_mmap(share, 0) > 0) 
     DBUG_RETURN(-1);
 
+  records=0;
   DBUG_RETURN(rc);
 }
 
@@ -800,21 +870,6 @@ THR_LOCK_DATA **ha_tina::store_lock(THD *thd,
   return to;
 }
 
-/* 
-  Range optimizer calls this.
-  I need to update the information on this.
-*/
-ha_rows ha_tina::records_in_range(int inx,
-                                  const byte *start_key,uint start_key_len,
-                                  enum ha_rkey_function start_search_flag,
-                                  const byte *end_key,uint end_key_len,
-                                  enum ha_rkey_function end_search_flag)
-{
-  DBUG_ENTER("ha_tina::records_in_range ");
-  DBUG_RETURN(records); // Good guess
-}
-
-
 /* 
   Create a table. You do not want to leave the table open after a call to
   this (the database will call ::open() if it needs to).
