@@ -19,20 +19,25 @@
 
 #include "mysql_priv.h"
 #include <mysys_err.h>
+#include "sp.h"
 #include <my_dir.h>
 #include <m_ctype.h>
 #ifdef __WIN__
 #include <direct.h>
 #endif
 
+#define MAX_DROP_TABLE_Q_LEN      1024
+
 const char *del_exts[]= {".frm", ".BAK", ".TMD",".opt", NullS};
 static TYPELIB deletable_extentions=
 {array_elements(del_exts)-1,"del_exts", del_exts, NULL};
 
 static long mysql_rm_known_files(THD *thd, MY_DIR *dirp,
-				 const char *db, const char *path,
-				 uint level);
-
+				 const char *db, const char *path, uint level, 
+                                 TABLE_LIST **dropped_tables);
+         
+static long mysql_rm_arc_files(THD *thd, MY_DIR *dirp, const char *org_path);
+static my_bool rm_dir_w_symlink(const char *org_path, my_bool send_error);
 /* Database options hash */
 static HASH dboptions;
 static my_bool dboptions_init= 0;
@@ -50,6 +55,7 @@ typedef struct my_dbopt_st
 /*
   Function we use in the creation of our hash to get key.
 */
+
 static byte* dboptions_get_key(my_dbopt_t *opt, uint *length,
                                my_bool not_used __attribute__((unused)))
 {
@@ -58,6 +64,21 @@ static byte* dboptions_get_key(my_dbopt_t *opt, uint *length,
 }
 
 
+/*
+  Helper function to write a query to binlog used by mysql_rm_db()
+*/
+
+static inline void write_to_binlog(THD *thd, char *query, uint q_len,
+                                   char *db, uint db_len)
+{
+  Query_log_event qinfo(thd, query, q_len, 0, 0);
+  qinfo.error_code= 0;
+  qinfo.db= db;
+  qinfo.db_len= db_len;
+  mysql_bin_log.write(&qinfo);
+}  
+
+
 /*
   Function to free dboptions hash element
 */
@@ -266,7 +287,7 @@ static bool write_db_opt(THD *thd, const char *path, HA_CREATE_INFO *create)
 }
 
 
-/* 
+/*
   Load database options file
 
   load_db_opt()
@@ -292,68 +313,72 @@ bool load_db_opt(THD *thd, const char *path, HA_CREATE_INFO *create)
 
   bzero((char*) create,sizeof(*create));
   create->default_table_charset= thd->variables.collation_server;
-  
+
   /* Check if options for this database are already in the hash */
   if (!get_dbopt(path, create))
-    DBUG_RETURN(0);	   
-  
-  /* Otherwise, load options from the .opt file */
-  if ((file=my_open(path, O_RDONLY | O_SHARE, MYF(0))) >= 0)
-  {
-    IO_CACHE cache;
-    init_io_cache(&cache, file, IO_SIZE, READ_CACHE, 0, 0, MYF(0));
+    DBUG_RETURN(0);
 
-    while ((int) (nbytes= my_b_gets(&cache, (char*) buf, sizeof(buf))) > 0)
+  /* Otherwise, load options from the .opt file */
+  if ((file=my_open(path, O_RDONLY | O_SHARE, MYF(0))) < 0)
+    goto err1;
+
+  IO_CACHE cache;
+  if (init_io_cache(&cache, file, IO_SIZE, READ_CACHE, 0, 0, MYF(0)))
+    goto err2;
+
+  while ((int) (nbytes= my_b_gets(&cache, (char*) buf, sizeof(buf))) > 0)
+  {
+    char *pos= buf+nbytes-1;
+    /* Remove end space and control characters */
+    while (pos > buf && !my_isgraph(&my_charset_latin1, pos[-1]))
+      pos--;
+    *pos=0;
+    if ((pos= strchr(buf, '=')))
     {
-      char *pos= buf+nbytes-1;
-      /* Remove end space and control characters */
-      while (pos > buf && !my_isgraph(&my_charset_latin1, pos[-1]))
-	pos--;
-      *pos=0;
-      if ((pos= strchr(buf, '=')))
+      if (!strncmp(buf,"default-character-set", (pos-buf)))
       {
-	if (!strncmp(buf,"default-character-set", (pos-buf)))
-	{
-          /*
-             Try character set name, and if it fails 
-             try collation name, probably it's an old
-             4.1.0 db.opt file, which didn't have
-             separate default-character-set and
-             default-collation commands.
-          */
-	  if (!(create->default_table_charset=
-		get_charset_by_csname(pos+1, MY_CS_PRIMARY, MYF(0))) &&
-              !(create->default_table_charset=
-                get_charset_by_name(pos+1, MYF(0))))
-	  {
-	    sql_print_error("Error while loading database options: '%s':",path);
-	    sql_print_error(ER(ER_UNKNOWN_CHARACTER_SET),pos+1);
-            create->default_table_charset= default_charset_info;
-	  }
-	}
-	else if (!strncmp(buf,"default-collation", (pos-buf)))
-	{
-	  if (!(create->default_table_charset= get_charset_by_name(pos+1,
-								   MYF(0))))
-	  {
-	    sql_print_error("Error while loading database options: '%s':",path);
-	    sql_print_error(ER(ER_UNKNOWN_COLLATION),pos+1);
-            create->default_table_charset= default_charset_info;
-	  }
-	}
+        /*
+           Try character set name, and if it fails
+           try collation name, probably it's an old
+           4.1.0 db.opt file, which didn't have
+           separate default-character-set and
+           default-collation commands.
+        */
+        if (!(create->default_table_charset=
+        get_charset_by_csname(pos+1, MY_CS_PRIMARY, MYF(0))) &&
+            !(create->default_table_charset=
+              get_charset_by_name(pos+1, MYF(0))))
+        {
+          sql_print_error("Error while loading database options: '%s':",path);
+          sql_print_error(ER(ER_UNKNOWN_CHARACTER_SET),pos+1);
+          create->default_table_charset= default_charset_info;
+        }
+      }
+      else if (!strncmp(buf,"default-collation", (pos-buf)))
+      {
+        if (!(create->default_table_charset= get_charset_by_name(pos+1,
+                                                           MYF(0))))
+        {
+          sql_print_error("Error while loading database options: '%s':",path);
+          sql_print_error(ER(ER_UNKNOWN_COLLATION),pos+1);
+          create->default_table_charset= default_charset_info;
+        }
       }
     }
-    end_io_cache(&cache);
-    my_close(file,MYF(0));
-    /*
-      Put the loaded value into the hash.
-      Note that another thread could've added the same
-      entry to the hash after we called get_dbopt(),
-      but it's not an error, as put_dbopt() takes this
-      possibility into account.
-    */
-    error= put_dbopt(path, create);
   }
+  /*
+    Put the loaded value into the hash.
+    Note that another thread could've added the same
+    entry to the hash after we called get_dbopt(),
+    but it's not an error, as put_dbopt() takes this
+    possibility into account.
+  */
+  error= put_dbopt(path, create);
+
+  end_io_cache(&cache);
+err2:
+  my_close(file,MYF(0));
+err1:
   DBUG_RETURN(error);
 }
 
@@ -370,14 +395,20 @@ bool load_db_opt(THD *thd, const char *path, HA_CREATE_INFO *create)
   silent	Used by replication when internally creating a database.
 		In this case the entry should not be logged.
 
+  SIDE-EFFECTS
+   1. Report back to client that command succeeded (send_ok)
+   2. Report errors to client
+   3. Log event to binary log
+   (The 'silent' flags turns off 1 and 3.)
+
   RETURN VALUES
-  0	ok
-  -1	Error
+  FALSE ok
+  TRUE  Error
 
 */
 
-int mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
-		    bool silent)
+bool mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
+                     bool silent)
 {
   char	 path[FN_REFLEN+16];
   long result= 1;
@@ -386,16 +417,34 @@ int mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
   uint create_options= create_info ? create_info->options : 0;
   uint path_len;
   DBUG_ENTER("mysql_create_db");
-  
-  VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
 
-  /* do not create database if another thread is holding read lock */
+  /* do not create 'information_schema' db */
+  if (!my_strcasecmp(system_charset_info, db, information_schema_name.str))
+  {
+    my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
+    DBUG_RETURN(-1);
+  }
+
+  /*
+    Do not create database if another thread is holding read lock.
+    Wait for global read lock before acquiring LOCK_mysql_create_db.
+    After wait_if_global_read_lock() we have protection against another
+    global read lock. If we would acquire LOCK_mysql_create_db first,
+    another thread could step in and get the global read lock before we
+    reach wait_if_global_read_lock(). If this thread tries the same as we
+    (admin a db), it would then go and wait on LOCK_mysql_create_db...
+    Furthermore wait_if_global_read_lock() checks if the current thread
+    has the global read lock and refuses the operation with
+    ER_CANT_UPDATE_WITH_READLOCK if applicable.
+  */
   if (wait_if_global_read_lock(thd, 0, 1))
   {
     error= -1;
     goto exit2;
   }
 
+  VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
+
   /* Check directory */
   strxmov(path, mysql_data_home, "/", db, NullS);
   path_len= unpack_dirname(path,path);    // Convert if not unix
@@ -403,24 +452,29 @@ int mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
 
   if (my_stat(path,&stat_info,MYF(0)))
   {
-   if (!(create_options & HA_LEX_CREATE_IF_NOT_EXISTS))
+    if (!(create_options & HA_LEX_CREATE_IF_NOT_EXISTS))
     {
-      my_error(ER_DB_CREATE_EXISTS,MYF(0),db);
+      my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
       error= -1;
       goto exit;
     }
-    result= 0;
+    push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_NOTE,
+			ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS), db);
+    if (!silent)
+      send_ok(thd);
+    error= 0;
+    goto exit;
   }
   else
   {
     if (my_errno != ENOENT)
     {
-      my_error(EE_STAT, MYF(0),path,my_errno);
+      my_error(EE_STAT, MYF(0), path, my_errno);
       goto exit;
     }
     if (my_mkdir(path,0777,MYF(0)) < 0)
     {
-      my_error(ER_CANT_CREATE_DB,MYF(0),db,my_errno);
+      my_error(ER_CANT_CREATE_DB, MYF(0), db, my_errno);
       error= -1;
       goto exit;
     }
@@ -463,7 +517,6 @@ int mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
       query= 	    thd->query;
       query_length= thd->query_length;
     }
-    mysql_update_log.write(thd, query, query_length);
     if (mysql_bin_log.is_open())
     {
       Query_log_event qinfo(thd, query, query_length, 0, 
@@ -495,28 +548,39 @@ int mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
   }
 
 exit:
+  VOID(pthread_mutex_unlock(&LOCK_mysql_create_db));
   start_waiting_global_read_lock(thd);
 exit2:
-  VOID(pthread_mutex_unlock(&LOCK_mysql_create_db));
   DBUG_RETURN(error);
 }
 
 
 /* db-name is already validated when we come here */
 
-int mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info)
+bool mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info)
 {
   char path[FN_REFLEN+16];
   long result=1;
   int error= 0;
   DBUG_ENTER("mysql_alter_db");
 
-  VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
-
-  /* do not alter database if another thread is holding read lock */
+  /*
+    Do not alter database if another thread is holding read lock.
+    Wait for global read lock before acquiring LOCK_mysql_create_db.
+    After wait_if_global_read_lock() we have protection against another
+    global read lock. If we would acquire LOCK_mysql_create_db first,
+    another thread could step in and get the global read lock before we
+    reach wait_if_global_read_lock(). If this thread tries the same as we
+    (admin a db), it would then go and wait on LOCK_mysql_create_db...
+    Furthermore wait_if_global_read_lock() checks if the current thread
+    has the global read lock and refuses the operation with
+    ER_CANT_UPDATE_WITH_READLOCK if applicable.
+  */
   if ((error=wait_if_global_read_lock(thd,0,1)))
     goto exit2;
 
+  VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
+
   /* Check directory */
   strxmov(path, mysql_data_home, "/", db, "/", MY_DB_OPT_FILE, NullS);
   fn_format(path, path, "", "", MYF(MY_UNPACK_FILENAME));
@@ -535,7 +599,6 @@ int mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info)
     thd->variables.collation_database= thd->db_charset;
   }
 
-  mysql_update_log.write(thd,thd->query, thd->query_length);
   if (mysql_bin_log.is_open())
   {
     Query_log_event qinfo(thd, thd->query, thd->query_length, 0,
@@ -555,10 +618,10 @@ int mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info)
   send_ok(thd, result);
 
 exit:
+  VOID(pthread_mutex_unlock(&LOCK_mysql_create_db));
   start_waiting_global_read_lock(thd);
 exit2:
-  VOID(pthread_mutex_unlock(&LOCK_mysql_create_db));
-  DBUG_RETURN(error ? -1 : 0); /* -1 to delegate send_error() */
+  DBUG_RETURN(error);
 }
 
 
@@ -569,33 +632,46 @@ exit2:
     mysql_rm_db()
     thd			Thread handle
     db			Database name in the case given by user
-		        It's already validated when we come here
+		        It's already validated and set to lower case
+                        (if needed) when we come here
     if_exists		Don't give error if database doesn't exists
     silent		Don't generate errors
 
   RETURN
-    0   ok (Database dropped)
-    -1	Error generated
+    FALSE ok (Database dropped)
+    ERROR Error
 */
 
-int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
+bool mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
 {
   long deleted=0;
   int error= 0;
-  char	path[FN_REFLEN+16], tmp_db[NAME_LEN+1];
+  char	path[FN_REFLEN+16];
   MY_DIR *dirp;
   uint length;
+  TABLE_LIST* dropped_tables= 0;
   DBUG_ENTER("mysql_rm_db");
 
-  VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
-
-  /* do not drop database if another thread is holding read lock */
+  /*
+    Do not drop database if another thread is holding read lock.
+    Wait for global read lock before acquiring LOCK_mysql_create_db.
+    After wait_if_global_read_lock() we have protection against another
+    global read lock. If we would acquire LOCK_mysql_create_db first,
+    another thread could step in and get the global read lock before we
+    reach wait_if_global_read_lock(). If this thread tries the same as we
+    (admin a db), it would then go and wait on LOCK_mysql_create_db...
+    Furthermore wait_if_global_read_lock() checks if the current thread
+    has the global read lock and refuses the operation with
+    ER_CANT_UPDATE_WITH_READLOCK if applicable.
+  */
   if (wait_if_global_read_lock(thd, 0, 1))
   {
     error= -1;
     goto exit2;
   }
 
+  VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
+
   (void) sprintf(path,"%s/%s",mysql_data_home,db);
   length= unpack_dirname(path,path);		// Convert if not unix
   strmov(path+length, MY_DB_OPT_FILE);		// Append db option file name
@@ -608,7 +684,7 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
     if (!if_exists)
     {
       error= -1;
-      my_error(ER_DB_DROP_EXISTS,MYF(0),db);
+      my_error(ER_DB_DROP_EXISTS, MYF(0), db);
       goto exit;
     }
     else
@@ -621,21 +697,16 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
     remove_db_from_cache(db);
     pthread_mutex_unlock(&LOCK_open);
 
+    
     error= -1;
-    if ((deleted= mysql_rm_known_files(thd, dirp, db, path, 0)) >= 0)
+    if ((deleted= mysql_rm_known_files(thd, dirp, db, path, 0,
+                                       &dropped_tables)) >= 0)
     {
       ha_drop_database(path);
       query_cache_invalidate1(db);
       error = 0;
     }
   }
-  if (lower_case_table_names)
-  {
-    /* Convert database to lower case */
-    strmov(tmp_db, db);
-    my_casedn_str(files_charset_info, tmp_db);
-    db= tmp_db;
-  }
   if (!silent && deleted>=0)
   {
     const char *query;
@@ -652,7 +723,6 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
       query =thd->query;
       query_length= thd->query_length;
     }
-    mysql_update_log.write(thd, query, query_length);
     if (mysql_bin_log.is_open())
     {
       Query_log_event qinfo(thd, query, query_length, 0, 
@@ -672,40 +742,55 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
     send_ok(thd, (ulong) deleted);
     thd->server_status&= ~SERVER_STATUS_DB_DROPPED;
   }
+  else if (mysql_bin_log.is_open())
+  {
+    char *query, *query_pos, *query_end, *query_data_start;
+    TABLE_LIST *tbl;
+    uint db_len;
+
+    if (!(query= thd->alloc(MAX_DROP_TABLE_Q_LEN)))
+      goto exit; /* not much else we can do */
+    query_pos= query_data_start= strmov(query,"drop table ");
+    query_end= query + MAX_DROP_TABLE_Q_LEN;
+    db_len= strlen(db);
+
+    for (tbl= dropped_tables; tbl; tbl= tbl->next_local)
+    {
+      uint tbl_name_len;
+
+      /* 3 for the quotes and the comma*/
+      tbl_name_len= strlen(tbl->table_name) + 3;
+      if (query_pos + tbl_name_len + 1 >= query_end)
+      {
+        write_to_binlog(thd, query, query_pos -1 - query, db, db_len);
+        query_pos= query_data_start;
+      }
+
+      *query_pos++ = '`';
+      query_pos= strmov(query_pos,tbl->table_name);
+      *query_pos++ = '`';
+      *query_pos++ = ',';
+    }
+
+    if (query_pos != query_data_start)
+    {
+      write_to_binlog(thd, query, query_pos -1 - query, db, db_len);
+    }
+  }
 
 exit:
-  start_waiting_global_read_lock(thd);
+  (void)sp_drop_db_routines(thd, db); /* QQ Ignore errors for now  */
   /*
-    If this database was the client's selected database, we silently change the
-    client's selected database to nothing (to have an empty SELECT DATABASE()
-    in the future). For this we free() thd->db and set it to 0. But we don't do
-    free() for the slave thread. Indeed, doing a x_free() on it leads to nasty
-    problems (i.e. long painful debugging) because in this thread, thd->db is
-    the same as data_buf and db of the Query_log_event which is dropping the
-    database. So if you free() thd->db, you're freeing data_buf. You set
-    thd->db to 0 but not data_buf (thd->db and data_buf are two distinct
-    pointers which point to the same place). Then in ~Query_log_event(), we
-    have 'if (data_buf) free(data_buf)' data_buf is !=0 so this makes a
-    DOUBLE free().
-    Side effects of this double free() are, randomly (depends on the machine),
-    when the slave is replicating a DROP DATABASE:
-    - garbage characters in the error message:
-    "Error 'Can't drop database 'test2'; database doesn't exist' on query
-    'h4zI'"
-    - segfault
-    - hang in "free(vio)" (yes!) in the I/O or SQL slave threads (so slave
-    server hangs at shutdown etc).
+    If this database was the client's selected database, we silently
+    change the client's selected database to nothing (to have an empty
+    SELECT DATABASE() in the future). For this we free() thd->db and set
+    it to 0.
   */
   if (thd->db && !strcmp(thd->db, db))
-  {
-    if (!(thd->slave_thread)) /* a slave thread will free it itself */
-      x_free(thd->db);
-    thd->db= 0;
-    thd->db_length= 0;
-  }
-exit2:
+    thd->set_db(NULL, 0);
   VOID(pthread_mutex_unlock(&LOCK_mysql_create_db));
-
+  start_waiting_global_read_lock(thd);
+exit2:
   DBUG_RETURN(error);
 }
 
@@ -716,7 +801,8 @@ exit2:
 */
 
 static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
-				 const char *org_path, uint level)
+				 const char *org_path, uint level,
+                                 TABLE_LIST **dropped_tables)
 {
   long deleted=0;
   ulong found_other_files=0;
@@ -742,9 +828,9 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
       continue;
 
     /* Check if file is a raid directory */
-    if ((my_isdigit(&my_charset_latin1, file->name[0]) ||
+    if ((my_isdigit(system_charset_info, file->name[0]) ||
 	 (file->name[0] >= 'a' && file->name[0] <= 'f')) &&
-	(my_isdigit(&my_charset_latin1, file->name[1]) ||
+	(my_isdigit(system_charset_info, file->name[1]) ||
 	 (file->name[1] >= 'a' && file->name[1] <= 'f')) &&
 	!file->name[2] && !level)
     {
@@ -758,7 +844,7 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
       if ((new_dirp = my_dir(newpath,MYF(MY_DONT_SORT))))
       {
 	DBUG_PRINT("my",("New subdir found: %s", newpath));
-	if ((mysql_rm_known_files(thd, new_dirp, NullS, newpath,1)) < 0)
+	if ((mysql_rm_known_files(thd, new_dirp, NullS, newpath,1,0)) < 0)
 	  goto err;
 	if (!(copy_of_path= thd->memdup(newpath, length+1)) ||
 	    !(dir= new (thd->mem_root) String(copy_of_path, length,
@@ -770,6 +856,24 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
       found_other_files++;
       continue;
     }
+    else if (file->name[0] == 'a' && file->name[1] == 'r' &&
+             file->name[2] == 'c' && file->name[3] == '\0')
+    {
+      /* .frm archive */
+      char newpath[FN_REFLEN];
+      MY_DIR *new_dirp;
+      strxmov(newpath, org_path, "/", "arc", NullS);
+      (void) unpack_filename(newpath, newpath);
+      if ((new_dirp = my_dir(newpath, MYF(MY_DONT_SORT))))
+      {
+	DBUG_PRINT("my",("Archive subdir found: %s", newpath));
+	if ((mysql_rm_arc_files(thd, new_dirp, newpath)) < 0)
+	  goto err;
+	continue;
+      }
+      found_other_files++;
+      continue;
+    }
     extension= fn_ext(file->name);
     if (find_type(extension, &deletable_extentions,1+2) <= 0)
     {
@@ -788,11 +892,11 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
       if (!table_list)
 	goto err;
       table_list->db= (char*) (table_list+1);
-      strmov(table_list->real_name= strmov(table_list->db,db)+1, file->name);
-      table_list->alias= table_list->real_name;	// If lower_case_table_names=2
+      strmov(table_list->table_name= strmov(table_list->db,db)+1, file->name);
+      table_list->alias= table_list->table_name;	// If lower_case_table_names=2
       /* Link into list */
       (*tot_list_next)= table_list;
-      tot_list_next= &table_list->next;
+      tot_list_next= &table_list->next_local;
       deleted++;
     }
     else
@@ -818,6 +922,9 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
   }
   my_dirend(dirp);  
   
+  if (dropped_tables)
+    *dropped_tables= tot_list;
+  
   /*
     If the directory is a symbolic link, remove the link first, then
     remove the directory the symbolic link pointed at
@@ -829,44 +936,11 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
   }
   else
   {
-    char tmp_path[FN_REFLEN], *pos;
-    char *path= tmp_path;
-    unpack_filename(tmp_path,org_path);
-#ifdef HAVE_READLINK
-    int error;
-    
-    /* Remove end FN_LIBCHAR as this causes problem on Linux in readlink */
-    pos=strend(path);
-    if (pos > path && pos[-1] == FN_LIBCHAR)
-      *--pos=0;
-
-    if ((error=my_readlink(filePath, path, MYF(MY_WME))) < 0)
-      DBUG_RETURN(-1);
-    if (!error)
-    {
-      if (my_delete(path,MYF(!level ? MY_WME : 0)))
-      {
-	/* Don't give errors if we can't delete 'RAID' directory */
-	if (level)
-	  DBUG_RETURN(deleted);
-	DBUG_RETURN(-1);
-      }
-      /* Delete directory symbolic link pointed at */
-      path= filePath;
-    }
-#endif
-    /* Remove last FN_LIBCHAR to not cause a problem on OS/2 */
-    pos=strend(path);
-
-    if (pos > path && pos[-1] == FN_LIBCHAR)
-      *--pos=0;
     /* Don't give errors if we can't delete 'RAID' directory */
-    if (rmdir(path) < 0 && !level)
-    {
-      my_error(ER_DB_DROP_RMDIR, MYF(0), path, errno);
+    if (rm_dir_w_symlink(org_path, level == 0))
       DBUG_RETURN(-1);
-    }
   }
+
   DBUG_RETURN(deleted);
 
 err:
@@ -876,94 +950,283 @@ err:
 
 
 /*
-  Change default database.
+  Remove directory with symlink
+
+  SYNOPSIS
+    rm_dir_w_symlink()
+    org_path    path of derictory
+    send_error  send errors
+  RETURN
+    0 OK
+    1 ERROR
+*/
+
+static my_bool rm_dir_w_symlink(const char *org_path, my_bool send_error)
+{
+  char tmp_path[FN_REFLEN], *pos;
+  char *path= tmp_path;
+  DBUG_ENTER("rm_dir_w_symlink");
+  unpack_filename(tmp_path, org_path);
+#ifdef HAVE_READLINK
+  int error;
+  char tmp2_path[FN_REFLEN];
+
+  /* Remove end FN_LIBCHAR as this causes problem on Linux in readlink */
+  pos= strend(path);
+  if (pos > path && pos[-1] == FN_LIBCHAR)
+    *--pos=0;
+
+  if ((error= my_readlink(tmp2_path, path, MYF(MY_WME))) < 0)
+    DBUG_RETURN(1);
+  if (!error)
+  {
+    if (my_delete(path, MYF(send_error ? MY_WME : 0)))
+    {
+      DBUG_RETURN(send_error);
+    }
+    /* Delete directory symbolic link pointed at */
+    path= tmp2_path;
+  }
+#endif
+  /* Remove last FN_LIBCHAR to not cause a problem on OS/2 */
+  pos= strend(path);
+
+  if (pos > path && pos[-1] == FN_LIBCHAR)
+    *--pos=0;
+  if (rmdir(path) < 0 && send_error)
+  {
+    my_error(ER_DB_DROP_RMDIR, MYF(0), path, errno);
+    DBUG_RETURN(1);
+  }
+  DBUG_RETURN(0);
+}
+
+
+/*
+  Remove .frm archives from directory
+
+  SYNOPSIS
+    thd       thread handler
+    dirp      list of files in archive directory
+    db        data base name
+    org_path  path of archive directory
+
+  RETURN
+    > 0 number of removed files
+    -1  error
+*/
+static long mysql_rm_arc_files(THD *thd, MY_DIR *dirp,
+				 const char *org_path)
+{
+  long deleted= 0;
+  ulong found_other_files= 0;
+  char filePath[FN_REFLEN];
+  DBUG_ENTER("mysql_rm_arc_files");
+  DBUG_PRINT("enter", ("path: %s", org_path));
+
+  for (uint idx=0 ;
+       idx < (uint) dirp->number_off_files && !thd->killed ;
+       idx++)
+  {
+    FILEINFO *file=dirp->dir_entry+idx;
+    char *extension, *revision;
+    DBUG_PRINT("info",("Examining: %s", file->name));
+
+    /* skiping . and .. */
+    if (file->name[0] == '.' && (!file->name[1] ||
+       (file->name[1] == '.' &&  !file->name[2])))
+      continue;
+
+    extension= fn_ext(file->name);
+    if (extension[0] != '.' ||
+        extension[1] != 'f' || extension[2] != 'r' ||
+        extension[3] != 'm' || extension[4] != '-')
+    {
+      found_other_files++;
+      continue;
+    }
+    revision= extension+5;
+    while (*revision && my_isdigit(system_charset_info, *revision))
+      revision++;
+    if (*revision)
+    {
+      found_other_files++;
+      continue;
+    }
+    strxmov(filePath, org_path, "/", file->name, NullS);
+    if (my_delete_with_symlink(filePath,MYF(MY_WME)))
+    {
+      goto err;
+    }
+  }
+  if (thd->killed)
+    goto err;
+
+  my_dirend(dirp);
+
+  /*
+    If the directory is a symbolic link, remove the link first, then
+    remove the directory the symbolic link pointed at
+  */
+  if (!found_other_files &&
+      rm_dir_w_symlink(org_path, 0))
+    DBUG_RETURN(-1);
+  DBUG_RETURN(deleted);
+
+err:
+  my_dirend(dirp);
+  DBUG_RETURN(-1);
+}
+
+
+/*
+  Change the current database.
 
   SYNOPSIS
     mysql_change_db()
-    thd		Thread handler
-    name	Databasename
+    thd			thread handle
+    name		database name
+    no_access_check	if TRUE, don't do access check. In this
+                        case name may be ""
 
   DESCRIPTION
-    Becasue the database name may have been given directly from the
-    communication packet (in case of 'connect' or 'COM_INIT_DB')
-    we have to do end space removal in this function.
+    Check that the database name corresponds to a valid and
+    existent database, check access rights (unless called with
+    no_access_check), and set the current database. This function
+    is called to change the current database upon user request
+    (COM_CHANGE_DB command) or temporarily, to execute a stored
+    routine.
 
   NOTES
-    Do as little as possible in this function, as it is not called for the
-    replication slave SQL thread (for that thread, setting of thd->db is done
-    in ::exec_event() methods of log_event.cc).
+    This function is not the only way to switch the database that
+    is currently employed. When the replication slave thread
+    switches the database before executing a query, it calls
+    thd->set_db directly. However, if the query, in turn, uses
+    a stored routine, the stored routine will use this function,
+    even if it's run on the slave.
+
+    This function allocates the name of the database on the system
+    heap: this is necessary to be able to uniformly change the
+    database from any module of the server. Up to 5.0 different
+    modules were using different memory to store the name of the
+    database, and this led to memory corruption: a stack pointer
+    set by Stored Procedures was used by replication after the
+    stack address was long gone.
+
+    This function does not send anything, including error
+    messages, to the client. If that should be sent to the client,
+    call net_send_error after this function.
 
   RETURN VALUES
-    0	ok
+    0	OK
     1	error
 */
 
-bool mysql_change_db(THD *thd, const char *name)
+bool mysql_change_db(THD *thd, const char *name, bool no_access_check)
 {
-  int length, db_length;
-  char *dbname=my_strdup((char*) name,MYF(MY_WME));
+  int path_length, db_length;
+  char *db_name;
   char	path[FN_REFLEN];
   HA_CREATE_INFO create;
+  bool system_db= 0;
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   ulong db_access;
+  Security_context *sctx= thd->security_ctx;
+  LINT_INIT(db_access);
 #endif
   DBUG_ENTER("mysql_change_db");
+  DBUG_PRINT("enter",("name: '%s'",name));
 
-  if (!dbname || !(db_length= strlen(dbname)))
+  if (name == NULL || name[0] == '\0' && no_access_check == FALSE)
   {
-    x_free(dbname);				/* purecov: inspected */
-    send_error(thd,ER_NO_DB_ERROR);             /* purecov: inspected */
+    my_message(ER_NO_DB_ERROR, ER(ER_NO_DB_ERROR), MYF(0));
     DBUG_RETURN(1);				/* purecov: inspected */
   }
-  if (check_db_name(dbname))
+  else if (name[0] == '\0')
   {
-    net_printf(thd, ER_WRONG_DB_NAME, dbname);
-    x_free(dbname);
+    /* Called from SP to restore the original database, which was NULL */
+    DBUG_ASSERT(no_access_check);
+    system_db= 1;
+    db_name= NULL;
+    db_length= 0;
+    goto end;
+  }
+  /*
+    Now we need to make a copy because check_db_name requires a
+    non-constant argument. TODO: fix check_db_name.
+  */
+  if ((db_name= my_strdup(name, MYF(MY_WME))) == NULL)
+    DBUG_RETURN(1);                             /* the error is set */
+  db_length= strlen(db_name);
+  if (check_db_name(db_name))
+  {
+    my_error(ER_WRONG_DB_NAME, MYF(0), db_name);
+    my_free(db_name, MYF(0));
     DBUG_RETURN(1);
   }
-  DBUG_PRINT("info",("Use database: %s", dbname));
-#ifndef NO_EMBEDDED_ACCESS_CHECKS
-  if (test_all_bits(thd->master_access,DB_ACLS))
-    db_access=DB_ACLS;
-  else
-    db_access= (acl_get(thd->host,thd->ip, thd->priv_user,dbname,0) |
-		thd->master_access);
-  if (!(db_access & DB_ACLS) && (!grant_option || check_grant_db(thd,dbname)))
+  DBUG_PRINT("info",("Use database: %s", db_name));
+  if (!my_strcasecmp(system_charset_info, db_name, information_schema_name.str))
   {
-    net_printf(thd,ER_DBACCESS_DENIED_ERROR,
-	       thd->priv_user,
-	       thd->priv_host,
-	       dbname);
-    mysql_log.write(thd,COM_INIT_DB,ER(ER_DBACCESS_DENIED_ERROR),
-		    thd->priv_user,
-		    thd->priv_host,
-		    dbname);
-    my_free(dbname,MYF(0));
-    DBUG_RETURN(1);
+    system_db= 1;
+#ifndef NO_EMBEDDED_ACCESS_CHECKS
+    db_access= SELECT_ACL;
+#endif
+    goto end;
+  }
+
+#ifndef NO_EMBEDDED_ACCESS_CHECKS
+  if (!no_access_check)
+  {
+    if (test_all_bits(sctx->master_access, DB_ACLS))
+      db_access=DB_ACLS;
+    else
+      db_access= (acl_get(sctx->host, sctx->ip, sctx->priv_user, db_name, 0) |
+                  sctx->master_access);
+    if (!(db_access & DB_ACLS) && (!grant_option ||
+                                   check_grant_db(thd,db_name)))
+    {
+      my_error(ER_DBACCESS_DENIED_ERROR, MYF(0),
+               sctx->priv_user,
+               sctx->priv_host,
+               db_name);
+      mysql_log.write(thd, COM_INIT_DB, ER(ER_DBACCESS_DENIED_ERROR),
+                      sctx->priv_user, sctx->priv_host, db_name);
+      my_free(db_name, MYF(0));
+      DBUG_RETURN(1);
+    }
   }
 #endif
-  (void) sprintf(path,"%s/%s",mysql_data_home,dbname);
-  length=unpack_dirname(path,path);		// Convert if not unix
-  if (length && path[length-1] == FN_LIBCHAR)
-    path[length-1]=0;				// remove ending '\'
+  (void) sprintf(path,"%s/%s", mysql_data_home, db_name);
+  path_length= unpack_dirname(path, path);      // Convert if not UNIX
+  if (path_length && path[path_length-1] == FN_LIBCHAR)
+    path[path_length-1]= '\0';                  // remove ending '\'
   if (my_access(path,F_OK))
   {
-    net_printf(thd,ER_BAD_DB_ERROR,dbname);
-    my_free(dbname,MYF(0));
+    my_error(ER_BAD_DB_ERROR, MYF(0), db_name);
+    my_free(db_name, MYF(0));
     DBUG_RETURN(1);
   }
-  send_ok(thd);
+end:
   x_free(thd->db);
-  thd->db=dbname;				// THD::~THD will free this
-  thd->db_length=db_length;
+  DBUG_ASSERT(db_name == NULL || db_name[0] != '\0');
+  thd->reset_db(db_name, db_length);            // THD::~THD will free this
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
-  thd->db_access=db_access;
+  if (!no_access_check)
+    sctx->db_access= db_access;
 #endif
-  strmov(path+unpack_dirname(path,path), MY_DB_OPT_FILE);
-  load_db_opt(thd, path, &create);
-  thd->db_charset= create.default_table_charset ?
-		   create.default_table_charset :
-		   thd->variables.collation_server;
-  thd->variables.collation_database= thd->db_charset;
+  if (system_db)
+  {
+    thd->db_charset= system_charset_info;
+    thd->variables.collation_database= system_charset_info;
+  }
+  else
+  {
+    strmov(path+unpack_dirname(path,path), MY_DB_OPT_FILE);
+    load_db_opt(thd, path, &create);
+    thd->db_charset= create.default_table_charset ?
+      create.default_table_charset :
+      thd->variables.collation_server;
+    thd->variables.collation_database= thd->db_charset;
+  }
   DBUG_RETURN(0);
 }
