@@ -149,7 +149,9 @@ static int write_dynamic_record(MI_INFO *info, const byte *record,
   {
     if (_mi_find_writepos(info,reclength,&filepos,&length))
       goto err;
-    if (_mi_write_part_record(info,filepos,length,info->s->state.dellink,
+    if (_mi_write_part_record(info,filepos,length,
+                              (info->append_insert_at_end ?
+                               HA_OFFSET_ERROR : info->s->state.dellink),
 			      (byte**) &record,&reclength,&flag))
       goto err;
   } while (reclength);
@@ -171,7 +173,8 @@ static int _mi_find_writepos(MI_INFO *info,
   ulong tmp;
   DBUG_ENTER("_mi_find_writepos");
 
-  if (info->s->state.dellink != HA_OFFSET_ERROR)
+  if (info->s->state.dellink != HA_OFFSET_ERROR &&
+      !info->append_insert_at_end)
   {
     /* Deleted blocks exists;  Get last used block */
     *filepos=info->s->state.dellink;
@@ -420,8 +423,9 @@ int _mi_write_part_record(MI_INFO *info,
   else if (length-long_block < *reclength+4)
   {						/* To short block */
     if (next_filepos == HA_OFFSET_ERROR)
-      next_filepos=info->s->state.dellink != HA_OFFSET_ERROR ?
-	info->s->state.dellink : info->state->data_file_length;
+      next_filepos= (info->s->state.dellink != HA_OFFSET_ERROR &&
+                     !info->append_insert_at_end ?
+                     info->s->state.dellink : info->state->data_file_length);
     if (*flag == 0)				/* First block */
     {
       if (*reclength > MI_MAX_BLOCK_LENGTH)
@@ -768,11 +772,21 @@ uint _mi_rec_pack(MI_INFO *info, register byte *to, register const byte *from)
       }
       else if (type == FIELD_VARCHAR)
       {
-	uint tmp_length=uint2korr(from);
-	store_key_length_inc(to,tmp_length);
-	memcpy(to,from+2,tmp_length);
-	to+=tmp_length;
-	continue;
+        uint pack_length= HA_VARCHAR_PACKLENGTH(rec->length -1);
+	uint tmp_length;
+        if (pack_length == 1)
+        {
+          tmp_length= (uint) *(uchar*) from;
+          *to++= *from;
+        }
+        else
+        {
+          tmp_length= uint2korr(from);
+          store_key_length_inc(to,tmp_length);
+        }
+        memcpy(to, from+pack_length,tmp_length);
+        to+= tmp_length;
+        continue;
       }
       else
       {
@@ -878,9 +892,20 @@ my_bool _mi_rec_check(MI_INFO *info,const char *record, byte *rec_buff,
       }
       else if (type == FIELD_VARCHAR)
       {
-	uint tmp_length=uint2korr(record);
-	to+=get_pack_length(tmp_length)+tmp_length;
-	continue;
+        uint pack_length= HA_VARCHAR_PACKLENGTH(rec->length -1);
+	uint tmp_length;
+        if (pack_length == 1)
+        {
+          tmp_length= (uint) *(uchar*) record;
+          to+= 1+ tmp_length;
+          continue;
+        }
+        else
+        {
+          tmp_length= uint2korr(record);
+          to+= get_pack_length(tmp_length)+tmp_length;
+        }
+        continue;
       }
       else
       {
@@ -894,9 +919,7 @@ my_bool _mi_rec_check(MI_INFO *info,const char *record, byte *rec_buff,
       }
     }
     else
-    {
-      to+=length;
-    }
+      to+= length;
   }
   if (packed_length != (uint) (to - rec_buff) + test(info->s->calc_checksum) ||
       (bit != 1 && (flag & ~(bit - 1))))
@@ -944,13 +967,27 @@ ulong _mi_rec_unpack(register MI_INFO *info, register byte *to, byte *from,
     {
       if (type == FIELD_VARCHAR)
       {
-	get_key_length(length,from);
-	if (length > rec_length-2)
-	  goto err;
-	int2store(to,length);
-	memcpy(to+2,from,length);
-	from+=length;
-	continue;
+        uint pack_length= HA_VARCHAR_PACKLENGTH(rec_length-1);
+        if (pack_length == 1)
+        {
+          length= (uint) *(uchar*) from;
+          if (length > rec_length-1)
+            goto err;
+          *to= *from++;
+        }
+        else
+        {
+          get_key_length(length, from);
+          if (length > rec_length-2)
+            goto err;
+          int2store(to,length);
+        }
+        if (from+length > from_end)
+          goto err;
+        memcpy(to+pack_length, from, length);
+        from+= length;
+        min_pack_length--;
+        continue;
       }
       if (flag & bit)
       {
@@ -1018,15 +1055,17 @@ ulong _mi_rec_unpack(register MI_INFO *info, register byte *to, byte *from,
       if (min_pack_length > (uint) (from_end - from))
 	goto err;
       min_pack_length-=rec_length;
-      memcpy(to,(byte*) from,(size_t) rec_length); from+=rec_length;
+      memcpy(to, (byte*) from, (size_t) rec_length);
+      from+=rec_length;
     }
   }
   if (info->s->calc_checksum)
     from++;
   if (to == to_end && from == from_end && (bit == 1 || !(flag & ~(bit-1))))
     DBUG_RETURN(found_length);
+
 err:
-  my_errno=HA_ERR_RECORD_DELETED;
+  my_errno= HA_ERR_WRONG_IN_RECORD;
   DBUG_PRINT("error",("to_end: %lx -> %lx  from_end: %lx -> %lx",
 		      to,to_end,from,from_end));
   DBUG_DUMP("from",(byte*) info->rec_buff,info->s->base.min_pack_length);
@@ -1116,9 +1155,6 @@ int _mi_read_dynamic_record(MI_INFO *info, my_off_t filepos, byte *buf)
 	  info->rec_cache.pos_in_file <= block_info.next_filepos &&
 	  flush_io_cache(&info->rec_cache))
 	goto err;
-      /* A corrupted table can have wrong pointers. (Bug# 19835) */
-      if (block_info.next_filepos == HA_OFFSET_ERROR)
-        goto panic;
       info->rec_cache.seek_not_done=1;
       if ((b_type=_mi_get_block_info(&block_info,file,
 				     block_info.next_filepos))
