@@ -18,33 +18,65 @@
 
 #include "myisamdef.h"
 #include "m_ctype.h"
-#include <assert.h>
+#include "sp_defs.h"
 #ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
 #endif
 
-#define CHECK_KEYS
+#define CHECK_KEYS                              /* Enable safety checks */
+
+#define FIX_LENGTH(cs, pos, length, char_length)                            \
+            do {                                                            \
+              if (length > char_length)                                     \
+                char_length= my_charpos(cs, pos, pos+length, char_length);  \
+              set_if_smaller(char_length,length);                           \
+            } while(0)
 
 static int _mi_put_key_in_record(MI_INFO *info,uint keynr,byte *record);
 
-	/*
-	** Make a intern key from a record
-	** Ret: Length of key
-	*/
+/*
+  Make a intern key from a record
+
+  SYNOPSIS
+    _mi_make_key()
+    info		MyiSAM handler
+    keynr		key number
+    key			Store created key here
+    record		Record
+    filepos		Position to record in the data file
+
+  RETURN
+    Length of key
+*/
 
 uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
 		  const byte *record, my_off_t filepos)
 {
   byte *pos,*end;
   uchar *start;
-  reg1 MI_KEYSEG *keyseg;
+  reg1 HA_KEYSEG *keyseg;
+  my_bool is_ft= info->s->keyinfo[keynr].flag & HA_FULLTEXT;
   DBUG_ENTER("_mi_make_key");
 
+  if (info->s->keyinfo[keynr].flag & HA_SPATIAL)
+  {
+    /*
+      TODO: nulls processing
+    */
+#ifdef HAVE_SPATIAL
+    return sp_make_key(info,keynr,key,record,filepos);
+#else
+    DBUG_ASSERT(0); /* mi_open should check that this never happens*/
+#endif
+  }
+
   start=key;
   for (keyseg=info->s->keyinfo[keynr].seg ; keyseg->type ;keyseg++)
   {
     enum ha_base_keytype type=(enum ha_base_keytype) keyseg->type;
     uint length=keyseg->length;
+    uint char_length;
+    CHARSET_INFO *cs=keyseg->charset;
 
     if (keyseg->null_bit)
     {
@@ -56,10 +88,26 @@ uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
       *key++=1;					/* Not NULL */
     }
 
+    char_length= ((!is_ft && cs && cs->mbmaxlen > 1) ? length/cs->mbmaxlen :
+                  length);
+
     pos= (byte*) record+keyseg->start;
+    if (type == HA_KEYTYPE_BIT)
+    {
+      if (keyseg->bit_length)
+      {
+        uchar bits= get_rec_bits((uchar*) record + keyseg->bit_pos,
+                                 keyseg->bit_start, keyseg->bit_length);
+        *key++= bits;
+        length--;
+      }
+      memcpy((byte*) key, pos, length);
+      key+= length;
+      continue;
+    }
     if (keyseg->flag & HA_SPACE_PACK)
     {
-      end=pos+length;
+      end= pos + length;
       if (type != HA_KEYTYPE_NUM)
       {
 	while (end > pos && end[-1] == ' ')
@@ -71,24 +119,35 @@ uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
 	  pos++;
       }
       length=(uint) (end-pos);
-      store_key_length_inc(key,length);
-      memcpy((byte*) key,(byte*) pos,(size_t) length);
-      key+=length;
+      FIX_LENGTH(cs, pos, length, char_length);
+      store_key_length_inc(key,char_length);
+      memcpy((byte*) key,(byte*) pos,(size_t) char_length);
+      key+=char_length;
       continue;
     }
-    if (keyseg->flag & HA_VAR_LENGTH)
+    if (keyseg->flag & HA_VAR_LENGTH_PART)
     {
-      uint tmp_length=uint2korr(pos);
-      pos+=2;					/* Skip VARCHAR length */
+      uint pack_length= (keyseg->bit_start == 1 ? 1 : 2);
+      uint tmp_length= (pack_length == 1 ? (uint) *(uchar*) pos :
+                        uint2korr(pos));
+      pos+= pack_length;			/* Skip VARCHAR length */
       set_if_smaller(length,tmp_length);
-      store_key_length_inc(key,length);
+      FIX_LENGTH(cs, pos, length, char_length);
+      store_key_length_inc(key,char_length);
+      memcpy((byte*) key,(byte*) pos,(size_t) char_length);
+      key+= char_length;
+      continue;
     }
     else if (keyseg->flag & HA_BLOB_PART)
     {
       uint tmp_length=_mi_calc_blob_length(keyseg->bit_start,pos);
       memcpy_fixed((byte*) &pos,pos+keyseg->bit_start,sizeof(char*));
       set_if_smaller(length,tmp_length);
-      store_key_length_inc(key,length);
+      FIX_LENGTH(cs, pos, length, char_length);
+      store_key_length_inc(key,char_length);
+      memcpy((byte*) key,(byte*) pos,(size_t) char_length);
+      key+= char_length;
+      continue;
     }
     else if (keyseg->flag & HA_SWAP_KEY)
     {						/* Numerical column */
@@ -100,7 +159,7 @@ uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
 	if (isnan(nr))
 	{
 	  /* Replace NAN with zero */
- 	  bzero(key,length);
+	  bzero(key,length);
 	  key+=length;
 	  continue;
 	}
@@ -111,7 +170,7 @@ uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
 	float8get(nr,pos);
 	if (isnan(nr))
 	{
- 	  bzero(key,length);
+	  bzero(key,length);
 	  key+=length;
 	  continue;
 	}
@@ -124,7 +183,10 @@ uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
       }
       continue;
     }
-    memcpy((byte*) key, pos, length);
+    FIX_LENGTH(cs, pos, length, char_length);
+    memcpy((byte*) key, pos, char_length);
+    if (length > char_length)
+      cs->cset->fill(cs, (char*) key+char_length, length-char_length, ' ');
     key+= length;
   }
   _mi_dpointer(info,key,filepos);
@@ -152,40 +214,46 @@ uint _mi_make_key(register MI_INFO *info, uint keynr, uchar *key,
    RETURN
      length of packed key
 
-     last_use_keyseg 	Store pointer to the keyseg after the last used one
+     last_use_keyseg    Store pointer to the keyseg after the last used one
 */
 
 uint _mi_pack_key(register MI_INFO *info, uint keynr, uchar *key, uchar *old,
-		  uint k_length, MI_KEYSEG **last_used_keyseg)
+		  uint k_length, HA_KEYSEG **last_used_keyseg)
 {
-  uint length;
-  uchar *pos,*end,*start_key=key;
-  reg1 MI_KEYSEG *keyseg;
-  enum ha_base_keytype type;
+  uchar *start_key=key;
+  HA_KEYSEG *keyseg;
+  my_bool is_ft= info->s->keyinfo[keynr].flag & HA_FULLTEXT;
   DBUG_ENTER("_mi_pack_key");
 
-  start_key=key;
   for (keyseg=info->s->keyinfo[keynr].seg ;
        keyseg->type && (int) k_length > 0;
        old+=keyseg->length, keyseg++)
   {
-    length=min((uint) keyseg->length,(uint) k_length);
-    type=(enum ha_base_keytype) keyseg->type;
+    enum ha_base_keytype type=(enum ha_base_keytype) keyseg->type;
+    uint length=min((uint) keyseg->length,(uint) k_length);
+    uint char_length;
+    uchar *pos;
+    CHARSET_INFO *cs=keyseg->charset;
+
     if (keyseg->null_bit)
     {
       k_length--;
       if (!(*key++= (char) 1-*old++))			/* Copy null marker */
       {
 	k_length-=length;
-        if (keyseg->flag & (HA_VAR_LENGTH | HA_BLOB_PART))
+        if (keyseg->flag & (HA_VAR_LENGTH_PART | HA_BLOB_PART))
+        {
           k_length-=2;                                  /* Skip length */
+          old+= 2;
+        }
 	continue;					/* Found NULL */
       }
     }
+    char_length= (!is_ft && cs && cs->mbmaxlen > 1) ? length/cs->mbmaxlen : length;
     pos=old;
     if (keyseg->flag & HA_SPACE_PACK)
     {
-      end=pos+length;
+      uchar *end=pos+length;
       if (type != HA_KEYTYPE_NUM)
       {
 	while (end > pos && end[-1] == ' ')
@@ -198,21 +266,24 @@ uint _mi_pack_key(register MI_INFO *info, uint keynr, uchar *key, uchar *old,
       }
       k_length-=length;
       length=(uint) (end-pos);
-      store_key_length_inc(key,length);
-      memcpy((byte*) key,pos,(size_t) length);
-      key+= length;
+      FIX_LENGTH(cs, pos, length, char_length);
+      store_key_length_inc(key,char_length);
+      memcpy((byte*) key,pos,(size_t) char_length);
+      key+= char_length;
       continue;
     }
-    else if (keyseg->flag & (HA_VAR_LENGTH | HA_BLOB_PART))
+    else if (keyseg->flag & (HA_VAR_LENGTH_PART | HA_BLOB_PART))
     {
       /* Length of key-part used with mi_rkey() always 2 */
       uint tmp_length=uint2korr(pos);
       k_length-= 2+length;
+      pos+=2;
       set_if_smaller(length,tmp_length);	/* Safety */
-      store_key_length_inc(key,length);
-      old+=2;					/* Skipp length */
-      memcpy((byte*) key, pos+2,(size_t) length);
-      key+= length;
+      FIX_LENGTH(cs, pos, length, char_length);
+      store_key_length_inc(key,char_length);
+      old+=2;					/* Skip length */
+      memcpy((byte*) key, pos,(size_t) char_length);
+      key+= char_length;
       continue;
     }
     else if (keyseg->flag & HA_SWAP_KEY)
@@ -225,7 +296,10 @@ uint _mi_pack_key(register MI_INFO *info, uint keynr, uchar *key, uchar *old,
       }
       continue;
     }
-    memcpy((byte*) key,pos,(size_t) length);
+    FIX_LENGTH(cs, pos, length, char_length);
+    memcpy((byte*) key, pos, char_length);
+    if (length > char_length)
+      cs->cset->fill(cs, (char*) key+char_length, length-char_length, ' ');
     key+= length;
     k_length-=length;
   }
@@ -280,11 +354,11 @@ static int _mi_put_key_in_record(register MI_INFO *info, uint keynr,
 {
   reg2 byte *key;
   byte *pos,*key_end;
-  reg1 MI_KEYSEG *keyseg;
+  reg1 HA_KEYSEG *keyseg;
   byte *blob_ptr;
   DBUG_ENTER("_mi_put_key_in_record");
 
-  blob_ptr= info->lastkey2;                     /* Place to put blob parts */
+  blob_ptr= (byte*) info->lastkey2;             /* Place to put blob parts */
   key=(byte*) info->lastkey;                    /* KEy that was read */
   key_end=key+info->lastkey_length;
   for (keyseg=info->s->keyinfo[keynr].seg ; keyseg->type ;keyseg++)
@@ -298,6 +372,26 @@ static int _mi_put_key_in_record(register MI_INFO *info, uint keynr,
       }
       record[keyseg->null_pos]&= ~keyseg->null_bit;
     }
+    if (keyseg->type == HA_KEYTYPE_BIT)
+    {
+      uint length= keyseg->length;
+
+      if (keyseg->bit_length)
+      {
+        uchar bits= *key++;
+        set_rec_bits(bits, record + keyseg->bit_pos, keyseg->bit_start,
+                     keyseg->bit_length);
+        length--;
+      }
+      else
+      {
+        clr_rec_bits(record + keyseg->bit_pos, keyseg->bit_start,
+                     keyseg->bit_length);
+      }
+      memcpy(record + keyseg->start, (byte*) key, length);
+      key+= length;
+      continue;
+    }
     if (keyseg->flag & HA_SPACE_PACK)
     {
       uint length;
@@ -321,7 +415,7 @@ static int _mi_put_key_in_record(register MI_INFO *info, uint keynr,
       continue;
     }
 
-    if (keyseg->flag & HA_VAR_LENGTH)
+    if (keyseg->flag & HA_VAR_LENGTH_PART)
     {
       uint length;
       get_key_length(length,key);
@@ -329,7 +423,13 @@ static int _mi_put_key_in_record(register MI_INFO *info, uint keynr,
       if (length > keyseg->length || key+length > key_end)
 	goto err;
 #endif
-      memcpy(record+keyseg->start,(byte*) key, length);
+      /* Store key length */
+      if (keyseg->bit_start == 1)
+        *(uchar*) (record+keyseg->start)= (uchar) length;
+      else
+        int2store(record+keyseg->start, length);
+      /* And key data */
+      memcpy(record+keyseg->start + keyseg->bit_start, (byte*) key, length);
       key+= length;
     }
     else if (keyseg->flag & HA_BLOB_PART)
@@ -344,6 +444,10 @@ static int _mi_put_key_in_record(register MI_INFO *info, uint keynr,
 	     (char*) &blob_ptr,sizeof(char*));
       memcpy(blob_ptr,key,length);
       blob_ptr+=length;
+
+      /* The above changed info->lastkey2. Inform mi_rnext_same(). */
+      info->update&= ~HA_STATE_RNEXT_SAME;
+
       _my_store_blob_length(record+keyseg->start,
 			    (uint) keyseg->bit_start,length);
       key+=length;
@@ -391,6 +495,7 @@ int _mi_read_key_record(MI_INFO *info, my_off_t filepos, byte *buf)
     {				/* Read only key */
       if (_mi_put_key_in_record(info,(uint) info->lastinx,buf))
       {
+        mi_print_error(info->s, HA_ERR_CRASHED);
 	my_errno=HA_ERR_CRASHED;
 	return -1;
       }
@@ -402,26 +507,25 @@ int _mi_read_key_record(MI_INFO *info, my_off_t filepos, byte *buf)
   return(-1);				/* Wrong data to read */
 }
 
-  
+
 /*
-  Update auto_increment info
+  Retrieve auto_increment info
 
   SYNOPSIS
-    update_auto_increment()
+    retrieve_auto_increment()
     info			MyISAM handler
     record			Row to update
 
   IMPLEMENTATION
-    Only replace the auto_increment value if it is higher than the previous
-    one. For signed columns we don't update the auto increment value if it's
+    For signed columns we don't retrieve the auto increment value if it's
     less than zero.
 */
 
-void update_auto_increment(MI_INFO *info,const byte *record)
+ulonglong retrieve_auto_increment(MI_INFO *info,const byte *record)
 {
   ulonglong value= 0;			/* Store unsigned values here */
   longlong s_value= 0;			/* Store signed values here */
-  MI_KEYSEG *keyseg= info->s->keyinfo[info->s->base.auto_key-1].seg;
+  HA_KEYSEG *keyseg= info->s->keyinfo[info->s->base.auto_key-1].seg;
   const uchar *key= (uchar*) record + keyseg->start;
 
   switch (keyseg->type) {
@@ -482,6 +586,5 @@ void update_auto_increment(MI_INFO *info,const byte *record)
     and if s_value == 0 then value will contain either s_value or the
     correct value.
   */
-  set_if_bigger(info->s->state.auto_increment,
-                (s_value > 0) ? (ulonglong) s_value : value);
+  return (s_value > 0) ? (ulonglong) s_value : value;
 }
