@@ -52,6 +52,9 @@ int mi_write(MI_INFO *info, byte *record)
   DBUG_ENTER("mi_write");
   DBUG_PRINT("enter",("isam: %d  data: %d",info->s->kfile,info->dfile));
 
+  DBUG_EXECUTE_IF("myisam_pretend_crashed_table_on_usage",
+                  mi_print_error(info->s, HA_ERR_CRASHED);
+                  DBUG_RETURN(my_errno= HA_ERR_CRASHED););
   if (share->options & HA_OPTION_READ_ONLY_DATA)
   {
     DBUG_RETURN(my_errno=EACCES);
@@ -64,7 +67,8 @@ int mi_write(MI_INFO *info, byte *record)
 			       MYF(MY_SEEK_NOT_DONE) | info->lock_wait))
     goto err;
 #endif
-  filepos= ((share->state.dellink != HA_OFFSET_ERROR) ?
+  filepos= ((share->state.dellink != HA_OFFSET_ERROR &&
+             !info->append_insert_at_end) ?
 	    share->state.dellink :
 	    info->state->data_file_length);
 
@@ -97,7 +101,7 @@ int mi_write(MI_INFO *info, byte *record)
   buff=info->lastkey2;
   for (i=0 ; i < share->base.keys ; i++)
   {
-    if (((ulonglong) 1 << i) & share->state.key_map)
+    if (mi_is_key_active(share->state.key_map, i))
     {
       bool local_lock_tree= (lock_tree &&
 			     !(info->bulk_insert &&
@@ -142,10 +146,11 @@ int mi_write(MI_INFO *info, byte *record)
   {
     if ((*share->write_record)(info,record))
       goto err;
-    share->state.checksum+=info->checksum;
+    info->state->checksum+=info->checksum;
   }
   if (share->base.auto_key)
-    update_auto_increment(info,record);
+    set_if_bigger(info->s->state.auto_increment,
+                  retrieve_auto_increment(info, record));
   info->update= (HA_STATE_CHANGED | HA_STATE_AKTIV | HA_STATE_WRITTEN |
 		 HA_STATE_ROW_CHANGED);
   info->state->records++;
@@ -175,7 +180,7 @@ err:
     info->errkey= (int) i;
     while ( i-- > 0)
     {
-      if (((ulonglong) 1 << i) & share->state.key_map)
+      if (mi_is_key_active(share->state.key_map, i))
       {
 	bool local_lock_tree= (lock_tree &&
 			       !(info->bulk_insert &&
@@ -207,7 +212,10 @@ err:
     }
   }
   else
+  {
+    mi_print_error(info->s, HA_ERR_CRASHED);
     mi_mark_crashed(info);
+  }
   info->update= (HA_STATE_CHANGED | HA_STATE_WRITTEN | HA_STATE_ROW_CHANGED);
   my_errno=save_errno;
 err2:
@@ -253,7 +261,7 @@ int _mi_ck_write_btree(register MI_INFO *info, uint keynr, uchar *key,
     comp_flag=SEARCH_BIGGER;			/* Put after same key */
   else if (keyinfo->flag & (HA_NOSAME|HA_FULLTEXT))
   {
-    comp_flag=SEARCH_FIND | SEARCH_UPDATE;	/* No dupplicates */
+    comp_flag=SEARCH_FIND | SEARCH_UPDATE;	/* No duplicates */
     if (keyinfo->flag & HA_NULL_ARE_EQUAL)
       comp_flag|= SEARCH_NULL_ARE_EQUAL;
   }
@@ -352,6 +360,7 @@ static int w_search(register MI_INFO *info, register MI_KEYDEF *keyinfo,
       dupp_key_pos=_mi_dpos(info,0,keybuff+tmp_key_length);
     else
       dupp_key_pos= HA_OFFSET_ERROR;
+
     if (keyinfo->flag & HA_FULLTEXT)
     {
       uint off;
@@ -482,6 +491,7 @@ int _mi_insert(register MI_INFO *info, register MI_KEYDEF *keyinfo,
   {
     if (t_length >= keyinfo->maxlength*2+MAX_POINTER_LENGTH)
     {
+      mi_print_error(info->s, HA_ERR_CRASHED);
       my_errno=HA_ERR_CRASHED;
       DBUG_RETURN(-1);
     }
@@ -491,6 +501,7 @@ int _mi_insert(register MI_INFO *info, register MI_KEYDEF *keyinfo,
   {
     if (-t_length >= keyinfo->maxlength*2+MAX_POINTER_LENGTH)
     {
+      mi_print_error(info->s, HA_ERR_CRASHED);
       my_errno=HA_ERR_CRASHED;
       DBUG_RETURN(-1);
     }
@@ -586,6 +597,7 @@ int _mi_split_page(register MI_INFO *info, register MI_KEYDEF *keyinfo,
 			      &after_key);
   if (!key_pos)
     DBUG_RETURN(-1);
+
   length=(uint) (key_pos-buff);
   a_length=mi_getint(buff);
   mi_putint(buff,length,nod_flag);
@@ -606,6 +618,7 @@ int _mi_split_page(register MI_INFO *info, register MI_KEYDEF *keyinfo,
 	/* Store new page */
   if (!(*keyinfo->get_key)(keyinfo,nod_flag,&key_pos,key_buff))
     DBUG_RETURN(-1);
+
   t_length=(*keyinfo->pack_key)(keyinfo,nod_flag,(uchar *) 0,
 				(uchar*) 0, (uchar*) 0,
 				key_buff, &s_temp);
@@ -712,6 +725,7 @@ static uchar *_mi_find_last_pos(MI_KEYDEF *keyinfo, uchar *page,
     memcpy(key, key_buff, length);		/* previous key */
     if (!(length=(*keyinfo->get_key)(keyinfo,0,&page,key_buff)))
     {
+      mi_print_error(keyinfo->share, HA_ERR_CRASHED);
       my_errno=HA_ERR_CRASHED;
       DBUG_RETURN(0);
     }
@@ -935,20 +949,21 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows)
   MI_KEYDEF *key=share->keyinfo;
   bulk_insert_param *params;
   uint i, num_keys, total_keylength;
-  ulonglong key_map=0;
+  ulonglong key_map;
   DBUG_ENTER("_mi_init_bulk_insert");
   DBUG_PRINT("enter",("cache_size: %lu", cache_size));
 
   DBUG_ASSERT(!info->bulk_insert &&
 	      (!rows || rows >= MI_MIN_ROWS_TO_USE_BULK_INSERT));
 
+  mi_clear_all_keys_active(key_map);
   for (i=total_keylength=num_keys=0 ; i < share->base.keys ; i++)
   {
-    if (!(key[i].flag & HA_NOSAME) && share->base.auto_key != i+1
-        && test(share->state.key_map & ((ulonglong) 1 << i)))
+    if (! (key[i].flag & HA_NOSAME) && (share->base.auto_key != i + 1) &&
+        mi_is_key_active(share->state.key_map, i))
     {
       num_keys++;
-      key_map |=((ulonglong) 1 << i);
+      mi_set_key_active(key_map, i);
       total_keylength+=key[i].maxlength+TREE_ELEMENT_EXTRA_SIZE;
     }
   }
@@ -972,7 +987,7 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows)
   params=(bulk_insert_param *)(info->bulk_insert+share->base.keys);
   for (i=0 ; i < share->base.keys ; i++)
   {
-    if (test(key_map & ((ulonglong) 1 << i)))
+    if (mi_is_key_active(key_map, i))
     {
       params->info=info;
       params->keynr=i;
