@@ -307,7 +307,7 @@ static ha_rows NEAR_F find_all_keys(MI_SORT_PARAM *info, uint keys,
 #ifdef THREAD
 /* Search after all keys and place them in a temp. file */
 
-pthread_handler_decl(thr_find_all_keys,arg)
+pthread_handler_t thr_find_all_keys(void *arg)
 {
   MI_SORT_PARAM *info= (MI_SORT_PARAM*) arg;
   int error;
@@ -376,7 +376,10 @@ pthread_handler_decl(thr_find_all_keys,arg)
     {
       if (my_init_dynamic_array(&info->buffpek, sizeof(BUFFPEK),
 				maxbuffer, maxbuffer/2))
+      {
         my_free((gptr) sort_keys,MYF(0));
+        sort_keys= (uchar **) NULL; /* for err: label */
+      }
       else
         break;
     }
@@ -479,7 +482,7 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
     }
     if (!got_error)
     {
-      share->state.key_map|=(ulonglong) 1 << sinfo->key;
+      mi_set_key_active(share->state.key_map, sinfo->key);
       if (param->testflag & T_STATISTICS)
         update_key_parts(sinfo->keyinfo, rec_per_key_part, sinfo->unique,
                          param->stats_method == MI_STATS_METHOD_IGNORE_NULLS?
@@ -866,7 +869,8 @@ merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
   uchar *strpos;
   BUFFPEK *buffpek,**refpek;
   QUEUE queue;
-  volatile my_bool *killed= killed_ptr(info->sort_info->param);
+  volatile int *killed= killed_ptr(info->sort_info->param);
+
   DBUG_ENTER("merge_buffers");
 
   count=error=0;
