@@ -12,16 +12,18 @@ use strict;
 #use POSIX ":sys_wait_h";
 use POSIX 'WNOHANG';
 
-sub mtr_run ($$$$$$);
-sub mtr_spawn ($$$$$$);
+sub mtr_run ($$$$$$;$);
+sub mtr_spawn ($$$$$$;$);
 sub mtr_stop_mysqld_servers ($);
 sub mtr_kill_leftovers ();
 sub mtr_record_dead_children ();
 sub mtr_exit ($);
 sub sleep_until_file_created ($$$);
+sub mtr_kill_processes ($);
+sub mtr_kill_process ($$$$);
 
 # static in C
-sub spawn_impl ($$$$$$$);
+sub spawn_impl ($$$$$$$$);
 
 ##############################################################################
 #
@@ -32,37 +34,43 @@ sub spawn_impl ($$$$$$$);
 # This function try to mimic the C version used in "netware/mysql_test_run.c"
 # FIXME learn it to handle append mode as well, a "new" flag or a "append"
 
-sub mtr_run ($$$$$$) {
+sub mtr_run ($$$$$$;$) {
   my $path=       shift;
   my $arg_list_t= shift;
   my $input=      shift;
   my $output=     shift;
   my $error=      shift;
   my $pid_file=   shift;
+  my $spawn_opts= shift;
 
-  return spawn_impl($path,$arg_list_t,'run',$input,$output,$error,$pid_file);
+  return spawn_impl($path,$arg_list_t,'run',$input,$output,$error,$pid_file,
+    $spawn_opts);
 }
 
-sub mtr_run_test ($$$$$$) {
+sub mtr_run_test ($$$$$$;$) {
   my $path=       shift;
   my $arg_list_t= shift;
   my $input=      shift;
   my $output=     shift;
   my $error=      shift;
   my $pid_file=   shift;
+  my $spawn_opts= shift;
 
-  return spawn_impl($path,$arg_list_t,'test',$input,$output,$error,$pid_file);
+  return spawn_impl($path,$arg_list_t,'test',$input,$output,$error,$pid_file,
+    $spawn_opts);
 }
 
-sub mtr_spawn ($$$$$$) {
+sub mtr_spawn ($$$$$$;$) {
   my $path=       shift;
   my $arg_list_t= shift;
   my $input=      shift;
   my $output=     shift;
   my $error=      shift;
   my $pid_file=   shift;
+  my $spawn_opts= shift;
 
-  return spawn_impl($path,$arg_list_t,'spawn',$input,$output,$error,$pid_file);
+  return spawn_impl($path,$arg_list_t,'spawn',$input,$output,$error,$pid_file,
+    $spawn_opts);
 }
 
 
@@ -72,7 +80,7 @@ sub mtr_spawn ($$$$$$) {
 #
 ##############################################################################
 
-sub spawn_impl ($$$$$$$) {
+sub spawn_impl ($$$$$$$$) {
   my $path=       shift;
   my $arg_list_t= shift;
   my $mode=       shift;
@@ -80,6 +88,7 @@ sub spawn_impl ($$$$$$$) {
   my $output=     shift;
   my $error=      shift;
   my $pid_file=   shift;                 # FIXME
+  my $spawn_opts= shift;
 
   if ( $::opt_script_debug )
   {
@@ -89,6 +98,18 @@ sub spawn_impl ($$$$$$$) {
     print STDERR "#### ", "STDOUT $output\n" if $output;
     print STDERR "#### ", "STDERR $error\n" if $error;
     print STDERR "#### ", "$mode : $path ", join(" ",@$arg_list_t), "\n";
+    print STDERR "#### ", "spawn options:\n";
+    if ($spawn_opts)
+    {
+      foreach my $key (sort keys %{$spawn_opts})
+      {
+        print STDERR "#### ", "  - $key: $spawn_opts->{$key}\n";
+      }
+    }
+    else
+    {
+      print STDERR "#### ", "  none\n";
+    }
     print STDERR "#### ", "-" x 78, "\n";
   }
 
@@ -135,11 +156,18 @@ sub spawn_impl ($$$$$$$) {
 #       $ENV{'COMSPEC'}= "$::glob_cygwin_shell -c";
       }
 
+      my $log_file_open_mode = '>';
+
+      if ($spawn_opts and $spawn_opts->{'append_log_file'})
+      {
+        $log_file_open_mode = '>>';
+      }
+
       if ( $output )
       {
-        if ( ! open(STDOUT,">",$output) )
+        if ( ! open(STDOUT,$log_file_open_mode,$output) )
         {
-          mtr_error("can't redirect STDOUT to \"$output\": $!");
+          mtr_child_error("can't redirect STDOUT to \"$output\": $!");
         }
       }
 
@@ -149,14 +177,14 @@ sub spawn_impl ($$$$$$$) {
         {
           if ( ! open(STDERR,">&STDOUT") )
           {
-            mtr_error("can't dup STDOUT: $!");
+            mtr_child_error("can't dup STDOUT: $!");
           }
         }
         else
         {
-          if ( ! open(STDERR,">",$error) )
+          if ( ! open(STDERR,$log_file_open_mode,$error) )
           {
-            mtr_error("can't redirect STDERR to \"$output\": $!");
+            mtr_child_error("can't redirect STDERR to \"$error\": $!");
           }
         }
       }
@@ -165,13 +193,13 @@ sub spawn_impl ($$$$$$$) {
       {
         if ( ! open(STDIN,"<",$input) )
         {
-          mtr_error("can't redirect STDIN to \"$input\": $!");
+          mtr_child_error("can't redirect STDIN to \"$input\": $!");
         }
       }
 
       if ( ! exec($path,@$arg_list_t) )
       {
-        mtr_error("failed to execute \"$path\": $!");
+        mtr_child_error("failed to execute \"$path\": $!");
       }
     }
   }
@@ -185,10 +213,6 @@ sub spawn_parent_impl {
 
   if ( $mode eq 'run' or $mode eq 'test' )
   {
-    my $exit_value= -1;
-#    my $signal_num=  0;
-#    my $dumped_core= 0;
-
     if ( $mode eq 'run' )
     {
       # Simple run of command, we wait for it to return
@@ -199,12 +223,7 @@ sub spawn_parent_impl {
         mtr_error("$path ($pid) got lost somehow");
       }
 
-      $exit_value=  $?;
-#      $exit_value=  $? >> 8;
-#      $signal_num=  $? & 127;
-#      $dumped_core= $? & 128;
-
-      return $exit_value;
+      return mtr_process_exit_status($?);
     }
     else
     {
@@ -218,6 +237,8 @@ sub spawn_parent_impl {
       # FIXME is this as it should be? Can't mysqld terminate
       # normally from running a test case?
 
+      my $exit_value= -1;
+      my $saved_exit_value;
       my $ret_pid;                      # What waitpid() returns
 
       while ( ($ret_pid= waitpid(-1,0)) != -1 )
@@ -227,13 +248,28 @@ sub spawn_parent_impl {
         # but not $exit_value, this is flagged from
         # 
 
+        my $timer_name= mtr_timer_timeout($::glob_timers, $ret_pid);
+        if ( $timer_name )
+        {
+          if ( $timer_name eq "suite" )
+          {
+            # We give up here
+            # FIXME we should only give up the suite, not all of the run?
+            print STDERR "\n";
+            mtr_error("Test suite timeout");
+          }
+          elsif ( $timer_name eq "testcase" )
+          {
+            $saved_exit_value=  63;       # Mark as timeout
+            kill(9, $pid);                # Kill mysqltest
+            next;                         # Go on and catch the termination
+          }
+        }
+
         if ( $ret_pid == $pid )
         {
           # We got termination of mysqltest, we are done
-          $exit_value=  $?;
-#          $exit_value=  $? >> 8;
-#          $signal_num=  $? & 127;
-#          $dumped_core= $? & 128;
+          $exit_value= mtr_process_exit_status($?);
           last;
         }
 
@@ -281,7 +317,7 @@ sub spawn_parent_impl {
         }
       }
 
-      return $exit_value;
+      return $saved_exit_value || $exit_value;
     }
   }
   else
@@ -292,6 +328,23 @@ sub spawn_parent_impl {
 }
 
 
+# ----------------------------------------------------------------------
+# We try to emulate how an Unix shell calculates the exit code
+# ----------------------------------------------------------------------
+
+sub mtr_process_exit_status {
+  my $raw_status= shift;
+
+  if ( $raw_status & 127 )
+  {
+    return ($raw_status & 127) + 128;  # Signal num + 128
+  }
+  else
+  {
+    return $raw_status >> 8;           # Exit code
+  }
+}
+
 
 ##############################################################################
 #
@@ -308,9 +361,20 @@ sub mtr_kill_leftovers () {
 
   # First, kill all masters and slaves that would conflict with
   # this run. Make sure to remove the PID file, if any.
+  # FIXME kill IM manager first, else it will restart the servers, how?!
 
   my @args;
 
+  for ( my $idx; $idx < 2; $idx++ )
+  {
+    push(@args,{
+                pid      => 0,          # We don't know the PID
+                pidfile  => $::instance_manager->{'instances'}->[$idx]->{'path_pid'},
+                sockfile => $::instance_manager->{'instances'}->[$idx]->{'path_sock'},
+                port     => $::instance_manager->{'instances'}->[$idx]->{'port'},
+               });
+  }
+
   for ( my $idx; $idx < 2; $idx++ )
   {
     push(@args,{
@@ -331,7 +395,7 @@ sub mtr_kill_leftovers () {
                });
   }
 
-  mtr_mysqladmin_shutdown(\@args);
+  mtr_mysqladmin_shutdown(\@args, 20);
 
   # We now have tried to terminate nice. We have waited for the listen
   # port to be free, but can't really tell if the mysqld process died
@@ -402,8 +466,7 @@ sub mtr_kill_leftovers () {
 
         if ( kill(0, @pids) )           # Check if some left
         {
-          # FIXME maybe just mtr_warning() ?
-          mtr_error("can't kill process(es) " . join(" ", @pids));
+          mtr_warning("can't kill process(es) " . join(" ", @pids));
         }
       }
     }
@@ -416,7 +479,7 @@ sub mtr_kill_leftovers () {
   {
     if ( mtr_ping_mysqld_server($srv->{'port'}, $srv->{'sockfile'}) )
     {
-      mtr_error("can't kill old mysqld holding port $srv->{'port'}");
+      mtr_warning("can't kill old mysqld holding port $srv->{'port'}");
     }
   }
 }
@@ -441,7 +504,8 @@ sub mtr_stop_mysqld_servers ($) {
   # First try nice normal shutdown using 'mysqladmin'
   # ----------------------------------------------------------------------
 
-  mtr_mysqladmin_shutdown($spec);
+  # Shutdown time must be high as slave may be in reconnect
+  mtr_mysqladmin_shutdown($spec, 70);
 
   # ----------------------------------------------------------------------
   # We loop with waitpid() nonblocking to see how many of the ones we
@@ -507,17 +571,8 @@ sub mtr_stop_mysqld_servers ($) {
 
   start_reap_all();                     # Avoid zombies
 
- SIGNAL:
-  foreach my $sig (15,9)
-  {
-    my $retries= 20;                    # FIXME 20 seconds, this is silly!
-    kill($sig, keys %mysqld_pids);
-    while ( $retries-- and  kill(0, keys %mysqld_pids) )
-    {
-      mtr_debug("Sleep 1 second waiting for processes to die");
-      sleep(1)                      # Wait one second
-    }
-  }
+  my @mysqld_pids= keys %mysqld_pids;
+  mtr_kill_processes(\@mysqld_pids);
 
   stop_reap_all();                      # Get into control again
 
@@ -591,8 +646,9 @@ sub mtr_stop_mysqld_servers ($) {
 #
 ##############################################################################
 
-sub mtr_mysqladmin_shutdown () {
+sub mtr_mysqladmin_shutdown {
   my $spec= shift;
+  my $adm_shutdown_tmo= shift;
 
   my %mysql_admin_pids;
   my @to_kill_specs;
@@ -608,8 +664,6 @@ sub mtr_mysqladmin_shutdown () {
 
   foreach my $srv ( @to_kill_specs )
   {
-    # FIXME wrong log.....
-    # FIXME, stderr.....
     # Shutdown time must be high as slave may be in reconnect
     my $args;
 
@@ -631,12 +685,16 @@ sub mtr_mysqladmin_shutdown () {
       mtr_add_arg($args, "--protocol=tcp"); # Needed if no --socket
     }
     mtr_add_arg($args, "--connect_timeout=5");
-    mtr_add_arg($args, "--shutdown_timeout=20");
+    mtr_add_arg($args, "--shutdown_timeout=$adm_shutdown_tmo");
     mtr_add_arg($args, "shutdown");
-    # We don't wait for termination of mysqladmin
+
+    my $path_mysqladmin_log= "$::opt_vardir/log/mysqladmin.log";
     my $pid= mtr_spawn($::exe_mysqladmin, $args,
-                       "", $::path_manager_log, $::path_manager_log, "");
+                       "", $path_mysqladmin_log, $path_mysqladmin_log, "",
+                       { append_log_file => 1 });
     $mysql_admin_pids{$pid}= 1;
+
+    # We don't wait for termination of mysqladmin
   }
 
   # As mysqladmin is such a simple program, we trust it to terminate.
@@ -680,8 +738,6 @@ sub mtr_mysqladmin_shutdown () {
 
   $timeout or mtr_debug("At least one server is still listening to its port");
 
-  sleep(5) if $::glob_win32;            # FIXME next startup fails if no sleep
-
   return $res;
 }
 
@@ -724,7 +780,15 @@ sub mtr_record_dead_children () {
 }
 
 sub start_reap_all {
-  $SIG{CHLD}= 'IGNORE';                 # FIXME is this enough?
+  # This causes terminating processes to not become zombies, avoiding
+  # the need for (or possibility of) explicit waitpid().
+  $SIG{CHLD}= 'IGNORE';
+
+  # On some platforms (Linux, QNX, OSX, ...) there is potential race
+  # here. If a process terminated before setting $SIG{CHLD} (but after
+  # any attempt to waitpid() it), it will still be a zombie. So we
+  # have to handle any such process here.
+  while(waitpid(-1, &WNOHANG) > 0) { };
 }
 
 sub stop_reap_all {
@@ -769,36 +833,87 @@ sub sleep_until_file_created ($$$) {
   my $pidfile= shift;
   my $timeout= shift;
   my $pid=     shift;
+  my $sleeptime= 100; # Milliseconds
+  my $loops= ($timeout * 1000) / $sleeptime;
 
-  for ( my $loop= 1; $loop <= $timeout; $loop++ )
+  for ( my $loop= 1; $loop <= $loops; $loop++ )
   {
     if ( -r $pidfile )
     {
       return $pid;
     }
 
-    # Check if it died after the fork() was successful 
-    if ( waitpid($pid,&WNOHANG) == $pid )
+    # Check if it died after the fork() was successful
+    if ( $pid != 0 && waitpid($pid,&WNOHANG) == $pid )
     {
       return 0;
     }
 
-    mtr_debug("Sleep 1 second waiting for creation of $pidfile");
+    mtr_debug("Sleep $sleeptime milliseconds waiting for ".
+	      "creation of $pidfile");
 
-    if ( $loop % 60 == 0 )
+    # Print extra message every 60 seconds
+    my $seconds= ($loop * $sleeptime) / 1000;
+    if ( $seconds > 1 and $seconds % 60 == 0 )
     {
-      my $left= $timeout - $loop;
-      mtr_warning("Waited $loop seconds for $pidfile to be created, " .
+      my $left= $timeout - $seconds;
+      mtr_warning("Waited $seconds seconds for $pidfile to be created, " .
                   "still waiting for $left seconds...");
     }
 
-    sleep(1);
+    # Millisceond sleep emulated with select
+    select(undef, undef, undef, ($sleeptime/1000));
   }
 
   return 0;
 }
 
 
+sub mtr_kill_processes ($) {
+  my $pids = shift;
+
+  foreach my $sig (15, 9)
+  {
+    my $retries= 10;
+    while (1)
+    {
+      kill($sig, @{$pids});
+      last unless kill (0, @{$pids}) and $retries--;
+
+      mtr_debug("Sleep 2 second waiting for processes to die");
+      sleep(2);
+    }
+  }
+}
+
+
+sub mtr_kill_process ($$$$) {
+  my $pid= shift;
+  my $signal= shift;
+  my $total_retries= shift;
+  my $timeout= shift;
+
+  for (my $cur_attempt= 1; $cur_attempt <= $total_retries; ++$cur_attempt)
+  {
+    mtr_debug("Sending $signal to $pid...");
+
+    kill($signal, $pid);
+
+    unless (kill (0, $pid))
+    {
+      mtr_debug("Process $pid died.");
+      return;
+    }
+
+    mtr_debug("Sleeping $timeout second(s) waiting for processes to die...");
+
+    sleep($timeout);
+  }
+
+  mtr_debug("Process $pid is still alive after $total_retries " .
+            "of sending signal $signal.");
+}
+
 ##############################################################################
 #
 #  When we exit, we kill off all children
@@ -808,11 +923,23 @@ sub sleep_until_file_created ($$$) {
 # FIXME something is wrong, we sometimes terminate with "Hangup" written
 # to tty, and no STDERR output telling us why.
 
+# FIXME for some readon, setting HUP to 'IGNORE' will cause exit() to
+# write out "Hangup", and maybe loose some output. We insert a sleep...
+
 sub mtr_exit ($) {
   my $code= shift;
 #  cluck("Called mtr_exit()");
+  mtr_timer_stop_all($::glob_timers);
   local $SIG{HUP} = 'IGNORE';
-  kill('HUP', -$$);
+  # ToDo: Signalling -$$ will only work if we are the process group
+  # leader (in fact on QNX it will signal our session group leader,
+  # which might be Do-compile or Pushbuild, causing tests to be
+  # aborted). So we only do it if we are the group leader. We might
+  # set ourselves as the group leader at startup (with
+  # POSIX::setpgrp(0,0)), but then care must be needed to always do
+  # proper child process cleanup.
+  kill('HUP', -$$) if $$ == getpgrp();
+  sleep 2;
   exit($code);
 }
 
