@@ -486,6 +486,7 @@ void start_master()
 #endif
   add_arg(&al, "--local-infile");
   add_arg(&al, "--core");
+  add_arg(&al, "--log-bin-trust-function-creators");
   add_arg(&al, "--datadir=%s", master_dir);
 #ifndef __WIN__
   add_arg(&al, "--pid-file=%s", master_pid);
@@ -1345,11 +1346,11 @@ void setup(char *file __attribute__((unused)))
 #endif /* HAVE_OPENSSL */
 
   /* OpenSSL paths */
-  snprintf(ca_cert, FN_REFLEN, "%s/SSL/cacert.pem", base_dir);
-  snprintf(server_cert, FN_REFLEN, "%s/SSL/server-cert.pem", base_dir);
-  snprintf(server_key, FN_REFLEN, "%s/SSL/server-key.pem", base_dir);
-  snprintf(client_cert, FN_REFLEN, "%s/SSL/client-cert.pem", base_dir);
-  snprintf(client_key, FN_REFLEN, "%s/SSL/client-key.pem", base_dir);
+  snprintf(ca_cert, FN_REFLEN, "%s/std_data/cacert.pem", mysql_test_dir);
+  snprintf(server_cert, FN_REFLEN, "%s/std_data/server-cert.pem", mysql_test_dir);
+  snprintf(server_key, FN_REFLEN, "%s/std_data/server-key.pem", mysql_test_dir);
+  snprintf(client_cert, FN_REFLEN, "%s/std_data/client-cert.pem", mysql_test_dir);
+  snprintf(client_key, FN_REFLEN, "%s/std_data/client-key.pem", mysql_test_dir);
 
   /* setup files */
   snprintf(mysqld_file, FN_REFLEN, "%s/mysqld", bin_dir);
@@ -1378,11 +1379,11 @@ void setup(char *file __attribute__((unused)))
 #endif /* HAVE_OPENSSL */
 
   /* OpenSSL paths */
-  snprintf(ca_cert, FN_REFLEN, "%s/SSL/cacert.pem", base_dir);
-  snprintf(server_cert, FN_REFLEN, "%s/SSL/server-cert.pem", base_dir);
-  snprintf(server_key, FN_REFLEN, "%s/SSL/server-key.pem", base_dir);
-  snprintf(client_cert, FN_REFLEN, "%s/SSL/client-cert.pem", base_dir);
-  snprintf(client_key, FN_REFLEN, "%s/SSL/client-key.pem", base_dir);
+  snprintf(ca_cert, FN_REFLEN, "%s/std_data/cacert.pem", mysql_test_dir);
+  snprintf(server_cert, FN_REFLEN, "%s/std_data/server-cert.pem", mysql_test_dir);
+  snprintf(server_key, FN_REFLEN, "%s/std_data/server-key.pem", mysql_test_dir);
+  snprintf(client_cert, FN_REFLEN, "%s/std_data/client-cert.pem", mysql_test_dir);
+  snprintf(client_key, FN_REFLEN, "%s/std_data/client-key.pem", mysql_test_dir);
 
   /* setup files */
 #ifdef _DEBUG 
@@ -1411,11 +1412,11 @@ void setup(char *file __attribute__((unused)))
 #endif /* HAVE_OPENSSL */
 
   /* OpenSSL paths */
-  snprintf(ca_cert, FN_REFLEN, "%s/SSL/cacert.pem", base_dir);
-  snprintf(server_cert, FN_REFLEN, "%s/SSL/server-cert.pem", base_dir);
-  snprintf(server_key, FN_REFLEN, "%s/SSL/server-key.pem", base_dir);
-  snprintf(client_cert, FN_REFLEN, "%s/SSL/client-cert.pem", base_dir);
-  snprintf(client_key, FN_REFLEN, "%s/SSL/client-key.pem", base_dir);
+  snprintf(ca_cert, FN_REFLEN, "%s/std_data/cacert.pem", mysql_test_dir);
+  snprintf(server_cert, FN_REFLEN, "%s/std_data/server-cert.pem", mysql_test_dir);
+  snprintf(server_key, FN_REFLEN, "%s/std_data/server-key.pem", mysql_test_dir);
+  snprintf(client_cert, FN_REFLEN, "%s/std_data/client-cert.pem", mysql_test_dir);
+  snprintf(client_key, FN_REFLEN, "%s/std_data/client-key.pem", mysql_test_dir);
 
   /* setup files */
   snprintf(mysqld_file, FN_REFLEN, "%s/sql/mysqld", base_dir);
@@ -1507,9 +1508,12 @@ void setup(char *file __attribute__((unused)))
     static char env_CHARSETSDIR[FN_REFLEN*2];
     static char env_MYSQL[FN_REFLEN*2];
     static char env_MYSQL_FIX_SYSTEM_TABLES[FN_REFLEN*2];
-    static char env_NDB_TOOLS_DIR[FN_REFLEN*2];
     static char env_CLIENT_BINDIR[FN_REFLEN*2];
     static char env_MYSQL_CLIENT_TEST[FN_REFLEN*2];
+    static char env_NDB_TOOLS_DIR[FN_REFLEN*2];
+    static char env_NDB_MGM[FN_REFLEN*2];
+    static char env_NDB_BACKUP_DIR[FN_REFLEN*2];
+    static char env_NDB_TOOLS_OUTPUT[FN_REFLEN*2];
     
     snprintf(env_MYSQL_TEST_DIR,FN_REFLEN*2,
              "MYSQL_TEST_DIR=%s",mysql_test_dir);
@@ -1549,9 +1553,6 @@ void setup(char *file __attribute__((unused)))
 	     base_dir,master_port, master_socket, base_dir, bin_dir);
     putenv(env_MYSQL_FIX_SYSTEM_TABLES);
      
-    snprintf(env_NDB_TOOLS_DIR, FN_REFLEN*2,
-             "NDB_TOOLS_DIR=%s/ndb/tools", base_dir);
-    putenv(env_NDB_TOOLS_DIR);
     
     snprintf(env_CLIENT_BINDIR, FN_REFLEN*2,
              "CLIENT_BINDIR=%s", bin_dir);
@@ -1563,6 +1564,30 @@ void setup(char *file __attribute__((unused)))
 	     base_dir, master_socket, master_port);
     putenv(env_MYSQL_CLIENT_TEST);
     
+    // NDB
+    
+    snprintf(env_NDB_TOOLS_DIR, FN_REFLEN*2,
+             "NDB_TOOLS_DIR=%s/ndb/tools", base_dir);
+    putenv(env_NDB_TOOLS_DIR);
+  
+    snprintf(env_NDB_MGM, FN_REFLEN*2,
+             "NDB_MGM=%s/ndb/src/mgmclient/ndb_mgm", base_dir);
+    putenv(env_NDB_MGM);
+  
+    //NDBCLUSTER_PORT=9350
+    snprintf(env_NDB_BACKUP_DIR, FN_REFLEN*2,
+             "NDB_BACKUP_DIR=%s/var/ndbcluster-%i", mysql_test_dir, 9350);
+    putenv(env_NDB_BACKUP_DIR);
+  
+    snprintf(env_NDB_TOOLS_OUTPUT, FN_REFLEN*2,
+             "NDB_TOOLS_OUTPUT=%s/var/log/ndb_tools.log", mysql_test_dir);
+    putenv(env_NDB_TOOLS_OUTPUT);
+
+    putenv((char *)"NDB_STATUS_OK=1");
+  
+// NDB_MGM="$BASEDIR/ndb/src/mgmclient/ndb_mgm"
+// NDB_BACKUP_DIR=$MYSQL_TEST_DIR/var/ndbcluster-$NDBCLUSTER_PORT
+// NDB_TOOLS_OUTPUT=$MYSQL_TEST_DIR/var/log/ndb_tools.log
   }
   
 #endif
