@@ -38,6 +38,22 @@ my_bool my_charset_same(CHARSET_INFO *cs1, CHARSET_INFO *cs2)
 }
 
 
+static uint
+get_collation_number_internal(const char *name)
+{
+  CHARSET_INFO **cs;
+  for (cs= all_charsets;
+       cs < all_charsets+array_elements(all_charsets)-1 ;
+       cs++)
+  {
+    if ( cs[0] && cs[0]->name && 
+         !my_strcasecmp(&my_charset_latin1, cs[0]->name, name))
+      return cs[0]->number;
+  }  
+  return 0;
+}
+
+
 static my_bool init_state_maps(CHARSET_INFO *cs)
 {
   uint i;
@@ -97,7 +113,7 @@ static my_bool init_state_maps(CHARSET_INFO *cs)
 
   /* Special handling of hex and binary strings */
   state_map[(uchar)'x']= state_map[(uchar)'X']= (uchar) MY_LEX_IDENT_OR_HEX;
-  state_map[(uchar)'b']= state_map[(uchar)'b']= (uchar) MY_LEX_IDENT_OR_BIN;
+  state_map[(uchar)'b']= state_map[(uchar)'B']= (uchar) MY_LEX_IDENT_OR_BIN;
   state_map[(uchar)'n']= state_map[(uchar)'N']= (uchar) MY_LEX_IDENT_OR_NCHAR;
   return 0;
 }
@@ -189,7 +205,8 @@ static my_bool simple_cs_is_full(CHARSET_INFO *cs)
 
 static int add_collation(CHARSET_INFO *cs)
 {
-  if (cs->name && (cs->number || (cs->number=get_collation_number(cs->name))))
+  if (cs->name && (cs->number ||
+                   (cs->number=get_collation_number_internal(cs->name))))
   {
     if (!all_charsets[cs->number])
     {
@@ -419,18 +436,8 @@ void free_charsets(void)
 
 uint get_collation_number(const char *name)
 {
-  CHARSET_INFO **cs;
   init_available_charsets(MYF(0));
-  
-  for (cs= all_charsets;
-       cs < all_charsets+array_elements(all_charsets)-1 ;
-       cs++)
-  {
-    if ( cs[0] && cs[0]->name && 
-         !my_strcasecmp(&my_charset_latin1, cs[0]->name, name))
-      return cs[0]->number;
-  }  
-  return 0;   /* this mimics find_type() */
+  return get_collation_number_internal(name);
 }
 
 
@@ -549,10 +556,10 @@ CHARSET_INFO *get_charset_by_csname(const char *cs_name,
   DBUG_PRINT("enter",("name: '%s'", cs_name));
 
   (void) init_available_charsets(MYF(0));	/* If it isn't initialized */
-  
+
   cs_number= get_charset_number(cs_name, cs_flags);
   cs= cs_number ? get_internal_charset(cs_number, flags) : NULL;
-  
+
   if (!cs && (flags & MY_WME))
   {
     char index_file[FN_REFLEN];
@@ -564,21 +571,54 @@ CHARSET_INFO *get_charset_by_csname(const char *cs_name,
 }
 
 
-ulong escape_string_for_mysql(CHARSET_INFO *charset_info, char *to,
+/*
+  Escape string with backslashes (\)
+
+  SYNOPSIS
+    escape_string_for_mysql()
+    charset_info        Charset of the strings
+    to                  Buffer for escaped string
+    to_length           Length of destination buffer, or 0
+    from                The string to escape
+    length              The length of the string to escape
+
+  DESCRIPTION
+    This escapes the contents of a string by adding backslashes before special
+    characters, and turning others into specific escape sequences, such as
+    turning newlines into \n and null bytes into \0.
+
+  NOTE
+    To maintain compatibility with the old C API, to_length may be 0 to mean
+    "big enough"
+
+  RETURN VALUES
+    ~0          The escaped string did not fit in the to buffer
+    >=0         The length of the escaped string
+*/
+
+ulong escape_string_for_mysql(CHARSET_INFO *charset_info,
+                              char *to, ulong to_length,
                               const char *from, ulong length)
 {
   const char *to_start= to;
-  const char *end;
+  const char *end, *to_end=to_start + (to_length ? to_length-1 : 2*length);
+  my_bool overflow= FALSE;
 #ifdef USE_MB
   my_bool use_mb_flag= use_mb(charset_info);
 #endif
-  for (end= from + length; from != end; from++)
+  for (end= from + length; from < end; from++)
   {
+    char escape= 0;
 #ifdef USE_MB
-    int l;
-    if (use_mb_flag && (l= my_ismbchar(charset_info, from, end)))
+    int tmp_length;
+    if (use_mb_flag && (tmp_length= my_ismbchar(charset_info, from, end)))
     {
-      while (l--)
+      if (to + tmp_length > to_end)
+      {
+        overflow= TRUE;
+        break;
+      }
+      while (tmp_length--)
 	*to++= *from++;
       from--;
       continue;
@@ -594,48 +634,55 @@ ulong escape_string_for_mysql(CHARSET_INFO *charset_info, char *to,
      multi-byte character into a valid one. For example, 0xbf27 is not
      a valid GBK character, but 0xbf5c is. (0x27 = ', 0x5c = \)
     */
-    if (use_mb_flag && (l= my_mbcharlen(charset_info, *from)) > 1)
-    {
-      *to++= '\\';
-      *to++= *from;
-      continue;
-    }
+    if (use_mb_flag && (tmp_length= my_mbcharlen(charset_info, *from)) > 1)
+      escape= *from;
+    else
 #endif
     switch (*from) {
     case 0:				/* Must be escaped for 'mysql' */
-      *to++= '\\';
-      *to++= '0';
+      escape= '0';
       break;
     case '\n':				/* Must be escaped for logs */
-      *to++= '\\';
-      *to++= 'n';
+      escape= 'n';
       break;
     case '\r':
-      *to++= '\\';
-      *to++= 'r';
+      escape= 'r';
       break;
     case '\\':
-      *to++= '\\';
-      *to++= '\\';
+      escape= '\\';
       break;
     case '\'':
-      *to++= '\\';
-      *to++= '\'';
+      escape= '\'';
       break;
     case '"':				/* Better safe than sorry */
-      *to++= '\\';
-      *to++= '"';
+      escape= '"';
       break;
     case '\032':			/* This gives problems on Win32 */
-      *to++= '\\';
-      *to++= 'Z';
+      escape= 'Z';
       break;
-    default:
+    }
+    if (escape)
+    {
+      if (to + 2 > to_end)
+      {
+        overflow= TRUE;
+        break;
+      }
+      *to++= '\\';
+      *to++= escape;
+    }
+    else
+    {
+      if (to + 1 > to_end)
+      {
+        overflow= TRUE;
+        break;
+      }
       *to++= *from;
     }
   }
   *to= 0;
-  return (ulong) (to - to_start);
+  return overflow ? (ulong)~0 : (ulong) (to - to_start);
 }
 
 
@@ -665,19 +712,82 @@ CHARSET_INFO *fs_character_set()
 #endif
 
 /*
-  Transforms a string into hex form.
- */
-char *bare_str_to_hex(char *to, const char *from, uint len)
+  Escape apostrophes by doubling them up
+
+  SYNOPSIS
+    escape_quotes_for_mysql()
+    charset_info        Charset of the strings
+    to                  Buffer for escaped string
+    to_length           Length of destination buffer, or 0
+    from                The string to escape
+    length              The length of the string to escape
+
+  DESCRIPTION
+    This escapes the contents of a string by doubling up any apostrophes that
+    it contains. This is used when the NO_BACKSLASH_ESCAPES SQL_MODE is in
+    effect on the server.
+
+  NOTE
+    To be consistent with escape_string_for_mysql(), to_length may be 0 to
+    mean "big enough"
+
+  RETURN VALUES
+    ~0          The escaped string did not fit in the to buffer
+    >=0         The length of the escaped string
+*/
+
+ulong escape_quotes_for_mysql(CHARSET_INFO *charset_info,
+                              char *to, ulong to_length,
+                              const char *from, ulong length)
 {
-  char *p= to;
-  uint i;
-  for (i= 0; i < len; i++, p+= 2)
+  const char *to_start= to;
+  const char *end, *to_end=to_start + (to_length ? to_length-1 : 2*length);
+  my_bool overflow= FALSE;
+#ifdef USE_MB
+  my_bool use_mb_flag= use_mb(charset_info);
+#endif
+  for (end= from + length; from < end; from++)
   {
-    /* val[i] is char. Casting to uchar helps greatly if val[i] < 0 */
-    uint tmp= (uint) (uchar) from[i];
-    p[0]= _dig_vec_upper[tmp >> 4];
-    p[1]= _dig_vec_upper[tmp & 15];
+#ifdef USE_MB
+    int tmp_length;
+    if (use_mb_flag && (tmp_length= my_ismbchar(charset_info, from, end)))
+    {
+      if (to + tmp_length > to_end)
+      {
+        overflow= TRUE;
+        break;
+      }
+      while (tmp_length--)
+	*to++= *from++;
+      from--;
+      continue;
+    }
+    /*
+      We don't have the same issue here with a non-multi-byte character being
+      turned into a multi-byte character by the addition of an escaping
+      character, because we are only escaping the ' character with itself.
+     */
+#endif
+    if (*from == '\'')
+    {
+      if (to + 2 > to_end)
+      {
+        overflow= TRUE;
+        break;
+      }
+      *to++= '\'';
+      *to++= '\'';
+    }
+    else
+    {
+      if (to + 1 > to_end)
+      {
+        overflow= TRUE;
+        break;
+      }
+      *to++= *from;
+    }
   }
-  *p= 0;
-  return p; /* pointer to end 0 of 'to' */
+  *to= 0;
+  return overflow ? (ulong)~0 : (ulong) (to - to_start);
 }
