@@ -180,7 +180,7 @@ int handle_options(int *argc, char ***argv,
 	}
 	opt_str= check_struct_option(cur_arg, key_name);
 	optend= strcend(opt_str, '=');
-	length= optend - opt_str;
+	length= (uint) (optend - opt_str);
 	if (*optend == '=')
 	  optend++;
 	else
@@ -205,7 +205,8 @@ int handle_options(int *argc, char ***argv,
 	    {
 	      if (!getopt_compare_strings(special_opt_prefix[i], opt_str,
 					  special_opt_prefix_lengths[i]) &&
-		  opt_str[special_opt_prefix_lengths[i]] == '-')
+		  (opt_str[special_opt_prefix_lengths[i]] == '-' ||
+		   opt_str[special_opt_prefix_lengths[i]] == '_'))
 	      {
 		/*
 		  We were called with a special prefix, we can reuse opt_found
@@ -322,7 +323,7 @@ int handle_options(int *argc, char ***argv,
 	  return EXIT_NO_ARGUMENT_ALLOWED;
 	}
 	value= optp->var_type & GET_ASK_ADDR ?
-	  (*getopt_get_addr)(key_name, strlen(key_name), optp) : optp->value;
+	  (*getopt_get_addr)(key_name, (uint) strlen(key_name), optp) : optp->value;
   
 	if (optp->arg_type == NO_ARG)
 	{
@@ -341,11 +342,24 @@ int handle_options(int *argc, char ***argv,
 	      --enable-'option-name'.
 	      *optend was set to '0' if one used --disable-option
 	      */
-	    my_bool tmp= (my_bool) (!optend || *optend == '1');
-	    *((my_bool*) value)= tmp;
 	    (*argc)--;
+	    if (!optend || *optend == '1' ||
+		!my_strcasecmp(&my_charset_latin1, optend, "true"))
+	      *((my_bool*) value)= (my_bool) 1;
+	    else if (*optend == '0' ||
+		     !my_strcasecmp(&my_charset_latin1, optend, "false"))
+	      *((my_bool*) value)= (my_bool) 0;
+	    else
+	    {
+	      my_getopt_error_reporter(WARNING_LEVEL,
+				       "%s: ignoring option '--%s' due to \
+invalid value '%s'\n",
+				       my_progname, optp->name, optend);
+	      continue;
+	    }
 	    get_one_option(optp->id, optp,
-			   tmp ? (char*) "1" : disabled_my_option);
+			   *((my_bool*) value) ?
+			   (char*) "1" : disabled_my_option);
 	    continue;
 	  }
 	  argument= optend;
@@ -518,7 +532,7 @@ static char *check_struct_option(char *cur_arg, char *key_name)
   */
   if (end - ptr > 1)
   {
-    uint len= ptr - cur_arg;
+    uint len= (uint) (ptr - cur_arg);
     set_if_smaller(len, FN_REFLEN-1);
     strmake(key_name, cur_arg, len);
     return ++ptr;
@@ -586,16 +600,27 @@ static int setval(const struct my_option *opts, gptr *value, char *argument,
   return 0;
 }
 
+
 /* 
-  function: findopt
+  Find option
 
-  Arguments: opt_pattern, length of opt_pattern, opt_struct, first found
-  name (ffname)
+  SYNOPSIS
+    findopt()
+    optpat	Prefix of option to find (with - or _)
+    length	Length of optpat
+    opt_res	Options
+    ffname	Place for pointer to first found name
 
-  Go through all options in the my_option struct. Return number
-  of options found that match the pattern and in the argument
-  list the option found, if any. In case of ambiguous option, store
-  the name in ffname argument
+  IMPLEMENTATION
+    Go through all options in the my_option struct. Return number
+    of options found that match the pattern and in the argument
+    list the option found, if any. In case of ambiguous option, store
+    the name in ffname argument
+
+    RETURN
+    0    No matching options
+    #   Number of matching options
+        ffname points to first matching option
 */
 
 static int findopt(char *optpat, uint length,
@@ -610,12 +635,21 @@ static int findopt(char *optpat, uint length,
     if (!getopt_compare_strings(opt->name, optpat, length)) /* match found */
     {
       (*opt_res)= opt;
-      if (!count)
-	*ffname= (char *) opt->name;	/* We only need to know one prev */
       if (!opt->name[length])		/* Exact match */
 	return 1;
-      if (!count || strcmp(*ffname, opt->name)) /* Don't count synonyms */
+      if (!count)
+      {
+	count= 1;
+	*ffname= (char *) opt->name;	/* We only need to know one prev */
+      }
+      else if (strcmp(*ffname, opt->name))
+      {
+	/*
+	  The above test is to not count same option twice
+	  (see mysql.cc, option "help")
+	*/
 	count++;
+      }
     }
   }
   return count;
@@ -689,10 +723,10 @@ static longlong getopt_ll(char *arg, const struct my_option *optp, int *err)
   ulonglong block_size= (optp->block_size ? (ulonglong) optp->block_size : 1L);
   
   num= eval_num_suffix(arg, err, (char*) optp->name);
-  if (num > 0 && (ulonglong) num > (ulonglong) (ulong) optp->max_value &&
+  if (num > 0 && (ulonglong) num > (ulonglong) optp->max_value &&
       optp->max_value) /* if max value is not set -> no upper limit */
-    num= (longlong) (ulong) optp->max_value;
-  num= ((num - (longlong) optp->sub_size) / block_size);
+    num= (ulonglong) optp->max_value;
+  num= ((num - optp->sub_size) / block_size);
   num= (longlong) (num * block_size);
   return max(num, optp->min_value);
 }
@@ -832,7 +866,7 @@ void my_print_help(const struct my_option *options)
     if (strlen(optp->name))
     {
       printf("--%s", optp->name);
-      col+= 2 + strlen(optp->name);
+      col+= 2 + (uint) strlen(optp->name);
       if ((optp->var_type & GET_TYPE_MASK) == GET_STR ||
 	  (optp->var_type & GET_TYPE_MASK) == GET_STR_ALLOC)
       {
@@ -903,7 +937,7 @@ void my_print_variables(const struct my_option *options)
     if (value)
     {
       printf("%s", optp->name);
-      length= strlen(optp->name);
+      length= (uint) strlen(optp->name);
       for (; length < name_space; length++)
 	putchar(' ');
       switch ((optp->var_type & GET_TYPE_MASK)) {
