@@ -17,7 +17,7 @@
 /* This makes a wrapper for mutex handling to make it easier to debug mutex */
 
 #include <my_global.h>
-#if defined(HAVE_LINUXTHREADS) && !defined (__USE_UNIX98)
+#if defined(TARGET_OS_LINUX) && !defined (__USE_UNIX98)
 #define __USE_UNIX98			/* To get rw locks under Linux */
 #endif
 #if defined(THREAD) && defined(SAFE_MUTEX)
@@ -54,12 +54,15 @@ void safe_mutex_global_init(void)
 
 int safe_mutex_init(safe_mutex_t *mp,
 		    const pthread_mutexattr_t *attr __attribute__((unused)),
-		    const char *file __attribute__((unused)),
-		    uint line __attribute__((unused)))
+		    const char *file,
+		    uint line)
 {
   bzero((char*) mp,sizeof(*mp));
   pthread_mutex_init(&mp->global,MY_MUTEX_INIT_ERRCHK);
   pthread_mutex_init(&mp->mutex,attr);
+  /* Mark that mutex is initialized */
+  mp->file= file;
+  mp->line= line;
 
 #ifdef SAFE_MUTEX_DETECT_DESTROY
   /*
@@ -70,7 +73,7 @@ int safe_mutex_init(safe_mutex_t *mp,
   {
     struct st_safe_mutex_info_t *info =mp->info;
 
-    info->init_file= (char *) file;
+    info->init_file= file;
     info->init_line= line;
     info->prev= NULL;
     info->next= NULL;
@@ -92,6 +95,15 @@ int safe_mutex_init(safe_mutex_t *mp,
 int safe_mutex_lock(safe_mutex_t *mp,const char *file, uint line)
 {
   int error;
+  if (!mp->file)
+  {
+    fprintf(stderr,
+	    "safe_mutex: Trying to lock unitialized mutex at %s, line %d\n",
+	    file, line);
+    fflush(stderr);
+    abort();
+  }
+
   pthread_mutex_lock(&mp->global);
   if (mp->count > 0 && pthread_equal(pthread_self(),mp->thread))
   {
@@ -109,6 +121,7 @@ int safe_mutex_lock(safe_mutex_t *mp,const char *file, uint line)
     fflush(stderr);
     abort();
   }
+  mp->thread= pthread_self();
   if (mp->count++)
   {
     fprintf(stderr,"safe_mutex: Error in thread libray: Got mutex at %s, \
@@ -116,8 +129,7 @@ line %d more than 1 time\n", file,line);
     fflush(stderr);
     abort();
   }
-  mp->thread=pthread_self();
-  mp->file= (char*) file;
+  mp->file= file;
   mp->line=line;
   pthread_mutex_unlock(&mp->global);
   return error;
@@ -142,6 +154,7 @@ int safe_mutex_unlock(safe_mutex_t *mp,const char *file, uint line)
     fflush(stderr);
     abort();
   }
+  mp->thread= 0;
   mp->count--;
 #ifdef __WIN__
   pthread_mutex_unlock(&mp->mutex);
@@ -195,16 +208,16 @@ int safe_cond_wait(pthread_cond_t *cond, safe_mutex_t *mp, const char *file,
     fflush(stderr);
     abort();
   }
+  mp->thread=pthread_self();
   if (mp->count++)
   {
     fprintf(stderr,
-	    "safe_mutex:  Count was %d in thread %lx when locking mutex at %s, line %d\n",
+	    "safe_mutex:  Count was %d in thread 0x%lx when locking mutex at %s, line %d\n",
 	    mp->count-1, my_thread_id(), file, line);
     fflush(stderr);
     abort();
   }
-  mp->thread=pthread_self();
-  mp->file= (char*) file;
+  mp->file= file;
   mp->line=line;
   pthread_mutex_unlock(&mp->global);
   return error;
@@ -227,22 +240,22 @@ int safe_cond_timedwait(pthread_cond_t *cond, safe_mutex_t *mp,
   pthread_mutex_unlock(&mp->global);
   error=pthread_cond_timedwait(cond,&mp->mutex,abstime);
 #ifdef EXTRA_DEBUG
-  if (error && (error != EINTR && error != ETIMEDOUT))
+  if (error && (error != EINTR && error != ETIMEDOUT && error != ETIME))
   {
     fprintf(stderr,"safe_mutex: Got error: %d (%d) when doing a safe_mutex_timedwait at %s, line %d\n", error, errno, file, line);
   }
 #endif
   pthread_mutex_lock(&mp->global);
+  mp->thread=pthread_self();
   if (mp->count++)
   {
     fprintf(stderr,
-	    "safe_mutex:  Count was %d in thread %lx when locking mutex at %s, line %d (error: %d (%d))\n",
+	    "safe_mutex:  Count was %d in thread 0x%lx when locking mutex at %s, line %d (error: %d (%d))\n",
 	    mp->count-1, my_thread_id(), file, line, error, error);
     fflush(stderr);
     abort();
   }
-  mp->thread=pthread_self();
-  mp->file= (char*) file;
+  mp->file= file;
   mp->line=line;
   pthread_mutex_unlock(&mp->global);
   return error;
@@ -252,6 +265,14 @@ int safe_cond_timedwait(pthread_cond_t *cond, safe_mutex_t *mp,
 int safe_mutex_destroy(safe_mutex_t *mp, const char *file, uint line)
 {
   int error=0;
+  if (!mp->file)
+  {
+    fprintf(stderr,
+	    "safe_mutex: Trying to destroy unitialized mutex at %s, line %d\n",
+	    file, line);
+    fflush(stderr);
+    abort();
+  }
   if (mp->count != 0)
   {
     fprintf(stderr,"safe_mutex: Trying to destroy a mutex that was locked at %s, line %d at %s, line %d\n",
@@ -268,6 +289,7 @@ int safe_mutex_destroy(safe_mutex_t *mp, const char *file, uint line)
   if (pthread_mutex_destroy(&mp->mutex))
     error=1;
 #endif
+  mp->file= 0;					/* Mark destroyed */
 
 #ifdef SAFE_MUTEX_DETECT_DESTROY
   if (mp->info)
