@@ -17,31 +17,49 @@
 /*
   Code for handling red-black (balanced) binary trees.
   key in tree is allocated accrding to following:
-  1) If free_element function is given to init_tree or size < 0 then tree
-     will not allocate keys and only a pointer to each key is saved in tree.
-     key_sizes must be 0 to init and search.
+
+  1) If size < 0 then tree will not allocate keys and only a pointer to
+     each key is saved in tree.
+     compare and search functions uses and returns key-pointer
+
+  2) If size == 0 then there are two options:
+       - key_size != 0 to tree_insert: The key will be stored in the tree.
+       - key_size == 0 to tree_insert:  A pointer to the key is stored.
      compare and search functions uses and returns key-pointer.
-  2) if key_size is given to init_tree then each node will continue the
+
+  3) if key_size is given to init_tree then each node will continue the
      key and calls to insert_key may increase length of key.
      if key_size > sizeof(pointer) and key_size is a multiple of 8 (double
      allign) then key will be put on a 8 alligned adress. Else
      the key will be on adress (element+1). This is transparent for user
      compare and search functions uses a pointer to given key-argument.
-  3) If init_tree - keysize is 0 then key_size must be given to tree_insert
-     and tree_insert will alloc space for key.
-     compare and search functions uses a pointer to given key-argument.
+
+  - If you use a free function for tree-elements and you are freeing
+    the element itself, you should use key_size = 0 to init_tree and
+    tree_search
 
   The actual key in TREE_ELEMENT is saved as a pointer or after the
   TREE_ELEMENT struct.
   If one uses only pointers in tree one can use tree_set_pointer() to
   change address of data.
-  Copyright Monty Program KB.
-  By monty.
+
+  Implemented by monty.
+*/
+
+/*
+  NOTE:
+  tree->compare function should be ALWAYS called as
+    (*tree->compare)(custom_arg, ELEMENT_KEY(tree,element), key)
+  and not other way around, as
+    (*tree->compare)(custom_arg, key, ELEMENT_KEY(tree,element))
+
+  ft_boolean_search.c (at least) relies on that.
 */
 
 #include "mysys_priv.h"
 #include <m_string.h>
 #include <my_tree.h>
+#include "my_base.h"
 
 #define BLACK		1
 #define RED		0
@@ -71,7 +89,7 @@ void init_tree(TREE *tree, uint default_alloc_size, uint memory_limit,
 	       tree_element_free free_element, void *custom_arg)
 {
   DBUG_ENTER("init_tree");
-  DBUG_PRINT("enter",("tree: %lx  size: %d",tree,size));
+  DBUG_PRINT("enter",("tree: 0x%lx  size: %d",tree,size));
 
   if (default_alloc_size < DEFAULT_ALLOC_SIZE)
     default_alloc_size= DEFAULT_ALLOC_SIZE;
@@ -87,6 +105,7 @@ void init_tree(TREE *tree, uint default_alloc_size, uint memory_limit,
   tree->custom_arg = custom_arg;
   tree->null_element.colour=BLACK;
   tree->null_element.left=tree->null_element.right=0;
+  tree->flag= 0;
   if (!free_element && size >= 0 &&
       ((uint) size <= sizeof(void*) || ((uint) size & (sizeof(void*)-1))))
   {
@@ -118,7 +137,7 @@ void init_tree(TREE *tree, uint default_alloc_size, uint memory_limit,
 static void free_tree(TREE *tree, myf free_flags)
 {
   DBUG_ENTER("free_tree");
-  DBUG_PRINT("enter",("tree: %lx",tree));
+  DBUG_PRINT("enter",("tree: 0x%lx",tree));
 
   if (tree->root)				/* If initialized */
   {
@@ -151,8 +170,8 @@ void delete_tree(TREE* tree)
 
 void reset_tree(TREE* tree)
 {
+  /* do not free mem_root, just mark blocks as free */
   free_tree(tree, MYF(MY_MARK_BLOCKS_FREE));
-  /* do not my_free() mem_root if applicable, just mark blocks as free */
 }
 
 
@@ -169,12 +188,17 @@ static void delete_tree_element(TREE *tree, TREE_ELEMENT *element)
   }
 }
 
-	/* Code for insert, search and delete of elements */
-	/*   parent[0] = & parent[-1][0]->left ||
-	     parent[0] = & parent[-1][0]->right */
 
+/*
+  insert, search and delete of elements
 
-TREE_ELEMENT *tree_insert(TREE *tree, void *key, uint key_size)
+  The following should be true:
+    parent[0] = & parent[-1][0]->left ||
+    parent[0] = & parent[-1][0]->right
+*/
+
+TREE_ELEMENT *tree_insert(TREE *tree, void *key, uint key_size, 
+                          void* custom_arg)
 {
   int cmp;
   TREE_ELEMENT *element,***parent;
@@ -184,8 +208,8 @@ TREE_ELEMENT *tree_insert(TREE *tree, void *key, uint key_size)
   for (;;)
   {
     if (element == &tree->null_element ||
-	(cmp=(*tree->compare)(tree->custom_arg,
-			      ELEMENT_KEY(tree,element),key)) == 0)
+	(cmp = (*tree->compare)(custom_arg, ELEMENT_KEY(tree,element),
+                                key)) == 0)
       break;
     if (cmp < 0)
     {
@@ -205,15 +229,14 @@ TREE_ELEMENT *tree_insert(TREE *tree, void *key, uint key_size)
                            && tree->allocated > tree->memory_limit)
     {
       reset_tree(tree);
-      return tree_insert(tree, key, key_size);
+      return tree_insert(tree, key, key_size, custom_arg);
     }
 
     key_size+=tree->size_of_element;
     if (tree->with_delete)
       element=(TREE_ELEMENT *) my_malloc(alloc_size, MYF(MY_WME));
     else
-      element=(TREE_ELEMENT *)
-	alloc_root(&tree->mem_root,alloc_size);
+      element=(TREE_ELEMENT *) alloc_root(&tree->mem_root,alloc_size);
     if (!element)
       return(NULL);
     **parent=element;
@@ -231,18 +254,24 @@ TREE_ELEMENT *tree_insert(TREE *tree, void *key, uint key_size)
     }
     else
       memcpy((byte*) element+tree->offset_to_key,key,(size_t) key_size);
-    element->count=1;				/* May give warning in purify */
+    element->count=1;			/* May give warning in purify */
     tree->elements_in_tree++;
-    rb_insert(tree,parent,element);		/* rebalance tree */
+    rb_insert(tree,parent,element);	/* rebalance tree */
   }
   else
+  {
+    if (tree->flag & TREE_NO_DUPS)
+      return(NULL);
     element->count++;
+    /* Avoid a wrap over of the count. */
+    if (! element->count)
+      element->count--;
+  }
   DBUG_EXECUTE("check_tree", test_rb_tree(tree->root););
   return element;
 }
 
-
-int tree_delete(TREE *tree, void *key)
+int tree_delete(TREE *tree, void *key, uint key_size, void *custom_arg)
 {
   int cmp,remove_colour;
   TREE_ELEMENT *element,***parent, ***org_parent, *nod;
@@ -255,8 +284,8 @@ int tree_delete(TREE *tree, void *key)
   {
     if (element == &tree->null_element)
       return 1;				/* Was not in tree */
-    if ((cmp=(*tree->compare)(tree->custom_arg,
-			      ELEMENT_KEY(tree,element),key)) == 0)
+    if ((cmp = (*tree->compare)(custom_arg, ELEMENT_KEY(tree,element),
+                                key)) == 0)
       break;
     if (cmp < 0)
     {
@@ -297,13 +326,14 @@ int tree_delete(TREE *tree, void *key)
     rb_delete_fixup(tree,parent);
   if (tree->free)
     (*tree->free)(ELEMENT_KEY(tree,element), free_free, tree->custom_arg);
+  tree->allocated-= sizeof(TREE_ELEMENT) + tree->size_of_element + key_size;
   my_free((gptr) element,MYF(0));
   tree->elements_in_tree--;
   return 0;
 }
 
 
-void *tree_search(TREE *tree, void *key)
+void *tree_search(TREE *tree, void *key, void *custom_arg)
 {
   int cmp;
   TREE_ELEMENT *element=tree->root;
@@ -312,8 +342,8 @@ void *tree_search(TREE *tree, void *key)
   {
     if (element == &tree->null_element)
       return (void*) 0;
-    if ((cmp=(*tree->compare)(tree->custom_arg,
-			      ELEMENT_KEY(tree,element),key)) == 0)
+    if ((cmp = (*tree->compare)(custom_arg, ELEMENT_KEY(tree,element),
+                                key)) == 0)
       return ELEMENT_KEY(tree,element);
     if (cmp < 0)
       element=element->right;
@@ -322,6 +352,176 @@ void *tree_search(TREE *tree, void *key)
   }
 }
 
+void *tree_search_key(TREE *tree, const void *key, 
+                      TREE_ELEMENT **parents, TREE_ELEMENT ***last_pos,
+                      enum ha_rkey_function flag, void *custom_arg)
+{
+  int cmp;
+  TREE_ELEMENT *element= tree->root;
+  TREE_ELEMENT **last_left_step_parent= NULL, **last_right_step_parent= NULL;
+  TREE_ELEMENT **last_equal_element= NULL;
+
+/* 
+  TODO: support for HA_READ_KEY_OR_PREV, HA_READ_PREFIX flags if needed.
+*/
+
+  *parents = &tree->null_element;
+  while (element != &tree->null_element)
+  {
+    *++parents= element;
+    if ((cmp= (*tree->compare)(custom_arg, ELEMENT_KEY(tree, element), 
+			       key)) == 0)
+    {
+      switch (flag) {
+      case HA_READ_KEY_EXACT:
+      case HA_READ_KEY_OR_NEXT:
+      case HA_READ_BEFORE_KEY:
+	last_equal_element= parents;
+	cmp= 1;
+	break;
+      case HA_READ_AFTER_KEY:
+	cmp= -1;
+	break;
+      case HA_READ_PREFIX_LAST:
+      case HA_READ_PREFIX_LAST_OR_PREV:
+	last_equal_element= parents;
+	cmp= -1;
+	break;
+      default:
+	return NULL;
+      }
+    }
+    if (cmp < 0) /* element < key */
+    {
+      last_right_step_parent= parents;
+      element= element->right;
+    }
+    else
+    {
+      last_left_step_parent= parents;
+      element= element->left;
+    }
+  }
+  switch (flag) {
+  case HA_READ_KEY_EXACT:
+  case HA_READ_PREFIX_LAST:
+    *last_pos= last_equal_element;
+    break;
+  case HA_READ_KEY_OR_NEXT:
+    *last_pos= last_equal_element ? last_equal_element : last_left_step_parent;
+    break;
+  case HA_READ_AFTER_KEY:
+    *last_pos= last_left_step_parent;
+    break;
+  case HA_READ_PREFIX_LAST_OR_PREV:
+    *last_pos= last_equal_element ? last_equal_element : last_right_step_parent;
+    break;
+  case HA_READ_BEFORE_KEY:
+    *last_pos= last_right_step_parent;
+    break;
+  default:
+    return NULL;
+  }
+  return *last_pos ? ELEMENT_KEY(tree, **last_pos) : NULL;
+}
+
+/* 
+  Search first (the most left) or last (the most right) tree element 
+*/
+void *tree_search_edge(TREE *tree, TREE_ELEMENT **parents, 
+		       TREE_ELEMENT ***last_pos, int child_offs)
+{
+  TREE_ELEMENT *element= tree->root;
+  
+  *parents= &tree->null_element;
+  while (element != &tree->null_element)
+  {
+    *++parents= element;
+    element= ELEMENT_CHILD(element, child_offs);
+  }
+  *last_pos= parents;
+  return **last_pos != &tree->null_element ? 
+    ELEMENT_KEY(tree, **last_pos) : NULL;
+}
+
+void *tree_search_next(TREE *tree, TREE_ELEMENT ***last_pos, int l_offs, 
+                       int r_offs)
+{
+  TREE_ELEMENT *x= **last_pos;
+  
+  if (ELEMENT_CHILD(x, r_offs) != &tree->null_element)
+  {
+    x= ELEMENT_CHILD(x, r_offs);
+    *++*last_pos= x;
+    while (ELEMENT_CHILD(x, l_offs) != &tree->null_element)
+    {
+      x= ELEMENT_CHILD(x, l_offs);
+      *++*last_pos= x;
+    }
+    return ELEMENT_KEY(tree, x);
+  }
+  else
+  {
+    TREE_ELEMENT *y= *--*last_pos;
+    while (y != &tree->null_element && x == ELEMENT_CHILD(y, r_offs))
+    {
+      x= y;
+      y= *--*last_pos;
+    }
+    return y == &tree->null_element ? NULL : ELEMENT_KEY(tree, y);
+  }
+}
+
+/*
+  Expected that tree is fully balanced
+  (each path from root to leaf has the same length)
+*/
+ha_rows tree_record_pos(TREE *tree, const void *key, 
+			enum ha_rkey_function flag, void *custom_arg)
+{
+  int cmp;
+  TREE_ELEMENT *element= tree->root;
+  double left= 1;
+  double right= tree->elements_in_tree;
+
+  while (element != &tree->null_element)
+  {
+    if ((cmp= (*tree->compare)(custom_arg, ELEMENT_KEY(tree, element), 
+			       key)) == 0)
+    {
+      switch (flag) {
+      case HA_READ_KEY_EXACT:
+      case HA_READ_BEFORE_KEY:
+        cmp= 1;
+        break;
+      case HA_READ_AFTER_KEY:
+        cmp= -1;
+        break;
+      default:
+        return HA_POS_ERROR;
+      }
+    }
+    if (cmp < 0) /* element < key */
+    {
+      element= element->right;
+      left= (left + right) / 2;
+    }
+    else
+    {
+      element= element->left;
+      right= (left + right) / 2;
+    }
+  }
+  switch (flag) {
+  case HA_READ_KEY_EXACT:
+  case HA_READ_BEFORE_KEY:
+    return (ha_rows) right;
+  case HA_READ_AFTER_KEY:
+    return (ha_rows) left;
+  default:
+    return HA_POS_ERROR;
+  }
+}
 
 int tree_walk(TREE *tree, tree_walk_action action, void *argument, TREE_WALK visit)
 {
