@@ -41,10 +41,7 @@ public:
   STATIC_CONST( SignalLength = 4 );
 
   enum ErrorCode {
-    ScanfragStateError = 1,
-    ScanfragTimeout = 2,
     GCPStopDetected = 3,
-    StartInProgressError = 4,
     CopyFragRefError = 5,
     TestStopOnError = 6
   };
