@@ -42,6 +42,8 @@ SocketServer::~SocketServer() {
     delete m_sessions[i].m_session;
   }
   for(i = 0; i<m_services.size(); i++){
+    if(m_services[i].m_socket)
+      NDB_CLOSE_SOCKET(m_services[i].m_socket);
     delete m_services[i].m_service;
   }
 }
@@ -84,15 +86,15 @@ SocketServer::tryBind(unsigned short port, const char * intface) {
 
 bool
 SocketServer::setup(SocketServer::Service * service, 
-		    unsigned short port, 
+		    unsigned short * port,
 		    const char * intface){
   DBUG_ENTER("SocketServer::setup");
-  DBUG_PRINT("enter",("interface=%s, port=%d", intface, port));
+  DBUG_PRINT("enter",("interface=%s, port=%u", intface, *port));
   struct sockaddr_in servaddr;
   memset(&servaddr, 0, sizeof(servaddr));
   servaddr.sin_family = AF_INET;
   servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
-  servaddr.sin_port = htons(port);
+  servaddr.sin_port = htons(*port);
   
   if(intface != 0){
     if(Ndb_getInAddr(&servaddr.sin_addr, intface))
@@ -123,7 +125,17 @@ SocketServer::setup(SocketServer::Service * service,
     NDB_CLOSE_SOCKET(sock);
     DBUG_RETURN(false);
   }
-  
+
+  /* Get the port we bound to */
+  SOCKET_SIZE_TYPE sock_len = sizeof(servaddr);
+  if(getsockname(sock,(struct sockaddr*)&servaddr,&sock_len)<0) {
+    ndbout_c("An error occurred while trying to find out what"
+	     " port we bound to. Error: %s",strerror(errno));
+    NDB_CLOSE_SOCKET(sock);
+    DBUG_RETURN(false);
+  }
+
+  DBUG_PRINT("info",("bound to %u",ntohs(servaddr.sin_port)));
   if (listen(sock, m_maxSessions > 32 ? 32 : m_maxSessions) == -1){
     DBUG_PRINT("error",("listen() - %d - %s",
 			errno, strerror(errno)));
@@ -135,6 +147,9 @@ SocketServer::setup(SocketServer::Service * service,
   i.m_socket = sock;
   i.m_service = service;
   m_services.push_back(i);
+
+  *port = ntohs(servaddr.sin_port);
+
   DBUG_RETURN(true);
 }
 
@@ -169,9 +184,12 @@ SocketServer::doAccept(){
 	SessionInstance s;
 	s.m_service = si.m_service;
 	s.m_session = si.m_service->newSession(childSock);
-	if(s.m_session != 0){
+	if(s.m_session != 0)
+	{
+	  m_session_mutex.lock();
 	  m_sessions.push_back(s);
 	  startSession(m_sessions.back());
+	  m_session_mutex.unlock();
 	}
 	
 	continue;
@@ -225,10 +243,13 @@ void
 SocketServer::doRun(){
 
   while(!m_stopThread){
-    checkSessions();
+    m_session_mutex.lock();
+    checkSessionsImpl();
     if(m_sessions.size() < m_maxSessions){
+      m_session_mutex.unlock();
       doAccept();
     } else {
+      m_session_mutex.unlock();
       NdbSleep_MilliSleep(200);
     }
   }
@@ -261,17 +282,30 @@ transfer(NDB_SOCKET_TYPE sock){
 void
 SocketServer::foreachSession(void (*func)(SocketServer::Session*, void *), void *data)
 {
+  m_session_mutex.lock();
   for(int i = m_sessions.size() - 1; i >= 0; i--){
     (*func)(m_sessions[i].m_session, data);
   }
-  checkSessions();
+  m_session_mutex.unlock();
 }
 
 void
-SocketServer::checkSessions(){
-  for(int i = m_sessions.size() - 1; i >= 0; i--){
-    if(m_sessions[i].m_session->m_stopped){
-      if(m_sessions[i].m_thread != 0){
+SocketServer::checkSessions()
+{
+  m_session_mutex.lock();
+  checkSessionsImpl();
+  m_session_mutex.unlock();  
+}
+
+void
+SocketServer::checkSessionsImpl()
+{
+  for(int i = m_sessions.size() - 1; i >= 0; i--)
+  {
+    if(m_sessions[i].m_session->m_stopped)
+    {
+      if(m_sessions[i].m_thread != 0)
+      {
 	void* ret;
 	NdbThread_WaitFor(m_sessions[i].m_thread, &ret);
 	NdbThread_Destroy(&m_sessions[i].m_thread);
@@ -286,19 +320,26 @@ SocketServer::checkSessions(){
 void
 SocketServer::stopSessions(bool wait){
   int i;
+  m_session_mutex.lock();
   for(i = m_sessions.size() - 1; i>=0; i--)
   {
     m_sessions[i].m_session->stopSession();
     m_sessions[i].m_session->m_stop = true; // to make sure
   }
+  m_session_mutex.unlock();
+  
   for(i = m_services.size() - 1; i>=0; i--)
     m_services[i].m_service->stopSessions();
   
   if(wait){
+    m_session_mutex.lock();
     while(m_sessions.size() > 0){
-      checkSessions();
+      checkSessionsImpl();
+      m_session_mutex.unlock();
       NdbSleep_MilliSleep(100);
+      m_session_mutex.lock();
     }
+    m_session_mutex.unlock();
   }
 }
 
@@ -314,11 +355,18 @@ sessionThread_C(void* _sc){
     return 0;
   }
   
-  if(!si->m_stop){
-    si->m_stopped = false;
-    si->runSession();
-  } else {
-    NDB_CLOSE_SOCKET(si->m_socket);
+  /**
+   * may have m_stopped set if we're transforming a mgm
+   * connection into a transporter connection.
+   */
+  if(!si->m_stopped)
+  {
+    if(!si->m_stop){
+      si->m_stopped = false;
+      si->runSession();
+    } else {
+      NDB_CLOSE_SOCKET(si->m_socket);
+    }
   }
   
   si->m_stopped = true;
@@ -326,4 +374,4 @@ sessionThread_C(void* _sc){
 }
 
 template class MutexVector<SocketServer::ServiceInstance>;
-template class MutexVector<SocketServer::SessionInstance>;
+template class Vector<SocketServer::SessionInstance>;
