@@ -26,6 +26,7 @@
 #include <pc.hpp>
 #include <ArrayList.hpp>
 #include <DLHashTable.hpp>
+#include <DLFifoList.hpp>
 #include <CArray.hpp>
 #include <KeyTable2.hpp>
 #include <SimulatedBlock.hpp>
@@ -50,6 +51,7 @@
 #include <signaldata/CreateTrig.hpp>
 #include <signaldata/DropTrig.hpp>
 #include <signaldata/AlterTrig.hpp>
+#include <signaldata/DictLock.hpp>
 #include "SchemaFile.hpp"
 #include <blocks/mutexes.hpp>
 #include <SafeCounter.hpp>
@@ -63,6 +65,7 @@
 /*--------------------------------------------------------------*/
 #define ZPACK_TABLE_INTO_PAGES 0
 #define ZSEND_GET_TAB_RESPONSE 3
+#define ZDICT_LOCK_POLL 4
 
 
 /*--------------------------------------------------------------*/
@@ -78,7 +81,8 @@
 /*--------------------------------------------------------------*/
 // Page constants
 /*--------------------------------------------------------------*/
-#define ZALLOCATE 1 //Variable number of page for NDBFS
+#define ZBAT_SCHEMA_FILE 0 //Variable number of page for NDBFS
+#define ZBAT_TABLE_FILE 1 //Variable number of page for NDBFS
 #define ZPAGE_HEADER_SIZE 32
 #define ZPOS_PAGE_SIZE 16
 #define ZPOS_CHECKSUM 17
@@ -92,7 +96,7 @@
 #define ZSIZE_OF_PAGES_IN_WORDS 8192
 #define ZLOG_SIZE_OF_PAGES_IN_WORDS 13
 #define ZMAX_PAGES_OF_TABLE_DEFINITION 8
-#define ZNUMBER_OF_PAGES (2 * ZMAX_PAGES_OF_TABLE_DEFINITION + 2)
+#define ZNUMBER_OF_PAGES (ZMAX_PAGES_OF_TABLE_DEFINITION + 1)
 #define ZNO_OF_FRAGRECORD 5
 
 /*--------------------------------------------------------------*/
@@ -130,6 +134,10 @@ public:
    * on disk.  Index trigger ids are volatile.
    */
   struct TableRecord : public MetaData::Table {
+    Uint32 maxRowsLow;
+    Uint32 maxRowsHigh;
+    Uint32 minRowsLow;
+    Uint32 minRowsHigh;
     /****************************************************
      *    Support variables for table handling
      ****************************************************/
@@ -212,7 +220,9 @@ public:
       IL_CREATED_TC = 1 << 0    // created in TC
     };
     Uint32 indexLocal;
-
+    
+    Uint32 noOfNullBits;
+    
     inline bool equal(TableRecord & rec) const {
       return strcmp(tableName, rec.tableName) == 0;
     }
@@ -426,6 +436,12 @@ public:
   typedef Ptr<PageRecord> PageRecordPtr;
   CArray<PageRecord> c_pageRecordArray;
 
+  struct SchemaPageRecord {
+    Uint32 word[NDB_SF_PAGE_SIZE_IN_WORDS];
+  };
+
+  CArray<SchemaPageRecord> c_schemaPageRecordArray;
+
   /**
    * A page for create index table signal.
    */
@@ -458,13 +474,11 @@ private:
   void execSCHEMA_INFOCONF(Signal* signal);
   void execREAD_NODESCONF(Signal* signal);
   void execFSCLOSECONF(Signal* signal);
-  void execFSCLOSEREF(Signal* signal);
   void execFSOPENCONF(Signal* signal);
   void execFSOPENREF(Signal* signal);
   void execFSREADCONF(Signal* signal);
   void execFSREADREF(Signal* signal);
   void execFSWRITECONF(Signal* signal);
-  void execFSWRITEREF(Signal* signal);
   void execNDB_STTOR(Signal* signal);
   void execREAD_CONFIG_REQ(Signal* signal);
   void execSTTOR(Signal* signal);
@@ -578,6 +592,10 @@ private:
   void execALTER_TAB_REQ(Signal* signal);
   void execALTER_TAB_REF(Signal* signal);
   void execALTER_TAB_CONF(Signal* signal);
+  bool check_ndb_versions() const;
+
+  void execDICT_LOCK_REQ(Signal* signal);
+  void execDICT_UNLOCK_ORD(Signal* signal);
 
   /*
    *  2.4 COMMON STORED VARIABLES
@@ -654,16 +672,20 @@ private:
   struct ReadSchemaRecord {
     /** Page Id of schema page */
     Uint32 pageId;
+    /** First page to read */
+    Uint32 firstPage;
+    /** Number of pages to read */
+    Uint32 noOfPages;
     /** State, indicates from where it was called */
     enum SchemaReadState {
       IDLE = 0,
-      INITIAL_READ = 1
+      INITIAL_READ_HEAD = 1,
+      INITIAL_READ = 2
     };
     SchemaReadState schemaReadState;
   };
   ReadSchemaRecord c_readSchemaRecord;
 
-private:
   /**
    * This record stores all the state needed 
    * when a schema file is being written to disk
@@ -671,6 +693,12 @@ private:
   struct WriteSchemaRecord {
     /** Page Id of schema page */
     Uint32 pageId;
+    /** Rewrite entire file */
+    Uint32 newFile;
+    /** First page to write */
+    Uint32 firstPage;
+    /** Number of pages to write */
+    Uint32 noOfPages;
     /** Schema Files Handled, local state variable */
     Uint32 noOfSchemaFilesHandled;
 
@@ -751,21 +779,33 @@ private:
    * Word 4: Currently zero
    ****************************************************************************/
   struct SchemaRecord {
-    /**    Schema page       */
+    /**    Schema file first page (0)   */
     Uint32 schemaPage;
 
-    /**    Old Schema page (used at node restart)   */
+    /**    Old Schema file first page (used at node restart)    */
     Uint32 oldSchemaPage;
     
     Callback m_callback;
   };
   SchemaRecord c_schemaRecord;
 
-  void initSchemaFile(SchemaFile *, Uint32 sz);
-  void computeChecksum(SchemaFile *);
-  bool validateChecksum(const SchemaFile *);
-  SchemaFile::TableEntry * getTableEntry(void * buf, Uint32 tableId, 
-					 bool allowTooBig = false);
+  /*
+   * Schema file, list of schema pages.  Use an array until a pool
+   * exists and NDBFS interface can use it.
+   */
+  struct XSchemaFile {
+    SchemaFile* schemaPage;
+    Uint32 noOfPages;
+  };
+  // 0-normal 1-old
+  XSchemaFile c_schemaFile[2];
+
+  void initSchemaFile(XSchemaFile *, Uint32 firstPage, Uint32 lastPage,
+                      bool initEntries);
+  void resizeSchemaFile(XSchemaFile * xsf, Uint32 noOfPages);
+  void computeChecksum(XSchemaFile *, Uint32 pageNo);
+  bool validateChecksum(const XSchemaFile *);
+  SchemaFile::TableEntry * getTableEntry(XSchemaFile *, Uint32 tableId);
 
   Uint32 computeChecksum(const Uint32 * src, Uint32 len);
 
@@ -787,12 +827,43 @@ private:
   // State variables
   /* ----------------------------------------------------------------------- */
   
+#ifndef ndb_dbdict_log_block_state
   enum BlockState {
     BS_IDLE = 0,
     BS_CREATE_TAB = 1,
     BS_BUSY = 2,
-    BS_NODE_FAILURE = 3
+    BS_NODE_FAILURE = 3,
+    BS_NODE_RESTART = 4
   };
+#else // quick hack to log changes
+  enum {
+    BS_IDLE = 0,
+    BS_CREATE_TAB = 1,
+    BS_BUSY = 2,
+    BS_NODE_FAILURE = 3,
+    BS_NODE_RESTART = 4
+  };
+  struct BlockState;
+  friend struct BlockState;
+  struct BlockState {
+    BlockState() :
+      m_value(BS_IDLE) {
+    }
+    BlockState(int value) :
+      m_value(value) {
+    }
+    operator int() const {
+      return m_value;
+    }
+    BlockState& operator=(const BlockState& bs) {
+      Dbdict* dict = (Dbdict*)globalData.getBlock(DBDICT);
+      dict->infoEvent("DICT: bs %d->%d", m_value, bs.m_value);
+      m_value = bs.m_value;
+      return *this;
+    }
+    int m_value;
+  };
+#endif
   BlockState c_blockState;
 
   struct PackTable {
@@ -925,7 +996,8 @@ private:
     enum {
       RF_LOCAL = 1 << 0,        // create on local node only
       RF_NOBUILD = 1 << 1,      // no need to build index
-      RF_NOTCTRIGGER = 1 << 2   // alter trigger: no trigger in TC
+      RF_NOTCTRIGGER = 1 << 2,  // alter trigger: no trigger in TC
+      RF_FORCE = 1 << 4         // force drop
     };
   };
 
@@ -945,6 +1017,7 @@ private:
     CreateIndxReq::RequestType m_requestType;
     Uint32 m_requestFlag;
     // error info
+    CreateIndxRef::ErrorCode m_lastError;
     CreateIndxRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -956,6 +1029,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = CreateIndxReq::RT_UNDEFINED;
       m_requestFlag = 0;
+      m_lastError = CreateIndxRef::NoError;
       m_errorCode = CreateIndxRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -965,34 +1039,49 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != CreateIndxRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != CreateIndxRef::NoError;
     }
     void setError(const CreateIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = CreateIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const CreateTableRef* ref) {
-      if (ref != 0 && ! hasError()) {
+      m_lastError = CreateIndxRef::NoError;
+      if (ref != 0) {
         switch (ref->getErrorCode()) {
         case CreateTableRef::TableAlreadyExist:
-          m_errorCode = CreateIndxRef::IndexExists;
+          m_lastError = CreateIndxRef::IndexExists;
           break;
         default:
-          m_errorCode = (CreateIndxRef::ErrorCode)ref->getErrorCode();
+          m_lastError = (CreateIndxRef::ErrorCode)ref->getErrorCode();
           break;
         }
-        m_errorLine = ref->getErrorLine();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+        }
       }
     }
     void setError(const AlterIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (CreateIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = CreateIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (CreateIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
   };
@@ -1011,6 +1100,7 @@ private:
     DropIndxReq::RequestType m_requestType;
     Uint32 m_requestFlag;
     // error info
+    DropIndxRef::ErrorCode m_lastError;
     DropIndxRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -1022,6 +1112,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = DropIndxReq::RT_UNDEFINED;
       m_requestFlag = 0;
+      m_lastError = DropIndxRef::NoError;
       m_errorCode = DropIndxRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -1031,44 +1122,59 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != DropIndxRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != DropIndxRef::NoError;
     }
     void setError(const DropIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = DropIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = ref->getErrorCode();
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const AlterIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (DropIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = DropIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (DropIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const DropTableRef* ref) {
-      if (ref != 0 && ! hasError()) {
-	switch(ref->errorCode) {
-	case(DropTableRef::Busy):
-	  m_errorCode = DropIndxRef::Busy;
+      m_lastError = DropIndxRef::NoError;
+      if (ref != 0) {
+	switch (ref->errorCode) {
+	case DropTableRef::Busy:
+	  m_lastError = DropIndxRef::Busy;
 	  break;
-	case(DropTableRef::NoSuchTable):
-	  m_errorCode = DropIndxRef::IndexNotFound;
+	case DropTableRef::NoSuchTable:
+	  m_lastError = DropIndxRef::IndexNotFound;
 	  break;
-	case(DropTableRef::DropInProgress):
-	  m_errorCode = DropIndxRef::Busy;
+	case DropTableRef::DropInProgress:
+	  m_lastError = DropIndxRef::Busy;
 	  break;
-	case(DropTableRef::NoDropTableRecordAvailable):
-	  m_errorCode = DropIndxRef::Busy;
+	case DropTableRef::NoDropTableRecordAvailable:
+	  m_lastError = DropIndxRef::Busy;
 	  break;
 	default:
-	  m_errorCode = (DropIndxRef::ErrorCode)ref->errorCode;
+	  m_lastError = (DropIndxRef::ErrorCode)ref->errorCode;
 	  break;
 	}
-        //m_errorLine = ref->getErrorLine();
-        //m_errorNode = ref->getErrorNode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = 0;
+          m_errorNode = 0;
+        }
       }
     }
   };
@@ -1089,6 +1195,7 @@ private:
     AlterIndxReq::RequestType m_requestType;
     Uint32 m_requestFlag;
     // error info
+    AlterIndxRef::ErrorCode m_lastError;
     AlterIndxRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -1101,6 +1208,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = AlterIndxReq::RT_UNDEFINED;
       m_requestFlag = 0;
+      m_lastError = AlterIndxRef::NoError;
       m_errorCode = AlterIndxRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -1111,47 +1219,76 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != AlterIndxRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != AlterIndxRef::NoError;
     }
     void setError(const AlterIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const CreateIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const DropIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const BuildIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterIndxRef::ErrorCode)ref->getErrorCode();
+      m_lastError = AlterIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = 0;
+          m_errorNode = 0;
+        }
       }
     }
     void setError(const CreateTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const DropTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
   };
@@ -1173,6 +1310,7 @@ private:
     Uint32 m_requestFlag;
     Uint32 m_constrTriggerId;
     // error info
+    BuildIndxRef::ErrorCode m_lastError;
     BuildIndxRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -1184,7 +1322,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = BuildIndxReq::RT_UNDEFINED;
       m_requestFlag = 0;
-//      Uint32 m_constrTriggerId = RNIL;
+      m_lastError = BuildIndxRef::NoError;
       m_errorCode = BuildIndxRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -1194,33 +1332,54 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != BuildIndxRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != BuildIndxRef::NoError;
     }
     void setError(const BuildIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = ref->getErrorCode();
+      m_lastError = BuildIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = 0;
+          m_errorNode = 0;
+        }
       }
     }
     void setError(const AlterIndxRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (BuildIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = BuildIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (BuildIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const CreateTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (BuildIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = BuildIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (BuildIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const DropTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (BuildIndxRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = BuildIndxRef::NoError;
+      if (ref != 0) {
+        m_lastError = (BuildIndxRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
   };
@@ -1353,6 +1512,7 @@ private:
     CreateTrigReq::RequestType m_requestType;
     Uint32 m_requestFlag;
     // error info
+    CreateTrigRef::ErrorCode m_lastError;
     CreateTrigRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -1364,6 +1524,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = CreateTrigReq::RT_UNDEFINED;
       m_requestFlag = 0;
+      m_lastError = CreateTrigRef::NoError;
       m_errorCode = CreateTrigRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -1373,21 +1534,32 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != CreateTrigRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != CreateTrigRef::NoError;
     }
     void setError(const CreateTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = CreateTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const AlterTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (CreateTrigRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = CreateTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = (CreateTrigRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
   };
@@ -1406,6 +1578,7 @@ private:
     DropTrigReq::RequestType m_requestType;
     Uint32 m_requestFlag;
     // error info
+    DropTrigRef::ErrorCode m_lastError;
     DropTrigRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -1417,6 +1590,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = DropTrigReq::RT_UNDEFINED;
       m_requestFlag = 0;
+      m_lastError = DropTrigRef::NoError;
       m_errorCode = DropTrigRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -1426,21 +1600,32 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != DropTrigRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != DropTrigRef::NoError;
     }
     void setError(const DropTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = DropTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const AlterTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (DropTrigRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = DropTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = (DropTrigRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
   };
@@ -1461,6 +1646,7 @@ private:
     AlterTrigReq::RequestType m_requestType;
     Uint32 m_requestFlag;
     // error info
+    AlterTrigRef::ErrorCode m_lastError;
     AlterTrigRef::ErrorCode m_errorCode;
     Uint32 m_errorLine;
     Uint32 m_errorNode;
@@ -1472,6 +1658,7 @@ private:
       m_coordinatorRef = 0;
       m_requestType = AlterTrigReq::RT_UNDEFINED;
       m_requestFlag = 0;
+      m_lastError = AlterTrigRef::NoError;
       m_errorCode = AlterTrigRef::NoError;
       m_errorLine = 0;
       m_errorNode = 0;
@@ -1481,28 +1668,43 @@ private:
       m_requestType = req->getRequestType();
       m_requestFlag = req->getRequestFlag();
     }
+    bool hasLastError() {
+      return m_lastError != AlterTrigRef::NoError;
+    }
     bool hasError() {
       return m_errorCode != AlterTrigRef::NoError;
     }
     void setError(const AlterTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterTrigRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterTrigRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const CreateTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterTrigRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterTrigRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
     void setError(const DropTrigRef* ref) {
-      if (ref != 0 && ! hasError()) {
-        m_errorCode = (AlterTrigRef::ErrorCode)ref->getErrorCode();
-        m_errorLine = ref->getErrorLine();
-        m_errorNode = ref->getErrorNode();
+      m_lastError = AlterTrigRef::NoError;
+      if (ref != 0) {
+        m_lastError = (AlterTrigRef::ErrorCode)ref->getErrorCode();
+        if (! hasError()) {
+          m_errorCode = m_lastError;
+          m_errorLine = ref->getErrorLine();
+          m_errorNode = ref->getErrorNode();
+        }
       }
     }
   };
@@ -1561,6 +1763,70 @@ private:
   // Unique key for operation  XXX move to some system table
   Uint32 c_opRecordSequence;
 
+  /*
+   * Master DICT can be locked in 2 mutually exclusive ways:
+   *
+   * 1) for schema ops, via operation records
+   * 2) against schema ops, via a lock queue
+   *
+   * Current use of 2) is by a starting node, to prevent schema ops
+   * until started.  The ops are refused (BlockState != BS_IDLE),
+   * not queued.
+   *
+   * Master failure is not handled, in node start case the starting
+   * node will crash too anyway.  Use lock table in future..
+   *
+   * The lock queue is "serial" but other behaviour is possible
+   * by checking lock types e.g. to allow parallel node starts.
+   *
+   * Checking release of last op record is not convenient with
+   * current structure (5.0).  Instead we poll via continueB.
+   *
+   * XXX only table ops check BlockState
+   */
+  struct DictLockType;
+  friend struct DictLockType;
+
+  struct DictLockType {
+    DictLockReq::LockType lockType;
+    BlockState blockState;
+    const char* text;
+  };
+
+  struct DictLockRecord;
+  friend struct DictLockRecord;
+
+  struct DictLockRecord {
+    DictLockReq req;
+    const DictLockType* lt;
+    bool locked;
+  union {
+    Uint32 nextPool;
+    Uint32 nextList;
+    };
+    Uint32 prevList;
+  };
+
+  typedef Ptr<DictLockRecord> DictLockPtr;
+  ArrayPool<DictLockRecord> c_dictLockPool;
+  DLFifoList<DictLockRecord> c_dictLockQueue;
+  bool c_dictLockPoll;
+
+  static const DictLockType* getDictLockType(Uint32 lockType);
+  void sendDictLockInfoEvent(Uint32 pollCount);
+  void sendDictLockInfoEvent(DictLockPtr lockPtr, const char* text);
+
+  void checkDictLockQueue(Signal* signal, bool poll);
+  void sendDictLockConf(Signal* signal, DictLockPtr lockPtr);
+  void sendDictLockRef(Signal* signal, DictLockReq req, Uint32 errorCode);
+
+  // control polling i.e. continueB loop
+  void setDictLockPoll(Signal* signal, bool on, Uint32 pollCount);
+
+  // NF handling
+  void removeStaleDictLocks(Signal* signal, const Uint32* theFailedNodes);
+
+
   // Statement blocks
 
   /* ------------------------------------------------------------ */
@@ -1583,7 +1849,8 @@ private:
   bool getNewAttributeRecord(TableRecordPtr tablePtr,
 			     AttributeRecordPtr & attrPtr);
   void packTableIntoPages(Signal* signal, Uint32 tableId, Uint32 pageId);
-  void packTableIntoPagesImpl(SimpleProperties::Writer &, TableRecordPtr);
+  void packTableIntoPagesImpl(SimpleProperties::Writer &, TableRecordPtr,
+			      Signal* signal= 0);
   
   void sendGET_TABINFOREQ(Signal* signal,
                           Uint32 tableId);
@@ -1631,7 +1898,8 @@ private:
   void openSchemaFile(Signal* signal,
                       Uint32 fileNo,
                       Uint32 fsPtr,
-                      bool writeFlag);
+                      bool writeFlag,
+                      bool newFile);
   void writeSchemaFile(Signal* signal, Uint32 filePtr, Uint32 fsPtr);
   void writeSchemaConf(Signal* signal,
                                FsConnectRecordPtr fsPtr);
@@ -1673,6 +1941,7 @@ private:
   void readSchemaRef(Signal* signal, FsConnectRecordPtr fsPtr);
   void closeReadSchemaConf(Signal* signal,
                            FsConnectRecordPtr fsPtr);
+  bool convertSchemaFileTo_5_0_6(XSchemaFile*);
 
   /* ------------------------------------------------------------ */
   // Get table definitions
