@@ -25,6 +25,7 @@
 #endif
 
 #include <mgmapi.h>
+#include <util/BaseString.hpp>
 
 class MgmtSrvr;
 
@@ -70,6 +71,9 @@ private:
    */
   void analyseAfterFirstToken(int processId, char* allAfterFirstTokenCstr);
 
+  void executeCommand(Vector<BaseString> &command_list,
+                      unsigned command_pos,
+                      int *node_ids, int no_of_nodes);
   /**
    *   Parse the block specification part of the LOG* commands,
    *   things after LOG*: [BLOCK = {ALL|<blockName>+}]
@@ -104,10 +108,14 @@ private:
 
 public:
   void executeStop(int processId, const char* parameters, bool all);
+  void executeStop(Vector<BaseString> &command_list, unsigned command_pos,
+                   int *node_ids, int no_of_nodes);
   void executeEnterSingleUser(char* parameters);
   void executeExitSingleUser(char* parameters);
   void executeStart(int processId, const char* parameters, bool all);
   void executeRestart(int processId, const char* parameters, bool all);
+  void executeRestart(Vector<BaseString> &command_list, unsigned command_pos,
+                      int *node_ids, int no_of_nodes);
   void executeLogLevel(int processId, const char* parameters, bool all);
   void executeError(int processId, const char* parameters, bool all);
   void executeLog(int processId, const char* parameters, bool all);
@@ -154,6 +162,7 @@ private:
 
   NdbMgmHandle m_mgmsrv;
   NdbMgmHandle m_mgmsrv2;
+  const char *m_constr;
   bool m_connected;
   int m_verbose;
   int try_reconnect;
@@ -382,22 +391,7 @@ convert(const char* s, int& val) {
 CommandInterpreter::CommandInterpreter(const char *_host,int verbose) 
   : m_verbose(verbose)
 {
-  m_mgmsrv = ndb_mgm_create_handle();
-  if(m_mgmsrv == NULL) {
-    ndbout_c("Cannot create handle to management server.");
-    exit(-1);
-  }
-  m_mgmsrv2 = ndb_mgm_create_handle();
-  if(m_mgmsrv2 == NULL) {
-    ndbout_c("Cannot create handle to management server.");
-    exit(-1);
-  }
-  if (ndb_mgm_set_connectstring(m_mgmsrv, _host))
-  {
-    printError();
-    exit(-1);
-  }
-
+  m_constr= _host;
   m_connected= false;
   m_event_thread= 0;
   try_reconnect = 0;
@@ -414,8 +408,6 @@ CommandInterpreter::CommandInterpreter(const char *_host,int verbose)
 CommandInterpreter::~CommandInterpreter() 
 {
   disconnect();
-  ndb_mgm_destroy_handle(&m_mgmsrv);
-  ndb_mgm_destroy_handle(&m_mgmsrv2);
 }
 
 static bool 
@@ -437,15 +429,14 @@ emptyString(const char* s)
 void
 CommandInterpreter::printError() 
 {
-  if (ndb_mgm_check_connection(m_mgmsrv))
-  {
-    m_connected= false;
-    disconnect();
-  }
   ndbout_c("* %5d: %s", 
 	   ndb_mgm_get_latest_error(m_mgmsrv),
 	   ndb_mgm_get_latest_error_msg(m_mgmsrv));
   ndbout_c("*        %s", ndb_mgm_get_latest_error_desc(m_mgmsrv));
+  if (ndb_mgm_check_connection(m_mgmsrv))
+  {
+    disconnect();
+  }
 }
 
 //*****************************************************************************
@@ -459,7 +450,9 @@ event_thread_run(void* m)
 
   NdbMgmHandle handle= *(NdbMgmHandle*)m;
 
-  int filter[] = { 15, NDB_MGM_EVENT_CATEGORY_BACKUP, 0 };
+  int filter[] = { 15, NDB_MGM_EVENT_CATEGORY_BACKUP,
+		   1, NDB_MGM_EVENT_CATEGORY_STARTUP,
+		   0 };
   int fd = ndb_mgm_listen_event(handle, filter);
   if (fd != NDB_INVALID_SOCKET)
   {
@@ -487,68 +480,97 @@ event_thread_run(void* m)
 }
 
 bool
-CommandInterpreter::connect() 
+CommandInterpreter::connect()
 {
   DBUG_ENTER("CommandInterpreter::connect");
-  if(!m_connected)
+
+  if(m_connected)
+    DBUG_RETURN(m_connected);
+
+  m_mgmsrv = ndb_mgm_create_handle();
+  if(m_mgmsrv == NULL) {
+    ndbout_c("Cannot create handle to management server.");
+    exit(-1);
+  }
+  m_mgmsrv2 = ndb_mgm_create_handle();
+  if(m_mgmsrv2 == NULL) {
+    ndbout_c("Cannot create 2:nd handle to management server.");
+    exit(-1);
+  }
+
+  if (ndb_mgm_set_connectstring(m_mgmsrv, m_constr))
   {
-    if(!ndb_mgm_connect(m_mgmsrv, try_reconnect-1, 5, 1))
+    printError();
+    exit(-1);
+  }
+
+  if(ndb_mgm_connect(m_mgmsrv, try_reconnect-1, 5, 1))
+    DBUG_RETURN(m_connected); // couldn't connect, always false
+
+  const char *host= ndb_mgm_get_connected_host(m_mgmsrv);
+  unsigned port= ndb_mgm_get_connected_port(m_mgmsrv);
+  BaseString constr;
+  constr.assfmt("%s:%d",host,port);
+  if(!ndb_mgm_set_connectstring(m_mgmsrv2, constr.c_str()) &&
+     !ndb_mgm_connect(m_mgmsrv2, try_reconnect-1, 5, 1))
+  {
+    DBUG_PRINT("info",("2:ndb connected to Management Server ok at: %s:%d",
+                       host, port));
+    assert(m_event_thread == 0);
+    assert(do_event_thread == 0);
+    do_event_thread= 0;
+    m_event_thread = NdbThread_Create(event_thread_run,
+                                      (void**)&m_mgmsrv2,
+                                      32768,
+                                      "CommandInterpreted_event_thread",
+                                      NDB_THREAD_PRIO_LOW);
+    if (m_event_thread != 0)
     {
-      const char *host= ndb_mgm_get_connected_host(m_mgmsrv);
-      unsigned port= ndb_mgm_get_connected_port(m_mgmsrv);
-      if(!ndb_mgm_set_connectstring(m_mgmsrv2,
-				    BaseString(host).appfmt(":%d",port).c_str())
-	 &&
-	 !ndb_mgm_connect(m_mgmsrv2, try_reconnect-1, 5, 1))
+      DBUG_PRINT("info",("Thread created ok, waiting for started..."));
+      int iter= 1000; // try for 30 seconds
+      while(do_event_thread == 0 &&
+            iter-- > 0)
+        NdbSleep_MilliSleep(30);
+    }
+    if (m_event_thread == 0 ||
+        do_event_thread == 0 ||
+        do_event_thread == -1)
+    {
+      DBUG_PRINT("info",("Warning, event thread startup failed, "
+                         "degraded printouts as result, errno=%d",
+                         errno));
+      printf("Warning, event thread startup failed, "
+             "degraded printouts as result, errno=%d\n", errno);
+      do_event_thread= 0;
+      if (m_event_thread)
       {
-	assert(m_event_thread == 0);
-	assert(do_event_thread == 0);
-	do_event_thread= 0;
-	m_event_thread = NdbThread_Create(event_thread_run,
-					  (void**)&m_mgmsrv2,
-					  32768,
-					  "CommandInterpreted_event_thread",
-					  NDB_THREAD_PRIO_LOW);
-	if (m_event_thread != 0)
-	{
-	  int iter= 1000; // try for 30 seconds
-	  while(do_event_thread == 0 &&
-		iter-- > 0)
-	    NdbSleep_MilliSleep(30);
-	}
-	if (m_event_thread == 0 ||
-	    do_event_thread == 0 ||
-	    do_event_thread == -1)
-	{
-	  DBUG_PRINT("info",("Warning, event thread startup failed, "
-			     "degraded printouts as result, errno=%d",
-			     errno));
-	  printf("Warning, event thread startup failed, "
-		 "degraded printouts as result, errno=%d\n", errno);
-	  do_event_thread= 0;
-	  if (m_event_thread)
-	  {
-	    void *res;
-	    NdbThread_WaitFor(m_event_thread, &res);
-	    NdbThread_Destroy(&m_event_thread);
-	  }
-	  ndb_mgm_disconnect(m_mgmsrv2);
-	}
-      }
-      else
-      {
-	printf("Warning, event connect failed, degraded printouts as result\n");
-      }
-      m_connected= true;
-      DBUG_PRINT("info",("Connected to Management Server at: %s:%d",
-			 host,port));
-      if (m_verbose)
-      {
-	printf("Connected to Management Server at: %s:%d\n",
-	       host, port);
+        void *res;
+        NdbThread_WaitFor(m_event_thread, &res);
+        NdbThread_Destroy(&m_event_thread);
       }
+      ndb_mgm_disconnect(m_mgmsrv2);
     }
   }
+  else
+  {
+    DBUG_PRINT("warning",
+               ("Could not do 2:nd connect to mgmtserver for event listening"));
+    DBUG_PRINT("info", ("code: %d, msg: %s",
+                        ndb_mgm_get_latest_error(m_mgmsrv2),
+                        ndb_mgm_get_latest_error_msg(m_mgmsrv2)));
+    printf("Warning, event connect failed, degraded printouts as result\n");
+    printf("code: %d, msg: %s\n",
+           ndb_mgm_get_latest_error(m_mgmsrv2),
+           ndb_mgm_get_latest_error_msg(m_mgmsrv2));
+  }
+  m_connected= true;
+  DBUG_PRINT("info",("Connected to Management Server at: %s:%d", host, port));
+  if (m_verbose)
+  {
+    printf("Connected to Management Server at: %s:%d\n",
+           host, port);
+  }
+
   DBUG_RETURN(m_connected);
 }
 
@@ -556,20 +578,18 @@ bool
 CommandInterpreter::disconnect() 
 {
   DBUG_ENTER("CommandInterpreter::disconnect");
+
   if (m_event_thread) {
     void *res;
     do_event_thread= 0;
     NdbThread_WaitFor(m_event_thread, &res);
     NdbThread_Destroy(&m_event_thread);
     m_event_thread= 0;
-    ndb_mgm_disconnect(m_mgmsrv2);
+    ndb_mgm_destroy_handle(&m_mgmsrv2);
   }
   if (m_connected)
   {
-    if (ndb_mgm_disconnect(m_mgmsrv) == -1) {
-      ndbout_c("Could not disconnect from management server");
-      printError();
-    }
+    ndb_mgm_destroy_handle(&m_mgmsrv);
     m_connected= false;
   }
   DBUG_RETURN(true);
@@ -631,9 +651,16 @@ CommandInterpreter::execute_impl(const char *_line)
     }
   } while (do_continue);
   // if there is anything in the line proceed
+  Vector<BaseString> command_list;
+  {
+    BaseString tmp(line);
+    tmp.split(command_list);
+    for (unsigned i= 0; i < command_list.size();)
+      command_list[i].c_str()[0] ? i++ : (command_list.erase(i),0);
+  }
   char* firstToken = strtok(line, " ");
   char* allAfterFirstToken = strtok(NULL, "");
-  
+
   if (strcasecmp(firstToken, "HELP") == 0 ||
       strcasecmp(firstToken, "?") == 0) {
     executeHelp(allAfterFirstToken);
@@ -711,22 +738,45 @@ CommandInterpreter::execute_impl(const char *_line)
     analyseAfterFirstToken(-1, allAfterFirstToken);
   } else {
     /**
-     * First token should be a digit, node ID
+     * First tokens should be digits, node ID's
      */
-    int nodeId;
-
-    if (! convert(firstToken, nodeId)) {
+    int node_ids[MAX_NODES];
+    unsigned pos;
+    for (pos= 0; pos < command_list.size(); pos++)
+    {
+      int node_id;
+      if (convert(command_list[pos].c_str(), node_id))
+      {
+        if (node_id <= 0) {
+          ndbout << "Invalid node ID: " << command_list[pos].c_str()
+                 << "." << endl;
+          DBUG_RETURN(true);
+        }
+        node_ids[pos]= node_id;
+        continue;
+      }
+      break;
+    }
+    int no_of_nodes= pos;
+    if (no_of_nodes == 0)
+    {
+      /* No digit found */
       invalid_command(_line);
       DBUG_RETURN(true);
     }
-
-    if (nodeId <= 0) {
-      ndbout << "Invalid node ID: " << firstToken << "." << endl;
+    if (pos == command_list.size())
+    {
+      /* No command found */
+      invalid_command(_line);
       DBUG_RETURN(true);
     }
-    
-    analyseAfterFirstToken(nodeId, allAfterFirstToken);
-    
+    if (no_of_nodes == 1)
+    {
+      analyseAfterFirstToken(node_ids[0], allAfterFirstToken);
+      DBUG_RETURN(true);
+    }
+    executeCommand(command_list, pos, node_ids, no_of_nodes);
+    DBUG_RETURN(true);
   }
   DBUG_RETURN(true);
 }
@@ -796,6 +846,27 @@ CommandInterpreter::analyseAfterFirstToken(int processId,
   ndbout << endl;
 }
 
+void
+CommandInterpreter::executeCommand(Vector<BaseString> &command_list,
+                                   unsigned command_pos,
+                                   int *node_ids, int no_of_nodes)
+{
+  const char *cmd= command_list[command_pos].c_str();
+  if (strcasecmp("STOP", cmd) == 0)
+  {
+    executeStop(command_list, command_pos+1, node_ids, no_of_nodes);
+    return;
+  }
+  if (strcasecmp("RESTART", cmd) == 0)
+  {
+    executeRestart(command_list, command_pos+1, node_ids, no_of_nodes);
+    return;
+  }
+  ndbout_c("Invalid command: '%s' after multi node id list. "
+           "Expected STOP or RESTART.", cmd);
+  return;
+}
+
 /**
  * Get next nodeid larger than the give node_id. node_id will be
  * set to the next node_id in the list. node_id should be set
@@ -986,42 +1057,21 @@ CommandInterpreter::executeShutdown(char* parameters)
   NdbAutoPtr<char> ap1((char*)state);
 
   int result = 0;
-  result = ndb_mgm_stop(m_mgmsrv, 0, 0);
+  int need_disconnect;
+  result = ndb_mgm_stop3(m_mgmsrv, -1, 0, 0, &need_disconnect);
   if (result < 0) {
-    ndbout << "Shutdown off NDB Cluster storage node(s) failed." << endl;
+    ndbout << "Shutdown of NDB Cluster node(s) failed." << endl;
     printError();
     return result;
   }
 
-  ndbout << result << " NDB Cluster storage node(s) have shutdown." << endl;
+  ndbout << result << " NDB Cluster node(s) have shutdown." << endl;
 
-  int mgm_id= 0;
-  for(int i=0; i < state->no_of_nodes; i++) {
-    if(state->node_states[i].node_type == NDB_MGM_NODE_TYPE_MGM &&
-       state->node_states[i].version != 0){
-      if (mgm_id == 0)
-	mgm_id= state->node_states[i].node_id;
-      else {
-	ndbout << "Unable to locate management server, "
-	       << "shutdown manually with <id> STOP"
-	       << endl;
-	return 1;
-      }
-    }
+  if(need_disconnect) {
+    ndbout << "Disconnecting to allow management server to shutdown."
+           << endl;
+    disconnect();
   }
-
-  result = ndb_mgm_stop(m_mgmsrv, 1, &mgm_id);
-  if (result <= 0) {
-    ndbout << "Shutdown of NDB Cluster management server failed." << endl;
-    printError();
-    if (result == 0)
-      return 1;
-    return result;
-  }
-
-  m_connected= false;
-  disconnect();
-  ndbout << "NDB Cluster management server shutdown." << endl;
   return 0;
 }
 
@@ -1085,7 +1135,7 @@ print_nodes(ndb_mgm_cluster_state *state, ndb_mgm_configuration_iterator *it,
 	  }
 	  if (node_state->node_group >= 0) {
 	    ndbout << ", Nodegroup: " << node_state->node_group;
-	    if (node_state->dynamic_id == master_id)
+	    if (master_id && node_state->dynamic_id == master_id)
 	      ndbout << ", Master";
 	  }
 	}
@@ -1245,12 +1295,7 @@ CommandInterpreter::executeConnect(char* parameters)
 {
   disconnect();
   if (!emptyString(parameters)) {
-    if (ndb_mgm_set_connectstring(m_mgmsrv,
-				  BaseString(parameters).trim().c_str()))
-    {
-      printError();
-      return;
-    }
+    m_constr= BaseString(parameters).trim().c_str();
   }
   connect();
 }
@@ -1268,7 +1313,7 @@ CommandInterpreter::executeClusterLog(char* parameters)
     DBUG_VOID_RETURN;
   }
 
-  enum ndb_mgm_clusterlog_level severity = NDB_MGM_CLUSTERLOG_ALL;
+  enum ndb_mgm_event_severity severity = NDB_MGM_EVENT_SEVERITY_ALL;
     
   char * tmpString = my_strdup(parameters,MYF(MY_WME));
   My_auto_ptr<char> ap1(tmpString);
@@ -1276,7 +1321,7 @@ CommandInterpreter::executeClusterLog(char* parameters)
   char * item = strtok_r(tmpString, " ", &tmpPtr);
   int enable;
 
-  Uint32 *enabled = ndb_mgm_get_logfilter(m_mgmsrv);
+  const unsigned int *enabled= ndb_mgm_get_logfilter(m_mgmsrv);
   if(enabled == NULL) {
     ndbout << "Couldn't get status" << endl;
     printError();
@@ -1298,8 +1343,8 @@ CommandInterpreter::executeClusterLog(char* parameters)
       printf("enabled[%d] = %d\n", i, enabled[i]);
 #endif
     ndbout << "Severities enabled: ";
-    for(i = 1; i < (int)NDB_MGM_CLUSTERLOG_ALL; i++) {
-      const char *str= ndb_mgm_get_clusterlog_level_string((ndb_mgm_clusterlog_level)i);
+    for(i = 1; i < (int)NDB_MGM_EVENT_SEVERITY_ALL; i++) {
+      const char *str= ndb_mgm_get_event_severity_string((ndb_mgm_event_severity)i);
       if (str == 0)
       {
 	DBUG_ASSERT(false);
@@ -1333,8 +1378,10 @@ CommandInterpreter::executeClusterLog(char* parameters)
   int res_enable;
   item = strtok_r(NULL, " ", &tmpPtr);
   if (item == NULL) {
-    res_enable= ndb_mgm_filter_clusterlog(m_mgmsrv,
-					  NDB_MGM_CLUSTERLOG_ON, enable, NULL);
+    res_enable=
+      ndb_mgm_set_clusterlog_severity_filter(m_mgmsrv,
+					     NDB_MGM_EVENT_SEVERITY_ON,
+					     enable, NULL);
     if (res_enable < 0)
     {
       ndbout << "Couldn't set filter" << endl;
@@ -1346,32 +1393,33 @@ CommandInterpreter::executeClusterLog(char* parameters)
   }
 
   do {
-    severity= NDB_MGM_ILLEGAL_CLUSTERLOG_LEVEL;
+    severity= NDB_MGM_ILLEGAL_EVENT_SEVERITY;
     if (strcasecmp(item, "ALL") == 0) {
-      severity = NDB_MGM_CLUSTERLOG_ALL;	
+      severity = NDB_MGM_EVENT_SEVERITY_ALL;	
     } else if (strcasecmp(item, "ALERT") == 0) {
-      severity = NDB_MGM_CLUSTERLOG_ALERT;
+      severity = NDB_MGM_EVENT_SEVERITY_ALERT;
     } else if (strcasecmp(item, "CRITICAL") == 0) { 
-      severity = NDB_MGM_CLUSTERLOG_CRITICAL;
+      severity = NDB_MGM_EVENT_SEVERITY_CRITICAL;
     } else if (strcasecmp(item, "ERROR") == 0) {
-      severity = NDB_MGM_CLUSTERLOG_ERROR;
+      severity = NDB_MGM_EVENT_SEVERITY_ERROR;
     } else if (strcasecmp(item, "WARNING") == 0) {
-      severity = NDB_MGM_CLUSTERLOG_WARNING;
+      severity = NDB_MGM_EVENT_SEVERITY_WARNING;
     } else if (strcasecmp(item, "INFO") == 0) {
-      severity = NDB_MGM_CLUSTERLOG_INFO;
+      severity = NDB_MGM_EVENT_SEVERITY_INFO;
     } else if (strcasecmp(item, "DEBUG") == 0) {
-      severity = NDB_MGM_CLUSTERLOG_DEBUG;
+      severity = NDB_MGM_EVENT_SEVERITY_DEBUG;
     } else if (strcasecmp(item, "OFF") == 0 ||
 	       strcasecmp(item, "ON") == 0) {
       if (enable < 0) // only makes sense with toggle
-	severity = NDB_MGM_CLUSTERLOG_ON;
+	severity = NDB_MGM_EVENT_SEVERITY_ON;
     }
-    if (severity == NDB_MGM_ILLEGAL_CLUSTERLOG_LEVEL) {
+    if (severity == NDB_MGM_ILLEGAL_EVENT_SEVERITY) {
       ndbout << "Invalid severity level: " << item << endl;
       DBUG_VOID_RETURN;
     }
 
-    res_enable = ndb_mgm_filter_clusterlog(m_mgmsrv, severity, enable, NULL);
+    res_enable= ndb_mgm_set_clusterlog_severity_filter(m_mgmsrv, severity,
+						       enable, NULL);
     if (res_enable < 0)
     {
       ndbout << "Couldn't set filter" << endl;
@@ -1390,24 +1438,69 @@ CommandInterpreter::executeClusterLog(char* parameters)
 //*****************************************************************************
 
 void
-CommandInterpreter::executeStop(int processId, const char *, bool all) 
+CommandInterpreter::executeStop(int processId, const char *parameters,
+                                bool all) 
 {
-  int result = 0;
-  if(all) {
-    result = ndb_mgm_stop(m_mgmsrv, 0, 0);
-  } else {
-    result = ndb_mgm_stop(m_mgmsrv, 1, &processId);
+  Vector<BaseString> command_list;
+  if (parameters)
+  {
+    BaseString tmp(parameters);
+    tmp.split(command_list);
+    for (unsigned i= 0; i < command_list.size();)
+      command_list[i].c_str()[0] ? i++ : (command_list.erase(i),0);
   }
-  if (result < 0) {
-    ndbout << "Shutdown failed." << endl;
-    printError();
-  } else
+  if (all)
+    executeStop(command_list, 0, 0, 0);
+  else
+    executeStop(command_list, 0, &processId, 1);
+}
+
+void
+CommandInterpreter::executeStop(Vector<BaseString> &command_list,
+                                unsigned command_pos,
+                                int *node_ids, int no_of_nodes)
+{
+  int need_disconnect;
+  int abort= 0;
+  for (; command_pos < command_list.size(); command_pos++)
+  {
+    const char *item= command_list[command_pos].c_str();
+    if (strcasecmp(item, "-A") == 0)
     {
-      if(all)
-	ndbout << "NDB Cluster has shutdown." << endl;
-      else
-	ndbout << "Node " << processId << " has shutdown." << endl;
+      abort= 1;
+      continue;
     }
+    ndbout_c("Invalid option: %s. Expecting -A after STOP",
+             item);
+    return;
+  }
+
+  int result= ndb_mgm_stop3(m_mgmsrv, no_of_nodes, node_ids, abort,
+                            &need_disconnect);
+  if (result < 0)
+  {
+    ndbout_c("Shutdown failed.");
+    printError();
+  }
+  else
+  {
+    if (node_ids == 0)
+      ndbout_c("NDB Cluster has shutdown.");
+    else
+    {
+      ndbout << "Node";
+      for (int i= 0; i < no_of_nodes; i++)
+          ndbout << " " << node_ids[i];
+      ndbout_c(" has shutdown.");
+    }
+  }
+
+  if(need_disconnect)
+  {
+    ndbout << "Disconnecting to allow Management Server to shutdown" << endl;
+    disconnect();
+  }
+
 }
 
 void
@@ -1430,9 +1523,8 @@ CommandInterpreter::executeEnterSingleUser(char* parameters)
     ndbout_c("Entering single user mode for node %d failed", nodeId);
     printError();
   } else {
-    ndbout_c("Entering single user mode");
-    ndbout_c("Access will be granted for API node %d only.", nodeId);
-    ndbout_c("Use ALL STATUS to see when single user mode has been entered.");
+    ndbout_c("Single user mode entered");
+    ndbout_c("Access is granted for API node %d only.", nodeId);
   }
 }
 
@@ -1445,7 +1537,7 @@ CommandInterpreter::executeExitSingleUser(char* parameters)
     printError();
   } else {
     ndbout_c("Exiting single user mode in progress.");
-    ndbout_c("Use ALL STATUS to see when single user mode has been exited.");
+    ndbout_c("Use ALL STATUS or SHOW to see when single user mode has been exited.");
   }
 }
 
@@ -1474,47 +1566,77 @@ CommandInterpreter::executeStart(int processId, const char* parameters,
 
 void
 CommandInterpreter::executeRestart(int processId, const char* parameters,
-				   bool all) 
+				   bool all)
+{
+  Vector<BaseString> command_list;
+  if (parameters)
+  {
+    BaseString tmp(parameters);
+    tmp.split(command_list);
+    for (unsigned i= 0; i < command_list.size();)
+      command_list[i].c_str()[0] ? i++ : (command_list.erase(i),0);
+  }
+  if (all)
+    executeRestart(command_list, 0, 0, 0);
+  else
+    executeRestart(command_list, 0, &processId, 1);
+}
+
+void
+CommandInterpreter::executeRestart(Vector<BaseString> &command_list,
+                                   unsigned command_pos,
+                                   int *node_ids, int no_of_nodes)
 {
   int result;
-  int nostart = 0;
-  int initialstart = 0;
-  int abort = 0;
+  int nostart= 0;
+  int initialstart= 0;
+  int abort= 0;
+  int need_disconnect= 0;
 
-  if(parameters != 0 && strlen(parameters) != 0){
-    char * tmpString = my_strdup(parameters,MYF(MY_WME));
-    My_auto_ptr<char> ap1(tmpString);
-    char * tmpPtr = 0;
-    char * item = strtok_r(tmpString, " ", &tmpPtr);
-    while(item != NULL){
-      if(strcasecmp(item, "-N") == 0)
-	nostart = 1;
-      if(strcasecmp(item, "-I") == 0)
-	initialstart = 1;
-      if(strcasecmp(item, "-A") == 0)
-	abort = 1;
-      item = strtok_r(NULL, " ", &tmpPtr);
-    }
-  }
-
-  if(all) {
-    result = ndb_mgm_restart2(m_mgmsrv, 0, NULL, initialstart, nostart, abort);
-  } else {
-    int v[1];
-    v[0] = processId;
-    result = ndb_mgm_restart2(m_mgmsrv, 1, v, initialstart, nostart, abort);
-  }
-  
-  if (result <= 0) {
-    ndbout.println("Restart failed.", result);
-    printError();
-  } else
+  for (; command_pos < command_list.size(); command_pos++)
+  {
+    const char *item= command_list[command_pos].c_str();
+    if (strcasecmp(item, "-N") == 0)
     {
-      if(all)
-	ndbout << "NDB Cluster is being restarted." << endl;
-      else
-	ndbout_c("Node %d is being restarted.", processId);
+      nostart= 1;
+      continue;
     }
+    if (strcasecmp(item, "-I") == 0)
+    {
+      initialstart= 1;
+      continue;
+    }
+    if (strcasecmp(item, "-A") == 0)
+    {
+      abort= 1;
+      continue;
+    }
+    ndbout_c("Invalid option: %s. Expecting -A,-N or -I after RESTART",
+             item);
+    return;
+  }
+
+  result= ndb_mgm_restart3(m_mgmsrv, no_of_nodes, node_ids,
+                           initialstart, nostart, abort, &need_disconnect);
+
+  if (result <= 0) {
+    ndbout_c("Restart failed.");
+    printError();
+  }
+  else
+  {
+    if (node_ids == 0)
+      ndbout_c("NDB Cluster is being restarted.");
+    else
+    {
+      ndbout << "Node";
+      for (int i= 0; i < no_of_nodes; i++)
+        ndbout << " " << node_ids[i];
+      ndbout_c(" is being restarted");
+    }
+    if(need_disconnect)
+      disconnect();
+  }
 }
 
 void
@@ -1909,47 +2031,54 @@ CommandInterpreter::executeEventReporting(int processId,
     return;
   }
   BaseString tmp(parameters);
-  Vector<BaseString> spec;
-  tmp.split(spec, "=");
-  if(spec.size() != 2){
-    ndbout << "Invalid loglevel specification: " << parameters << endl;
-    return;
-  }
+  Vector<BaseString> specs;
+  tmp.split(specs, " ");
 
-  spec[0].trim().ndb_toupper();
-  int category = ndb_mgm_match_event_category(spec[0].c_str());
-  if(category == NDB_MGM_ILLEGAL_EVENT_CATEGORY){
-    if(!convert(spec[0].c_str(), category) ||
-       category < NDB_MGM_MIN_EVENT_CATEGORY ||
-       category > NDB_MGM_MAX_EVENT_CATEGORY){
-      ndbout << "Unknown category: \"" << spec[0].c_str() << "\"" << endl;
-      return;
+  for (int i=0; i < specs.size(); i++)
+  {
+    Vector<BaseString> spec;
+    specs[i].split(spec, "=");
+    if(spec.size() != 2){
+      ndbout << "Invalid loglevel specification: " << specs[i] << endl;
+      continue;
+    }
+
+    spec[0].trim().ndb_toupper();
+    int category = ndb_mgm_match_event_category(spec[0].c_str());
+    if(category == NDB_MGM_ILLEGAL_EVENT_CATEGORY){
+      if(!convert(spec[0].c_str(), category) ||
+	 category < NDB_MGM_MIN_EVENT_CATEGORY ||
+	 category > NDB_MGM_MAX_EVENT_CATEGORY){
+	ndbout << "Unknown category: \"" << spec[0].c_str() << "\"" << endl;
+	continue;
+      }
+    }
+
+    int level;
+    if (!convert(spec[1].c_str(),level))
+    {
+      ndbout << "Invalid level: " << spec[1].c_str() << endl;
+      continue;
+    }
+
+    ndbout << "Executing CLUSTERLOG " << spec[0] << "=" << spec[1]
+	   << " on node " << processId << flush;
+
+    struct ndb_mgm_reply reply;
+    int result;
+    result = ndb_mgm_set_loglevel_clusterlog(m_mgmsrv, 
+					     processId,
+					     (ndb_mgm_event_category)category,
+					     level, 
+					     &reply);
+  
+    if (result != 0) {
+      ndbout_c(" failed."); 
+      printError();
+    } else {
+      ndbout_c(" OK!"); 
     }
   }
-
-  int level;
-  if (!convert(spec[1].c_str(),level))
-  {
-    ndbout << "Invalid level: " << spec[1].c_str() << endl;
-    return;
-  }
-
-  ndbout << "Executing CLUSTERLOG on node " << processId << flush;
-
-  struct ndb_mgm_reply reply;
-  int result;
-  result = ndb_mgm_set_loglevel_clusterlog(m_mgmsrv, 
-					   processId,
-					   (ndb_mgm_event_category)category,
-					   level, 
-					   &reply);
-  
-  if (result != 0) {
-    ndbout_c(" failed."); 
-    printError();
-  } else {
-    ndbout_c(" OK!"); 
-  }  
 }
 
 /*****************************************************************************
