@@ -73,13 +73,13 @@ public:
      */
     Uint32 hbFrequency; // Heartbeat frequence 
     Uint32 hbCounter;   // # milliseconds passed since last hb sent
-    Uint32 hbSent;      // # heartbeats sent (without answer)
   };
   
   const Node &  getNodeInfo(NodeId) const;
   Uint32        getNoOfConnectedNodes() const;
+  void          hb_received(NodeId);
+
   Uint32        m_connect_count;
-  
 private:
   Uint32        noOfAliveNodes;
   Uint32        noOfConnectedNodes;
@@ -129,6 +129,12 @@ ClusterMgr::getNoOfConnectedNodes() const {
   return noOfConnectedNodes;
 }
 
+inline
+void
+ClusterMgr::hb_received(NodeId nodeId) {
+  theNodes[nodeId].m_info.m_heartbeat_cnt= 0;
+}
+
 /*****************************************************************************/
 
 /**
