@@ -19,12 +19,12 @@
 
 #include "NdbApiSignal.hpp"
 #include "NdbImpl.hpp"
-#include "NdbOperation.hpp"
-#include "NdbIndexOperation.hpp"
-#include "NdbScanOperation.hpp"
-#include "NdbConnection.hpp"
-#include "NdbRecAttr.hpp"
-#include "NdbReceiver.hpp"
+#include <NdbTransaction.hpp>
+#include <NdbOperation.hpp>
+#include <NdbIndexOperation.hpp>
+#include <NdbScanOperation.hpp>
+#include <NdbRecAttr.hpp>
+#include <NdbReceiver.hpp>
 #include "API.hpp"
 
 #include <signaldata/TcCommit.hpp>
@@ -107,15 +107,13 @@ Ndb::init(int aMaxNoOfTransactions)
     goto error_handler;
   }
   
-  tMaxNoOfTransactions = aMaxNoOfTransactions * 3;
-  if (tMaxNoOfTransactions > 1024) {
-    tMaxNoOfTransactions = 1024;
-  }//if
+
+  tMaxNoOfTransactions = aMaxNoOfTransactions;
   theMaxNoOfTransactions = tMaxNoOfTransactions;
-  
-  thePreparedTransactionsArray = new NdbConnection* [tMaxNoOfTransactions];
-  theSentTransactionsArray = new NdbConnection* [tMaxNoOfTransactions];
-  theCompletedTransactionsArray = new NdbConnection* [tMaxNoOfTransactions];
+  theRemainingStartTransactions= tMaxNoOfTransactions;  
+  thePreparedTransactionsArray = new NdbTransaction* [tMaxNoOfTransactions];
+  theSentTransactionsArray = new NdbTransaction* [tMaxNoOfTransactions];
+  theCompletedTransactionsArray = new NdbTransaction* [tMaxNoOfTransactions];
   
   if ((thePreparedTransactionsArray == NULL) ||
       (theSentTransactionsArray == NULL) ||
@@ -143,15 +141,6 @@ Ndb::init(int aMaxNoOfTransactions)
 error_handler:
   ndbout << "error_handler" << endl;
   releaseTransactionArrays();
-  while ( theConIdleList != NULL )
-    freeNdbCon();
-  while ( theSignalIdleList != NULL )
-    freeSignal();
-  while (theRecAttrIdleList != NULL)
-    freeRecAttr(); 
-  while (theOpIdleList != NULL)
-    freeOperation();
-  
   delete theDictionary;
   TransporterFacade::instance()->close(theNdbBlockNumber, 0);
   DBUG_RETURN(-1);
@@ -272,11 +261,11 @@ Ndb::abortTransactionsAfterNodeFailure(Uint16 aNodeId)
 {  
   Uint32 tNoSentTransactions = theNoOfSentTransactions;
   for (int i = tNoSentTransactions - 1; i >= 0; i--) {
-    NdbConnection* localCon = theSentTransactionsArray[i];
+    NdbTransaction* localCon = theSentTransactionsArray[i];
     if (localCon->getConnectedNodeId() == aNodeId) {
-      const NdbConnection::SendStatusType sendStatus = localCon->theSendStatus;
-      if (sendStatus == NdbConnection::sendTC_OP || 
-	  sendStatus == NdbConnection::sendTC_COMMIT) {
+      const NdbTransaction::SendStatusType sendStatus = localCon->theSendStatus;
+      if (sendStatus == NdbTransaction::sendTC_OP || 
+	  sendStatus == NdbTransaction::sendTC_COMMIT) {
         /*
         A transaction was interrupted in the prepare phase by a node
         failure. Since the transaction was not found in the phase
@@ -284,13 +273,13 @@ Ndb::abortTransactionsAfterNodeFailure(Uint16 aNodeId)
         we report a normal node failure abort.
         */
 	localCon->setOperationErrorCodeAbort(4010);
-        localCon->theCompletionStatus = NdbConnection::CompletedFailure;
-      } else if (sendStatus == NdbConnection::sendTC_ROLLBACK) {
+        localCon->theCompletionStatus = NdbTransaction::CompletedFailure;
+      } else if (sendStatus == NdbTransaction::sendTC_ROLLBACK) {
         /*
         We aimed for abort and abort we got even if it was by a node
         failure. We will thus report it as a success.
         */
-        localCon->theCompletionStatus = NdbConnection::CompletedSuccess;
+        localCon->theCompletionStatus = NdbTransaction::CompletedSuccess;
       } else {
 #ifdef VM_TRACE
         printState("abortTransactionsAfterNodeFailure %x", this);
@@ -302,7 +291,8 @@ Ndb::abortTransactionsAfterNodeFailure(Uint16 aNodeId)
       intact since the node was failing and they were aborted. Thus we
       set commit state to Aborted and set state to release on close.
       */
-      localCon->theCommitStatus = NdbConnection::Aborted;
+      localCon->theReturnStatus = NdbTransaction::ReturnFailure;
+      localCon->theCommitStatus = NdbTransaction::Aborted;
       localCon->theReleaseOnClose = true;
       completedTransaction(localCon);
     }
@@ -325,7 +315,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 {
   NdbOperation* tOp;
   NdbIndexOperation* tIndexOp;
-  NdbConnection* tCon;
+  NdbTransaction* tCon;
   int tReturnCode = -1;
   const Uint32* tDataPtr = aSignal->getDataPtr();
   const Uint32 tWaitState = theImpl->theWaiter.m_state;
@@ -355,14 +345,14 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 
       tCon = void2con(tFirstDataPtr);
       if ((tCon->checkMagicNumber() == 0) &&
-          (tCon->theSendStatus == NdbConnection::sendTC_OP)) {
+          (tCon->theSendStatus == NdbTransaction::sendTC_OP)) {
         tReturnCode = tCon->receiveTCKEYCONF(keyConf, tLen);
         if (tReturnCode != -1) {
           completedTransaction(tCon);
         }//if
 
 	if(TcKeyConf::getMarkerFlag(keyConf->confInfo)){
-	  NdbConnection::sendTC_COMMIT_ACK(theCommitAckSignal,
+	  NdbTransaction::sendTC_COMMIT_ACK(theCommitAckSignal,
 					   keyConf->transId1, 
 					   keyConf->transId2,
 					   aTCRef);
@@ -387,24 +377,24 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 	com = tRec->execTRANSID_AI(tDataPtr + TransIdAI::HeaderLength, 
 				   tLen - TransIdAI::HeaderLength);
       }
+
+      if(com == 0)
+	return;
       
-      if(com == 1){
-	switch(tRec->getType()){
-	case NdbReceiver::NDB_OPERATION:
-	case NdbReceiver::NDB_INDEX_OPERATION:
-	  if(tCon->OpCompleteSuccess() != -1){
-	    completedTransaction(tCon);
-	    return;
-	  }
-	  break;
-	case NdbReceiver::NDB_SCANRECEIVER:
-	  tCon->theScanningOp->receiver_delivered(tRec);
-	  theImpl->theWaiter.m_state = (((WaitSignalType) tWaitState) == WAIT_SCAN ? 
-			       (Uint32) NO_WAIT : tWaitState);
-	  break;
-	default:
-	  goto InvalidSignal;
+      switch(tRec->getType()){
+      case NdbReceiver::NDB_OPERATION:
+      case NdbReceiver::NDB_INDEX_OPERATION:
+	if(tCon->OpCompleteSuccess() != -1){
+	  completedTransaction(tCon);
 	}
+	return;
+      case NdbReceiver::NDB_SCANRECEIVER:
+	tCon->theScanningOp->receiver_delivered(tRec);
+	theImpl->theWaiter.m_state = (((WaitSignalType) tWaitState) == WAIT_SCAN ? 
+				      (Uint32) NO_WAIT : tWaitState);
+	break;
+      default:
+	goto InvalidSignal;
       }
       break;
     } else {
@@ -426,8 +416,8 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 	if (tOp->checkMagicNumber(false) == 0) {
 	  tCon = tOp->theNdbCon;
 	  if (tCon != NULL) {
-	    if ((tCon->theSendStatus == NdbConnection::sendTC_OP) ||
-		(tCon->theSendStatus == NdbConnection::sendTC_COMMIT)) {
+	    if ((tCon->theSendStatus == NdbTransaction::sendTC_OP) ||
+		(tCon->theSendStatus == NdbTransaction::sendTC_COMMIT)) {
 	      tReturnCode = tCon->receiveTCKEY_FAILCONF(failConf);
 	      if (tReturnCode != -1) {
 		completedTransaction(tCon);
@@ -441,7 +431,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 #endif
       }
       if(tFirstData & 1){
-	NdbConnection::sendTC_COMMIT_ACK(theCommitAckSignal,
+	NdbTransaction::sendTC_COMMIT_ACK(theCommitAckSignal,
 					 failConf->transId1, 
 					 failConf->transId2,
 					 aTCRef);
@@ -456,8 +446,8 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 	if (tOp->checkMagicNumber(false) == 0) {
 	  tCon = tOp->theNdbCon;
 	  if (tCon != NULL) {
-	    if ((tCon->theSendStatus == NdbConnection::sendTC_OP) ||
-		(tCon->theSendStatus == NdbConnection::sendTC_ROLLBACK)) {
+	    if ((tCon->theSendStatus == NdbTransaction::sendTC_OP) ||
+		(tCon->theSendStatus == NdbTransaction::sendTC_ROLLBACK)) {
 	      tReturnCode = tCon->receiveTCKEY_FAILREF(aSignal);
 	      if (tReturnCode != -1) {
 		completedTransaction(tCon);
@@ -482,7 +472,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       if (tOp->checkMagicNumber() == 0) {
 	tCon = tOp->theNdbCon;
 	if (tCon != NULL) {
-	  if (tCon->theSendStatus == NdbConnection::sendTC_OP) {
+	  if (tCon->theSendStatus == NdbTransaction::sendTC_OP) {
 	    tReturnCode = tOp->receiveTCKEYREF(aSignal);
 	    if (tReturnCode != -1) {
 	      completedTransaction(tCon);
@@ -505,14 +495,14 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       
       tCon = void2con(tFirstDataPtr);
       if ((tCon->checkMagicNumber() == 0) &&
-	  (tCon->theSendStatus == NdbConnection::sendTC_COMMIT)) {
+	  (tCon->theSendStatus == NdbTransaction::sendTC_COMMIT)) {
 	tReturnCode = tCon->receiveTC_COMMITCONF(commitConf);
 	if (tReturnCode != -1) {
 	  completedTransaction(tCon);
 	}//if
 
 	if(tFirstData & 1){
-	  NdbConnection::sendTC_COMMIT_ACK(theCommitAckSignal,
+	  NdbTransaction::sendTC_COMMIT_ACK(theCommitAckSignal,
 					   commitConf->transId1, 
 					   commitConf->transId2,
 					   aTCRef);
@@ -530,7 +520,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 
       tCon = void2con(tFirstDataPtr);
       if ((tCon->checkMagicNumber() == 0) &&
-	  (tCon->theSendStatus == NdbConnection::sendTC_COMMIT)) {
+	  (tCon->theSendStatus == NdbTransaction::sendTC_COMMIT)) {
 	tReturnCode = tCon->receiveTC_COMMITREF(aSignal);
 	if (tReturnCode != -1) {
 	  completedTransaction(tCon);
@@ -545,7 +535,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 
       tCon = void2con(tFirstDataPtr);
       if ((tCon->checkMagicNumber() == 0) &&
-	  (tCon->theSendStatus == NdbConnection::sendTC_ROLLBACK)) {
+	  (tCon->theSendStatus == NdbTransaction::sendTC_ROLLBACK)) {
 	tReturnCode = tCon->receiveTCROLLBACKCONF(aSignal);
 	if (tReturnCode != -1) {
 	  completedTransaction(tCon);
@@ -560,7 +550,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 
       tCon = void2con(tFirstDataPtr);
       if ((tCon->checkMagicNumber() == 0) &&
-	  (tCon->theSendStatus == NdbConnection::sendTC_ROLLBACK)) {
+	  (tCon->theSendStatus == NdbTransaction::sendTC_ROLLBACK)) {
 	tReturnCode = tCon->receiveTCROLLBACKREF(aSignal);
 	if (tReturnCode != -1) {
 	  completedTransaction(tCon);
@@ -797,7 +787,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
     const BlockReference aTCRef = aSignal->theSendersBlockRef;
     tCon = void2con(tFirstDataPtr);
     if ((tCon->checkMagicNumber() == 0) &&
-	(tCon->theSendStatus == NdbConnection::sendTC_OP)) {
+	(tCon->theSendStatus == NdbTransaction::sendTC_OP)) {
       tReturnCode = tCon->receiveTCINDXCONF(indxConf, tLen);
       if (tReturnCode != -1) { 
 	completedTransaction(tCon);
@@ -805,7 +795,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
     }//if
     
     if(TcIndxConf::getMarkerFlag(indxConf->confInfo)){
-      NdbConnection::sendTC_COMMIT_ACK(theCommitAckSignal,
+      NdbTransaction::sendTC_COMMIT_ACK(theCommitAckSignal,
 				       indxConf->transId1, 
 				       indxConf->transId2,
 				       aTCRef);
@@ -820,7 +810,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
     if (tIndexOp->checkMagicNumber() == 0) {
       tCon = tIndexOp->theNdbCon;
       if (tCon != NULL) {
-	if (tCon->theSendStatus == NdbConnection::sendTC_OP) {
+	if (tCon->theSendStatus == NdbTransaction::sendTC_OP) {
 	  tReturnCode = tIndexOp->receiveTCINDXREF(aSignal);
 	  if (tReturnCode != -1) {
 	    completedTransaction(tCon);
@@ -861,7 +851,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 
 
 /*****************************************************************************
-void completedTransaction(NdbConnection* aCon);
+void completedTransaction(NdbTransaction* aCon);
 
 Remark:   One transaction has been completed.
           Remove it from send array and put it into the completed
@@ -869,14 +859,14 @@ Remark:   One transaction has been completed.
           up a poller.
 ******************************************************************************/
 void	
-Ndb::completedTransaction(NdbConnection* aCon)
+Ndb::completedTransaction(NdbTransaction* aCon)
 {
   Uint32 tTransArrayIndex = aCon->theTransArrayIndex;
   Uint32 tNoSentTransactions = theNoOfSentTransactions;
   Uint32 tNoCompletedTransactions = theNoOfCompletedTransactions;
-  if ((tNoSentTransactions > 0) && (aCon->theListState == NdbConnection::InSendList) &&
+  if ((tNoSentTransactions > 0) && (aCon->theListState == NdbTransaction::InSendList) &&
       (tTransArrayIndex < tNoSentTransactions)) {
-    NdbConnection* tMoveCon = theSentTransactionsArray[tNoSentTransactions - 1];
+    NdbTransaction* tMoveCon = theSentTransactionsArray[tNoSentTransactions - 1];
 
     theCompletedTransactionsArray[tNoCompletedTransactions] = aCon;
     aCon->theTransArrayIndex = tNoCompletedTransactions;
@@ -888,7 +878,7 @@ Ndb::completedTransaction(NdbConnection* aCon)
     theNoOfCompletedTransactions = tNoCompletedTransactions + 1;
 
     theNoOfSentTransactions = tNoSentTransactions - 1;
-    aCon->theListState = NdbConnection::InCompletedList;
+    aCon->theListState = NdbTransaction::InCompletedList;
     aCon->handleExecuteCompletion();
     if ((theMinNoOfEventsToWakeUp != 0) &&
         (theNoOfCompletedTransactions >= theMinNoOfEventsToWakeUp)) {
@@ -909,12 +899,12 @@ Ndb::completedTransaction(NdbConnection* aCon)
 }//Ndb::completedTransaction()
 
 /*****************************************************************************
-void reportCallback(NdbConnection** aCopyArray, Uint32 aNoOfCompletedTrans);
+void reportCallback(NdbTransaction** aCopyArray, Uint32 aNoOfCompletedTrans);
 
 Remark:   Call the callback methods of the completed transactions.
 ******************************************************************************/
 void	
-Ndb::reportCallback(NdbConnection** aCopyArray, Uint32 aNoOfCompletedTrans)
+Ndb::reportCallback(NdbTransaction** aCopyArray, Uint32 aNoOfCompletedTrans)
 {
   Uint32         i;
   if (aNoOfCompletedTrans > 0) {
@@ -923,7 +913,7 @@ Ndb::reportCallback(NdbConnection** aCopyArray, Uint32 aNoOfCompletedTrans)
       NdbAsynchCallback aCallback = aCopyArray[i]->theCallbackFunction;
       int tResult = 0;
       if (aCallback != NULL) {
-        if (aCopyArray[i]->theReturnStatus == NdbConnection::ReturnFailure) {
+        if (aCopyArray[i]->theReturnStatus == NdbTransaction::ReturnFailure) {
           tResult = -1;
         }//if
         (*aCallback)(tResult, aCopyArray[i], anyObject);
@@ -933,13 +923,13 @@ Ndb::reportCallback(NdbConnection** aCopyArray, Uint32 aNoOfCompletedTrans)
 }//Ndb::reportCallback()
 
 /*****************************************************************************
-Uint32 pollCompleted(NdbConnection** aCopyArray);
+Uint32 pollCompleted(NdbTransaction** aCopyArray);
 
 Remark:   Transfer the data from the completed transaction to a local array.
           This support is used by a number of the poll-methods.
 ******************************************************************************/
 Uint32	
-Ndb::pollCompleted(NdbConnection** aCopyArray)
+Ndb::pollCompleted(NdbTransaction** aCopyArray)
 {
   check_send_timeout();
   Uint32         i;
@@ -947,13 +937,13 @@ Ndb::pollCompleted(NdbConnection** aCopyArray)
   if (tNoCompletedTransactions > 0) {
     for (i = 0; i < tNoCompletedTransactions; i++) {
       aCopyArray[i] = theCompletedTransactionsArray[i];
-      if (aCopyArray[i]->theListState != NdbConnection::InCompletedList) {
+      if (aCopyArray[i]->theListState != NdbTransaction::InCompletedList) {
         ndbout << "pollCompleted error ";
         ndbout << (int) aCopyArray[i]->theListState << endl;
 	abort();
       }//if
       theCompletedTransactionsArray[i] = NULL;
-      aCopyArray[i]->theListState = NdbConnection::NotInList;
+      aCopyArray[i]->theListState = NdbTransaction::NotInList;
     }//for
   }//if
   theNoOfCompletedTransactions = 0;
@@ -963,24 +953,26 @@ Ndb::pollCompleted(NdbConnection** aCopyArray)
 void
 Ndb::check_send_timeout()
 {
+  Uint32 timeout = TransporterFacade::instance()->m_waitfor_timeout;
   NDB_TICKS current_time = NdbTick_CurrentMillisecond();
   if (current_time - the_last_check_time > 1000) {
     the_last_check_time = current_time;
     Uint32 no_of_sent = theNoOfSentTransactions;
     for (Uint32 i = 0; i < no_of_sent; i++) {
-      NdbConnection* a_con = theSentTransactionsArray[i];
-      if ((current_time - a_con->theStartTransTime) >
-          WAITFOR_RESPONSE_TIMEOUT) {
+      NdbTransaction* a_con = theSentTransactionsArray[i];
+      if ((current_time - a_con->theStartTransTime) > timeout)
+      {
 #ifdef VM_TRACE
         a_con->printState();
 	Uint32 t1 = a_con->theTransactionId;
 	Uint32 t2 = a_con->theTransactionId >> 32;
-	ndbout_c("[%.8x %.8x]", t1, t2);
-	abort();
+	ndbout_c("4012 [%.8x %.8x]", t1, t2);
+	//abort();
 #endif
+        a_con->theReleaseOnClose = true;
         a_con->setOperationErrorCodeAbort(4012);
-        a_con->theCommitStatus = NdbConnection::Aborted;
-        a_con->theCompletionStatus = NdbConnection::CompletedFailure;
+	a_con->theCommitStatus = NdbTransaction::NeedAbort;
+        a_con->theCompletionStatus = NdbTransaction::CompletedFailure;
         a_con->handleExecuteCompletion();
         remove_sent_list(i);
         insert_completed_list(a_con);
@@ -996,7 +988,7 @@ Ndb::remove_sent_list(Uint32 list_index)
 {
   Uint32 last_index = theNoOfSentTransactions - 1;
   if (list_index < last_index) {
-    NdbConnection* t_con = theSentTransactionsArray[last_index];
+    NdbTransaction* t_con = theSentTransactionsArray[last_index];
     theSentTransactionsArray[list_index] = t_con;
   }//if
   theNoOfSentTransactions = last_index;
@@ -1004,23 +996,23 @@ Ndb::remove_sent_list(Uint32 list_index)
 }
 
 Uint32
-Ndb::insert_completed_list(NdbConnection* a_con)
+Ndb::insert_completed_list(NdbTransaction* a_con)
 {
   Uint32 no_of_comp = theNoOfCompletedTransactions;
   theCompletedTransactionsArray[no_of_comp] = a_con;
   theNoOfCompletedTransactions = no_of_comp + 1;
-  a_con->theListState = NdbConnection::InCompletedList;
+  a_con->theListState = NdbTransaction::InCompletedList;
   a_con->theTransArrayIndex = no_of_comp;
   return no_of_comp;
 }
 
 Uint32
-Ndb::insert_sent_list(NdbConnection* a_con)
+Ndb::insert_sent_list(NdbTransaction* a_con)
 {
   Uint32 no_of_sent = theNoOfSentTransactions;
   theSentTransactionsArray[no_of_sent] = a_con;
   theNoOfSentTransactions = no_of_sent + 1;
-  a_con->theListState = NdbConnection::InSendList;
+  a_con->theListState = NdbTransaction::InSendList;
   a_con->theTransArrayIndex = no_of_sent;
   return no_of_sent;
 }
@@ -1052,16 +1044,16 @@ Ndb::sendPrepTrans(int forceSend)
   TransporterFacade* tp = TransporterFacade::instance();
   Uint32 no_of_prep_trans = theNoOfPreparedTransactions;
   for (i = 0; i < no_of_prep_trans; i++) {
-    NdbConnection * a_con = thePreparedTransactionsArray[i];
+    NdbTransaction * a_con = thePreparedTransactionsArray[i];
     thePreparedTransactionsArray[i] = NULL;
     Uint32 node_id = a_con->getConnectedNodeId();
     if ((tp->getNodeSequence(node_id) == a_con->theNodeSequence) &&
          tp->get_node_alive(node_id) ||
          (tp->get_node_stopping(node_id) && 
-         ((a_con->theSendStatus == NdbConnection::sendABORT) ||
-          (a_con->theSendStatus == NdbConnection::sendABORTfail) ||
-          (a_con->theSendStatus == NdbConnection::sendCOMMITstate) ||
-          (a_con->theSendStatus == NdbConnection::sendCompleted)))) {
+         ((a_con->theSendStatus == NdbTransaction::sendABORT) ||
+          (a_con->theSendStatus == NdbTransaction::sendABORTfail) ||
+          (a_con->theSendStatus == NdbTransaction::sendCOMMITstate) ||
+          (a_con->theSendStatus == NdbTransaction::sendCompleted)))) {
       /*
       We will send if
       1) Node is alive and sequences are correct OR
@@ -1093,13 +1085,13 @@ Ndb::sendPrepTrans(int forceSend)
         again and will thus set the state to Aborted to avoid a more or
         less eternal loop of tries.
         */
-        if (a_con->theSendStatus == NdbConnection::sendOperations) {
+        if (a_con->theSendStatus == NdbTransaction::sendOperations) {
           a_con->setOperationErrorCodeAbort(4021);
-          a_con->theCommitStatus = NdbConnection::NeedAbort;
+          a_con->theCommitStatus = NdbTransaction::NeedAbort;
           TRACE_DEBUG("Send buffer full and sendOperations");
         } else {
           a_con->setOperationErrorCodeAbort(4026);
-          a_con->theCommitStatus = NdbConnection::Aborted;
+          a_con->theCommitStatus = NdbTransaction::Aborted;
           TRACE_DEBUG("Send buffer full, set state to Aborted");
         }//if
       }//if
@@ -1116,7 +1108,7 @@ Ndb::sendPrepTrans(int forceSend)
         */
         TRACE_DEBUG("Abort a transaction when stopping a node");
         a_con->setOperationErrorCodeAbort(4023);
-        a_con->theCommitStatus = NdbConnection::NeedAbort;
+        a_con->theCommitStatus = NdbTransaction::NeedAbort;
       } else {
         /*
         The node is hard dead and we cannot continue. We will also release
@@ -1126,10 +1118,11 @@ Ndb::sendPrepTrans(int forceSend)
         a_con->setOperationErrorCodeAbort(4025);
         a_con->theReleaseOnClose = true;
         a_con->theTransactionIsStarted = false;
-        a_con->theCommitStatus = NdbConnection::Aborted;
+        a_con->theCommitStatus = NdbTransaction::Aborted;
       }//if
     }//if
-    a_con->theCompletionStatus = NdbConnection::CompletedFailure;
+    a_con->theReturnStatus = NdbTransaction::ReturnFailure;
+    a_con->theCompletionStatus = NdbTransaction::CompletedFailure;
     a_con->handleExecuteCompletion();
     insert_completed_list(a_con);
   }//for
@@ -1203,7 +1196,7 @@ Remark:   First send all prepared operations and then check if there are any
 int	
 Ndb::sendPollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup, int forceSend)
 {
-  NdbConnection* tConArray[1024];
+  NdbTransaction* tConArray[1024];
   Uint32         tNoCompletedTransactions;
 
   //theCurrentConnectCounter = 0;
@@ -1236,7 +1229,7 @@ Remark:   Check if there are any transactions already completed. Wait for not
 int	
 Ndb::pollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup)
 {
-  NdbConnection* tConArray[1024];
+  NdbTransaction* tConArray[1024];
   Uint32         tNoCompletedTransactions;
 
   //theCurrentConnectCounter = 0;
@@ -1343,7 +1336,7 @@ Ndb::sendRecSignal(Uint16 node_id,
 }//Ndb::sendRecSignal()
 
 void
-NdbConnection::sendTC_COMMIT_ACK(NdbApiSignal * aSignal,
+NdbTransaction::sendTC_COMMIT_ACK(NdbApiSignal * aSignal,
 				 Uint32 transId1, Uint32 transId2, 
 				 Uint32 aTCRef){
 #ifdef MARKER_TRACE
@@ -1361,6 +1354,6 @@ NdbConnection::sendTC_COMMIT_ACK(NdbApiSignal * aSignal,
   Uint32 * dataPtr = aSignal->getDataPtrSend();
   dataPtr[0] = transId1;
   dataPtr[1] = transId2;
-
-  tp->sendSignal(aSignal, refToNode(aTCRef));
+  
+  tp->sendSignalUnCond(aSignal, refToNode(aTCRef));
 }
