@@ -35,6 +35,7 @@
 #include <ndb_version.h>
 #include <SignalLoggerManager.hpp>
 #include <kernel/ndb_limits.h>
+#include <signaldata/AlterTable.hpp>
 
 //#define REPORT_TRANSPORTER
 //#define API_TRACE;
@@ -62,13 +63,16 @@ TransporterFacade* TransporterFacade::theFacadeInstance = NULL;
  *****************************************************************************/
 
 void
-reportError(void * callbackObj, NodeId nodeId, TransporterError errorCode){
+reportError(void * callbackObj, NodeId nodeId,
+	    TransporterError errorCode, const char *info)
+{
 #ifdef REPORT_TRANSPORTER
-  ndbout_c("REPORT_TRANSP: reportError (nodeId=%d, errorCode=%d)", 
-	   (int)nodeId, (int)errorCode);
+  ndbout_c("REPORT_TRANSP: reportError (nodeId=%d, errorCode=%d) %s", 
+	   (int)nodeId, (int)errorCode, info ? info : "");
 #endif
-  if(errorCode & 0x8000) {
-    ndbout_c("reportError (%d, %d)\n", (int)nodeId, (int)errorCode);
+  if(errorCode & TE_DO_DISCONNECT) {
+    ndbout_c("reportError (%d, %d) %s", (int)nodeId, (int)errorCode,
+	     info ? info : "");
     ((TransporterFacade*)(callbackObj))->doDisconnect(nodeId);
   }
 }
@@ -126,6 +130,10 @@ reportDisconnect(void * callbackObj, NodeId nodeId, Uint32 error){
   //TransporterFacade::instance()->reportDisconnected(nodeId);
 }
 
+void
+transporter_recv_from(void * callbackObj, NodeId nodeId){
+  ((TransporterFacade*)(callbackObj))->hb_received(nodeId);
+}
 
 /****************************************************************************
  * 
@@ -305,6 +313,17 @@ execute(void * callbackObj, SignalHeader * const header,
 	 theFacade->theArbitMgr->doStop(theData);
        break;
 
+     case GSN_ALTER_TABLE_REP:
+     {
+       const AlterTableRep* rep = (const AlterTableRep*)theData;
+       theFacade->m_globalDictCache.lock();
+       theFacade->m_globalDictCache.
+	 alter_table_rep((const char*)ptr[0].p, 
+			 rep->tableId,
+			 rep->tableVersion,
+			 rep->changeType == AlterTableRep::CT_ALTERED);
+       theFacade->m_globalDictCache.unlock();
+     }
      default:
        break;
        
@@ -517,43 +536,45 @@ TransporterFacade::init(Uint32 nodeId, const ndb_mgm_configuration* props)
   iter.first();
   theClusterMgr->init(iter);
   
-  /**
-   * Unless there is a "Name", the initiated transporter is within 
-   * an NDB Cluster.  (If "Name" is defined, then the transporter
-   * is used to connect to a different system, i.e. NDB Cluster.)
-   */
-#if 0  
-  if (!props->contains("Name")) {
-#endif
-    iter.first();
-    if(iter.find(CFG_NODE_ID, nodeId)){
-      TRP_DEBUG( "Node info missing from config." );
-      DBUG_RETURN(false);
-    }
-    
-    Uint32 rank = 0;
-    if(!iter.get(CFG_NODE_ARBIT_RANK, &rank) && rank>0){
-      theArbitMgr = new ArbitMgr(* this);
-      theArbitMgr->setRank(rank);
-      Uint32 delay = 0;
-      iter.get(CFG_NODE_ARBIT_DELAY, &delay);
-      theArbitMgr->setDelay(delay);
-    }
-    Uint32 scan_batch_size= 0;
-    if (!iter.get(CFG_MAX_SCAN_BATCH_SIZE, &scan_batch_size)) {
-      m_scan_batch_size= scan_batch_size;
-    }
-    Uint32 batch_byte_size= 0;
-    if (!iter.get(CFG_BATCH_BYTE_SIZE, &batch_byte_size)) {
-      m_batch_byte_size= batch_byte_size;
-    }
-    Uint32 batch_size= 0;
-    if (!iter.get(CFG_BATCH_SIZE, &batch_size)) {
-      m_batch_size= batch_size;
-    }
-#if 0
+  iter.first();
+  if(iter.find(CFG_NODE_ID, nodeId)){
+    TRP_DEBUG( "Node info missing from config." );
+    DBUG_RETURN(false);
   }
-#endif
+  
+  Uint32 rank = 0;
+  if(!iter.get(CFG_NODE_ARBIT_RANK, &rank) && rank>0){
+    theArbitMgr = new ArbitMgr(* this);
+    theArbitMgr->setRank(rank);
+    Uint32 delay = 0;
+    iter.get(CFG_NODE_ARBIT_DELAY, &delay);
+    theArbitMgr->setDelay(delay);
+  }
+  Uint32 scan_batch_size= 0;
+  if (!iter.get(CFG_MAX_SCAN_BATCH_SIZE, &scan_batch_size)) {
+    m_scan_batch_size= scan_batch_size;
+  }
+  Uint32 batch_byte_size= 0;
+  if (!iter.get(CFG_BATCH_BYTE_SIZE, &batch_byte_size)) {
+    m_batch_byte_size= batch_byte_size;
+  }
+  Uint32 batch_size= 0;
+  if (!iter.get(CFG_BATCH_SIZE, &batch_size)) {
+    m_batch_size= batch_size;
+  }
+  
+  Uint32 timeout = 120000;
+  iter.first();
+  for (iter.first(); iter.valid(); iter.next())
+  {
+    Uint32 tmp1 = 0, tmp2 = 0;
+    iter.get(CFG_DB_TRANSACTION_CHECK_INTERVAL, &tmp1);
+    iter.get(CFG_DB_TRANSACTION_DEADLOCK_TIMEOUT, &tmp2);
+    tmp1 += tmp2;
+    if (tmp1 > timeout)
+      timeout = tmp1;
+  }
+  m_waitfor_timeout = timeout;
   
   if (!theTransporterRegistry->start_service(m_socket_server)){
     ndbout_c("Unable to start theTransporterRegistry->start_service");
@@ -792,7 +813,7 @@ TransporterFacade::sendSignal(NdbApiSignal * aSignal, NodeId aNode){
       LinearSectionPtr ptr[3];
       signalLogger.sendSignal(* aSignal,
 			      1,
-			      aSignal->getDataPtr(),
+			      tDataPtr,
 			      aNode, ptr, 0);
       signalLogger.flushSignalLog();
       aSignal->theSendersBlockRef = tmp;
@@ -819,6 +840,7 @@ TransporterFacade::sendSignal(NdbApiSignal * aSignal, NodeId aNode){
 
 int
 TransporterFacade::sendSignalUnCond(NdbApiSignal * aSignal, NodeId aNode){
+  Uint32* tDataPtr = aSignal->getDataPtrSend();
 #ifdef API_TRACE
   if(setSignalLog() && TRACE_GSN(aSignal->theVerId_signalNumber)){
     Uint32 tmp = aSignal->theSendersBlockRef;
@@ -826,7 +848,7 @@ TransporterFacade::sendSignalUnCond(NdbApiSignal * aSignal, NodeId aNode){
     LinearSectionPtr ptr[3];
     signalLogger.sendSignal(* aSignal,
 			    0,
-			    aSignal->getDataPtr(),
+			    tDataPtr,
 			    aNode, ptr, 0);
     signalLogger.flushSignalLog();
     aSignal->theSendersBlockRef = tmp;
@@ -837,7 +859,7 @@ TransporterFacade::sendSignalUnCond(NdbApiSignal * aSignal, NodeId aNode){
          (aSignal->theReceiversBlockNumber != 0));
   SendStatus ss = theTransporterRegistry->prepareSend(aSignal, 
 						      0, 
-						      aSignal->getDataPtr(), 
+						      tDataPtr,
 						      aNode, 
 						      0);
   
