@@ -316,11 +316,16 @@ int runScanReadIndex(NDBT_Context* ctx, NDBT_Step* step){
   while (pIdx && i<loops && !ctx->isTestStopped()) {
     g_info << i << ": ";
     bool sort = (rand() % 100) > 50 ? true : false;
+    bool desc = (rand() % 100) > 50 ? true : false;
+    desc = false;       // random causes too many deadlocks
+    int scan_flags =
+      (NdbScanOperation::SF_OrderBy & -(int)sort) |
+      (NdbScanOperation::SF_Descending & -(int)desc);
     NdbOperation::LockMode lm = (NdbOperation::LockMode)(rand() % 3);
     if (hugoTrans.scanReadRecords(GETNDB(step), pIdx,
 				  records, abort, parallelism,
 				  lm,
-				  sort) != 0){
+				  scan_flags) != 0){
       return NDBT_FAILED;
     }
     i++;
@@ -333,6 +338,8 @@ int runScanReadCommitted(NDBT_Context* ctx, NDBT_Step* step){
   int records = ctx->getNumRecords();
   int parallelism = ctx->getProperty("Parallelism", 240);
   int abort = ctx->getProperty("AbortProb", 5);
+  bool tupScan = ctx->getProperty("TupScan");
+  int scan_flags = (NdbScanOperation::SF_TupScan & -(int)tupScan);
 
   int i = 0;
   HugoTransactions hugoTrans(*ctx->getTab());
@@ -340,7 +347,8 @@ int runScanReadCommitted(NDBT_Context* ctx, NDBT_Step* step){
     g_info << i << ": ";
     if (hugoTrans.scanReadRecords(GETNDB(step), records, 
 				  abort, parallelism, 
-				  NdbOperation::LM_CommittedRead) != 0){
+				  NdbOperation::LM_CommittedRead,
+                                  scan_flags) != 0){
       return NDBT_FAILED;
     }
     i++;
@@ -1013,8 +1021,7 @@ int runScanRestart(NDBT_Context* ctx, NDBT_Step* step){
       return NDBT_FAILED;
     }
     
-    NdbResultSet* rs = pOp->readTuples();
-    if( rs == 0 ) {
+    if( pOp->readTuples() ) {
       ERR(pCon->getNdbError());
       return NDBT_FAILED;
     }
@@ -1042,7 +1049,7 @@ int runScanRestart(NDBT_Context* ctx, NDBT_Step* step){
 
     int res;
     int row = 0;
-    while(row < record && (res = rs->nextResult()) == 0) {
+    while(row < record && (res = pOp->nextResult()) == 0) {
       if(calc.verifyRowValues(&tmpRow) != 0){
 	abort();
 	return NDBT_FAILED;
@@ -1055,14 +1062,14 @@ int runScanRestart(NDBT_Context* ctx, NDBT_Step* step){
       return NDBT_FAILED;
     }
     g_info << " restarting" << endl;
-    if((res = rs->restart()) != 0){
+    if((res = pOp->restart()) != 0){
       ERR(pCon->getNdbError());
       abort();
       return NDBT_FAILED;
     }      
 
     row = 0;
-    while((res = rs->nextResult()) == 0) {
+    while((res = pOp->nextResult()) == 0) {
       if(calc.verifyRowValues(&tmpRow) != 0){
 	abort();
 	return NDBT_FAILED;
@@ -1080,6 +1087,44 @@ int runScanRestart(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 
+int 
+runScanParallelism(NDBT_Context* ctx, NDBT_Step* step){
+  int loops = ctx->getNumLoops() + 3;
+  int records = ctx->getNumRecords();
+  int abort = ctx->getProperty("AbortProb", 15);
+  
+  Uint32 fib[] = { 1, 2 };
+  Uint32 parallelism = 0; // start with 0
+  int i = 0;
+  HugoTransactions hugoTrans(*ctx->getTab());
+  while (i<loops && !ctx->isTestStopped()) {
+    g_info << i << ": ";
+
+    if (hugoTrans.scanReadRecords(GETNDB(step), records, abort, parallelism,
+				  NdbOperation::LM_Read) != 0){
+      return NDBT_FAILED;
+    }
+    if (hugoTrans.scanReadRecords(GETNDB(step), records, abort, parallelism,
+				  NdbOperation::LM_Exclusive) != 0){
+      return NDBT_FAILED;
+    }
+    if (hugoTrans.scanReadRecords(GETNDB(step), records, abort, parallelism,
+				  NdbOperation::LM_CommittedRead) != 0){
+      return NDBT_FAILED;
+    }
+    if (hugoTrans.scanUpdateRecords(GETNDB(step), records, abort, parallelism)
+	!= 0){
+      return NDBT_FAILED;
+    }
+    i++;
+    parallelism = fib[0];
+    Uint32 next = fib[0] + fib[1];
+    fib[0] = fib[1];
+    fib[1] = next;
+  }
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(testScan);
 TESTCASE("ScanRead", 
 	 "Verify scan requirement: It should be possible "\
@@ -1113,6 +1158,18 @@ TESTCASE("ScanReadCommitted240",
 	 "downgraded to the maximum parallelism value for the current config)"){
   INITIALIZER(runLoadTable);
   TC_PROPERTY("Parallelism", 240);
+  TC_PROPERTY("TupScan", (Uint32)0);
+  STEP(runScanReadCommitted);
+  FINALIZER(runClearTable);
+}
+TESTCASE("ScanTupReadCommitted240", 
+	 "Verify scan requirement: It should be possible to scan read committed with "\
+	 "parallelism, test with parallelism 240(240 would automatically be "\
+	 "downgraded to the maximum parallelism value for the current config). "\
+         "Scans TUP pages directly without using ACC."){
+  INITIALIZER(runLoadTable);
+  TC_PROPERTY("Parallelism", 240);
+  TC_PROPERTY("TupScan", 1);
   STEP(runScanReadCommitted);
   FINALIZER(runClearTable);
 }
@@ -1540,6 +1597,12 @@ TESTCASE("ScanRestart",
   STEP(runScanRestart);
   FINALIZER(runClearTable);
 }
+TESTCASE("ScanParallelism", 
+	 "Test scan with different parallelism"){
+  INITIALIZER(runLoadTable);
+  STEP(runScanParallelism);
+  FINALIZER(runClearTable);
+}
 NDBT_TESTSUITE_END(testScan);
 
 int main(int argc, const char** argv){
