@@ -80,7 +80,12 @@ RestoreMetaData::RestoreMetaData(const char* path, Uint32 nodeId, Uint32 bNo) {
 
 RestoreMetaData::~RestoreMetaData(){
   for(Uint32 i= 0; i < allTables.size(); i++)
-    delete allTables[i];
+  {
+    TableS *table = allTables[i];
+    for(Uint32 j= 0; j < table->m_fragmentInfo.size(); j++)
+      delete table->m_fragmentInfo[j];
+    delete table;
+  }
   allTables.clear();
 }
 
@@ -111,6 +116,9 @@ RestoreMetaData::loadContent()
   }
   if(!readGCPEntry())
     return 0;
+
+  if(!readFragmentInfo())
+    return 0;
   return 1;
 }
 
@@ -192,6 +200,52 @@ RestoreMetaData::readGCPEntry() {
   return true;
 }
 
+bool
+RestoreMetaData::readFragmentInfo()
+{
+  BackupFormat::CtlFile::FragmentInfo fragInfo;
+  TableS * table = 0;
+  Uint32 tableId = RNIL;
+
+  while (buffer_read(&fragInfo, 4, 2) == 2)
+  {
+    fragInfo.SectionType = ntohl(fragInfo.SectionType);
+    fragInfo.SectionLength = ntohl(fragInfo.SectionLength);
+
+    if (fragInfo.SectionType != BackupFormat::FRAGMENT_INFO)
+    {
+      err << "readFragmentInfo invalid section type: " <<
+        fragInfo.SectionType << endl;
+      return false;
+    }
+
+    if (buffer_read(&fragInfo.TableId, (fragInfo.SectionLength-2)*4, 1) != 1)
+    {
+      err << "readFragmentInfo invalid section length: " <<
+        fragInfo.SectionLength << endl;
+      return false;
+    }
+
+    fragInfo.TableId = ntohl(fragInfo.TableId);
+    if (fragInfo.TableId != tableId)
+    {
+      tableId = fragInfo.TableId;
+      table = getTable(tableId);
+    }
+
+    FragmentInfo * tmp = new FragmentInfo;
+    tmp->fragmentNo = ntohl(fragInfo.FragmentNo);
+    tmp->noOfRecords = ntohl(fragInfo.NoOfRecordsLow) +
+      (((Uint64)ntohl(fragInfo.NoOfRecordsHigh)) << 32);
+    tmp->filePosLow = ntohl(fragInfo.FilePosLow);
+    tmp->filePosHigh = ntohl(fragInfo.FilePosHigh);
+
+    table->m_fragmentInfo.push_back(tmp);
+    table->m_noOfRecords += tmp->noOfRecords;
+  }
+  return true;
+}
+
 TableS::TableS(Uint32 version, NdbTableImpl* tableImpl)
   : m_dictTable(tableImpl)
 {
@@ -199,6 +253,7 @@ TableS::TableS(Uint32 version, NdbTableImpl* tableImpl)
   m_noOfNullable = m_nullBitmaskSize = 0;
   m_auto_val_id= ~(Uint32)0;
   m_max_auto_val= 0;
+  m_noOfRecords= 0;
   backupVersion = version;
   
   for (int i = 0; i < tableImpl->getNoOfColumns(); i++)
@@ -646,7 +701,7 @@ bool RestoreDataIterator::readFragmentHeader(int & ret)
   }
 
   info << "_____________________________________________________" << endl
-       << "Restoring data in table: " << m_currentTable->getTableName() 
+       << "Processing data in table: " << m_currentTable->getTableName() 
        << "(" << Header.TableId << ") fragment " 
        << Header.FragmentNo << endl;
   
@@ -869,7 +924,7 @@ operator<<(NdbOut& ndbout, const AttributeS& attr){
     return ndbout;
   }
   
-  NdbRecAttr tmprec;
+  NdbRecAttr tmprec(0);
   tmprec.setup(desc.m_column, (char *)data.void_value);
   ndbout << tmprec;
 
@@ -925,23 +980,17 @@ operator<<(NdbOut& ndbout, const LogEntry& logE)
   return ndbout;
 }
 
+#include <NDBT.hpp>
 
 NdbOut & 
 operator<<(NdbOut& ndbout, const TableS & table){
-  ndbout << endl << "Table: " << table.getTableName() << endl;
-  for (int j = 0; j < table.getNoOfAttributes(); j++) 
-  {
-    const AttributeDesc * desc = table[j];
-    ndbout << desc->m_column->getName() << ": "
-	   << (Uint32) desc->m_column->getType();
-    ndbout << " key: "  << (Uint32) desc->m_column->getPrimaryKey();
-    ndbout << " array: " << desc->arraySize;
-    ndbout << " size: " << desc->size << endl;
-  } // for
+  
+  ndbout << (* (NDBT_Table*)table.m_dictTable) << endl;
   return ndbout;
 }
 
 template class Vector<TableS*>;
 template class Vector<AttributeS*>;
 template class Vector<AttributeDesc*>;
+template class Vector<FragmentInfo*>;
 
