@@ -24,7 +24,10 @@ extern FilteredNdbOut err;
 extern FilteredNdbOut info;
 extern FilteredNdbOut debug;
 
-static void callback(int, NdbConnection*, void*);
+static void callback(int, NdbTransaction*, void*);
+
+extern const char * g_connect_string;
+extern BaseString g_options;
 
 bool
 BackupRestore::init()
@@ -34,7 +37,14 @@ BackupRestore::init()
   if (!m_restore && !m_restore_meta)
     return true;
 
-  m_ndb = new Ndb();
+  m_cluster_connection = new Ndb_cluster_connection(g_connect_string);
+  m_cluster_connection->set_name(g_options.c_str());
+  if(m_cluster_connection->connect(12, 5, 1) != 0)
+  {
+    return false;
+  }
+
+  m_ndb = new Ndb(m_cluster_connection);
 
   if (m_ndb == NULL)
     return false;
@@ -80,6 +90,12 @@ void BackupRestore::release()
     delete [] m_callback;
     m_callback= 0;
   }
+
+  if (m_cluster_connection)
+  {
+    delete m_cluster_connection;
+    m_cluster_connection= 0;
+  }
 }
 
 BackupRestore::~BackupRestore()
@@ -132,9 +148,12 @@ BackupRestore::finalize_table(const TableS & table){
   if (table.have_auto_inc())
   {
     Uint64 max_val= table.get_max_auto_val();
-    Uint64 auto_val= m_ndb->readAutoIncrementValue(get_table(table.m_dictTable));
-    if (max_val+1 > auto_val || auto_val == ~(Uint64)0)
-      ret= m_ndb->setAutoIncrementValue(get_table(table.m_dictTable), max_val+1, false);
+    Uint64 auto_val;
+    int r= m_ndb->readAutoIncrementValue(get_table(table.m_dictTable), auto_val);
+    if (r == -1 && m_ndb->getNdbError().code != 626)
+      ret= false;
+    else if (r == -1 || max_val+1 > auto_val)
+      ret= m_ndb->setAutoIncrementValue(get_table(table.m_dictTable), max_val+1, false) != -1;
   }
   return ret;
 }
@@ -174,6 +193,16 @@ BackupRestore::table(const TableS & table){
 
     copy.setName(split[2].c_str());
 
+    /*
+      update min and max rows to reflect the table, this to
+      ensure that memory is allocated properly in the ndb kernel
+    */
+    copy.setMinRows(table.getNoOfRecords());
+    if (table.getNoOfRecords() > copy.getMaxRows())
+    {
+      copy.setMaxRows(table.getNoOfRecords());
+    }
+
     if (dict->createTable(copy) == -1) 
     {
       err << "Create table " << table.getTableName() << " failed: "
@@ -204,7 +233,7 @@ BackupRestore::endOfTables(){
 
   NdbDictionary::Dictionary* dict = m_ndb->getDictionary();
   for(size_t i = 0; i<m_indexes.size(); i++){
-    const NdbTableImpl & indtab = NdbTableImpl::getImpl(* m_indexes[i]);
+    NdbTableImpl & indtab = NdbTableImpl::getImpl(* m_indexes[i]);
 
     BaseString tmp(indtab.m_primaryTable.c_str());
     Vector<BaseString> split;
@@ -364,7 +393,8 @@ void BackupRestore::tuple_a(restore_callback_t *cb)
     }
 
     // Prepare transaction (the transaction is NOT yet sent to NDB)
-    cb->connection->executeAsynchPrepare(Commit, &callback, cb);
+    cb->connection->executeAsynchPrepare(NdbTransaction::Commit,
+					 &callback, cb);
     m_transactions++;
     return;
   }
@@ -494,7 +524,7 @@ BackupRestore::logEntry(const LogEntry & tup)
   if (!m_restore)
     return;
 
-  NdbConnection * trans = m_ndb->startTransaction();
+  NdbTransaction * trans = m_ndb->startTransaction();
   if (trans == NULL) 
   {
     // Deep shit, TODO: handle the error
@@ -564,7 +594,7 @@ BackupRestore::logEntry(const LogEntry & tup)
     } // if
   }
   
-  const int ret = trans->execute(Commit);
+  const int ret = trans->execute(NdbTransaction::Commit);
   if (ret != 0)
   {
     // Both insert update and delete can fail during log running
@@ -612,12 +642,12 @@ BackupRestore::endOfLogEntrys()
  *              
  *   (This function must have three arguments: 
  *   - The result of the transaction, 
- *   - The NdbConnection object, and 
+ *   - The NdbTransaction object, and 
  *   - A pointer to an arbitrary object.)
  */
 
 static void
-callback(int result, NdbConnection* trans, void* aObject)
+callback(int result, NdbTransaction* trans, void* aObject)
 {
   restore_callback_t *cb = (restore_callback_t *)aObject;
   (cb->restore)->cback(result, cb);
@@ -631,7 +661,7 @@ BackupRestore::tuple(const TupleS & tup)
     return;
   while (1) 
   {
-    NdbConnection * trans = m_ndb->startTransaction();
+    NdbTransaction * trans = m_ndb->startTransaction();
     if (trans == NULL) 
     {
       // Deep shit, TODO: handle the error
@@ -682,7 +712,7 @@ BackupRestore::tuple(const TupleS & tup)
 	else
 	  op->setValue(i, dataPtr, length);
     }
-    int ret = trans->execute(Commit);
+    int ret = trans->execute(NdbTransaction::Commit);
     if (ret != 0)
     {
       ndbout << "execute failed: ";
