@@ -15,8 +15,15 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Compability file */
-#ifdef __GNUC__
-#pragma implementation
+
+#ifdef USE_PRAGMA_IMPLEMENTATION
+#pragma implementation				// gcc: Class implementation
 #endif
 
 #include "mysql_priv.h"
+
+Field *Item_sum_unique_users::create_tmp_field(bool group, TABLE *table,
+                                               uint convert_blob_length)
+{
+  return new Field_long(9,maybe_null,name,table,1);
+}
