@@ -14,6 +14,13 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+/*
+  This file is the net layer API for the MySQL client/server protocol,
+  which is a tightly coupled, proprietary protocol owned by MySQL AB.
+  Any re-implementations of this protocol must also be under GPL
+  unless one has got an license from MySQL AB stating otherwise.
+*/
+
 /*
   Write and read of logical packets to/from socket
 
@@ -26,6 +33,10 @@
   C file.
 */
 
+/*
+  HFTODO this must be hidden if we don't want client capabilities in 
+  embedded library
+ */
 #ifdef __WIN__
 #include <winsock.h>
 #endif
@@ -41,6 +52,17 @@
 #include <signal.h>
 #include <errno.h>
 
+#ifdef __NETWARE__
+#include <sys/select.h>
+#endif
+
+#ifdef EMBEDDED_LIBRARY
+#undef MYSQL_SERVER
+#undef MYSQL_CLIENT
+#define MYSQL_CLIENT
+#endif /*EMBEDDED_LIBRARY */
+
+
 /*
   The following handles the differences when this is linked between the
   client and the server.
@@ -54,7 +76,7 @@
   /* The following is because alarms doesn't work on windows. */
 #define NO_ALARM
 #endif
-  
+
 #ifndef NO_ALARM
 #include "my_pthread.h"
 void sql_print_error(const char *format,...);
@@ -65,16 +87,24 @@ void sql_print_error(const char *format,...);
 #include "thr_alarm.h"
 
 #ifdef MYSQL_SERVER
-#define USE_QUERY_CACHE
+/*
+  The following variables/functions should really not be declared
+  extern, but as it's hard to include mysql_priv.h here, we have to
+  live with this for a while.
+*/
 extern uint test_flags;
-extern void query_cache_insert(NET *net, const char *packet, ulong length);
 extern ulong bytes_sent, bytes_received, net_big_packet_count;
 extern pthread_mutex_t LOCK_bytes_sent , LOCK_bytes_received;
-#else
-#undef statistic_add
-#undef statistic_increment
-#define statistic_add(A,B,C)
-#define statistic_increment(A,B)
+#ifndef MYSQL_INSTANCE_MANAGER
+extern void query_cache_insert(NET *net, const char *packet, ulong length);
+#define USE_QUERY_CACHE
+#define update_statistics(A) A
+#endif /* MYSQL_INSTANCE_MANGER */
+#endif /* defined(MYSQL_SERVER) && !defined(MYSQL_INSTANCE_MANAGER) */
+
+#if !defined(MYSQL_SERVER) || defined(MYSQL_INSTANCE_MANAGER)
+#define update_statistics(A)
+#define thd_increment_bytes_sent(N)
 #endif
 
 #define TEST_BLOCKING		8
@@ -85,7 +115,7 @@ static my_bool net_write_buff(NET *net,const char *packet,ulong len);
 
 	/* Init with packet info */
 
-int my_net_init(NET *net, Vio* vio)
+my_bool my_net_init(NET *net, Vio* vio)
 {
   DBUG_ENTER("my_net_init");
   my_net_local_init(net);			/* Set some limits */
@@ -95,7 +125,7 @@ int my_net_init(NET *net, Vio* vio)
     DBUG_RETURN(1);
   net->buff_end=net->buff+net->max_packet;
   net->vio = vio;
-  net->no_send_ok = 0;
+  net->no_send_ok= net->no_send_eof= net->no_send_error= 0;
   net->error=0; net->return_errno=0; net->return_status=0;
   net->pkt_nr=net->compress_pkt_nr=0;
   net->write_pos=net->read_pos = net->buff;
@@ -104,11 +134,12 @@ int my_net_init(NET *net, Vio* vio)
   net->where_b = net->remain_in_buf=0;
   net->last_errno=0;
   net->query_cache_query=0;
+  net->report_error= 0;
 
   if (vio != 0)					/* If real connection */
   {
     net->fd  = vio_fd(vio);			/* For perl DBI/DBD */
-#if defined(MYSQL_SERVER) && !defined(___WIN__) && !defined(__EMX__) && !defined(OS2)
+#if defined(MYSQL_SERVER) && !defined(__WIN__) && !defined(__EMX__) && !defined(OS2)
     if (!(test_flags & TEST_BLOCKING))
     {
       my_bool old_mode;
@@ -132,7 +163,7 @@ void net_end(NET *net)
 
 /* Realloc the packet buffer */
 
-static my_bool net_realloc(NET *net, ulong length)
+my_bool net_realloc(NET *net, ulong length)
 {
   uchar *buff;
   ulong pkt_length;
@@ -141,10 +172,11 @@ static my_bool net_realloc(NET *net, ulong length)
 
   if (length >= net->max_packet_size)
   {
-    DBUG_PRINT("error",("Packet too large. Max sixe: %lu",
-			net->max_packet_size));
-    net->error=1;
-    net->last_errno=ER_NET_PACKET_TOO_LARGE;
+    DBUG_PRINT("error", ("Packet too large. Max size: %lu",
+               net->max_packet_size));
+    net->error= 1;
+    net->report_error= 1;
+    net->last_errno= ER_NET_PACKET_TOO_LARGE;
     DBUG_RETURN(1);
   }
   pkt_length = (length+IO_SIZE-1) & ~(IO_SIZE-1); 
@@ -156,10 +188,9 @@ static my_bool net_realloc(NET *net, ulong length)
 				 NET_HEADER_SIZE + COMP_HEADER_SIZE,
 				 MYF(MY_WME))))
   {
-    net->error=1;
-#ifdef MYSQL_SERVER
-    net->last_errno=ER_OUT_OF_RESOURCES;
-#endif
+    net->error= 1;
+    net->report_error= 1;
+    net->last_errno= ER_OUT_OF_RESOURCES;
     DBUG_RETURN(1);
   }
   net->buff=net->write_pos=buff;
@@ -167,40 +198,140 @@ static my_bool net_realloc(NET *net, ulong length)
   DBUG_RETURN(0);
 }
 
-	/* Remove unwanted characters from connection */
+
+/*
+  Check if there is any data to be read from the socket
+
+  SYNOPSIS
+    net_data_is_ready()
+    sd   socket descriptor
+
+  DESCRIPTION
+    Check if there is any data to be read from the socket.
+
+  RETURN VALUES
+    0	No data to read
+    1	Data or EOF to read
+    -1  Don't know if data is ready or not
+*/
+
+static int net_data_is_ready(my_socket sd)
+{
+#ifdef HAVE_POLL
+  struct pollfd ufds;
+  int res;
+
+  ufds.fd= sd;
+  ufds.events= POLLIN | POLLPRI;
+  if (!(res= poll(&ufds, 1, 0)))
+    return 0;
+  if (res < 0 || !(ufds.revents & (POLLIN | POLLPRI)))
+    return 0;
+  return 1;
+#else
+  fd_set sfds;
+  struct timeval tv;
+  int res;
+
+#ifndef __WIN__
+  /* Windows uses an _array_ of 64 fd's as default, so it's safe */
+  if (sd >= FD_SETSIZE)
+    return -1;
+#define NET_DATA_IS_READY_CAN_RETURN_MINUS_ONE
+#endif
+
+  FD_ZERO(&sfds);
+  FD_SET(sd, &sfds);
+
+  tv.tv_sec= tv.tv_usec= 0;
+
+  if ((res= select(sd+1, &sfds, NULL, NULL, &tv)) < 0)
+    return 0;
+  else
+    return test(res ? FD_ISSET(sd, &sfds) : 0);
+#endif
+}
+
+
+/*
+  Remove unwanted characters from connection
+  and check if disconnected
+
+  SYNOPSIS
+    net_clear()
+    net			NET handler
+
+  DESCRIPTION
+    Read from socket until there is nothing more to read. Discard
+    what is read.
+
+    If there is anything when to read 'net_clear' is called this
+    normally indicates an error in the protocol.
+
+    When connection is properly closed (for TCP it means with
+    a FIN packet), then select() considers a socket "ready to read",
+    in the sense that there's EOF to read, but read() returns 0.
+
+*/
 
 void net_clear(NET *net)
 {
+  int count, ready;
   DBUG_ENTER("net_clear");
-#if !defined(EXTRA_DEBUG) && !defined(EMBEDDED_LIBRARY)
+#if !defined(EMBEDDED_LIBRARY)
+  while((ready= net_data_is_ready(net->vio->sd)) > 0)
   {
-    int count;					/* One may get 'unused' warn */
+    /* The socket is ready */
+    if ((count= vio_read(net->vio, (char*) (net->buff),
+                         (uint32) net->max_packet)) > 0)
+    {
+      DBUG_PRINT("info",("skipped %d bytes from file: %s",
+                         count, vio_description(net->vio)));
+#ifdef EXTRA_DEBUG
+      fprintf(stderr,"skipped %d bytes from file: %s\n",
+              count, vio_description(net->vio));
+#endif
+    }
+    else
+    {
+      DBUG_PRINT("info",("socket ready but only EOF to read - disconnected"));
+      net->error= 2;
+      break;
+    }
+  }
+#ifdef NET_DATA_IS_READY_CAN_RETURN_MINUS_ONE
+  /* 'net_data_is_ready' returned "don't know" */
+  if (ready == -1)
+  {
+    /* Read unblocking to clear net */
     my_bool old_mode;
     if (!vio_blocking(net->vio, FALSE, &old_mode))
     {
-      while ((count = vio_read(net->vio, (char*) (net->buff),
-			       (uint32) net->max_packet)) > 0)
+      while ((count= vio_read(net->vio, (char*) (net->buff),
+                              (uint32) net->max_packet)) > 0)
 	DBUG_PRINT("info",("skipped %d bytes from file: %s",
 			   count, vio_description(net->vio)));
       vio_blocking(net->vio, TRUE, &old_mode);
     }
   }
-#endif /* EXTRA_DEBUG */
+#endif
+#endif
   net->pkt_nr=net->compress_pkt_nr=0;		/* Ready for new command */
   net->write_pos=net->buff;
   DBUG_VOID_RETURN;
 }
 
+
 	/* Flush write_buffer if not empty. */
 
-int net_flush(NET *net)
+my_bool net_flush(NET *net)
 {
-  int error=0;
+  my_bool error= 0;
   DBUG_ENTER("net_flush");
   if (net->buff != net->write_pos)
   {
-    error=net_real_write(net,(char*) net->buff,
-			 (ulong) (net->write_pos - net->buff));
+    error=test(net_real_write(net,(char*) net->buff,
+			      (ulong) (net->write_pos - net->buff)));
     net->write_pos=net->buff;
   }
   /* Sync packet number if using compression */
@@ -223,7 +354,7 @@ int net_flush(NET *net)
     If compression is used the original package is modified!
 */
 
-int
+my_bool
 my_net_write(NET *net,const char *packet,ulong len)
 {
   uchar buff[NET_HEADER_SIZE];
@@ -250,23 +381,46 @@ my_net_write(NET *net,const char *packet,ulong len)
   buff[3]= (uchar) net->pkt_nr++;
   if (net_write_buff(net,(char*) buff,NET_HEADER_SIZE))
     return 1;
+#ifndef DEBUG_DATA_PACKETS
   DBUG_DUMP("packet_header",(char*) buff,NET_HEADER_SIZE);
+#endif
   return test(net_write_buff(net,packet,len));
 }
 
 /*
   Send a command to the server.
-  As the command is part of the first data packet, we have to do some data
-  juggling to put the command in there, without having to create a new
-  packet.
-  This function will split big packets into sub-packets if needed.
-  (Each sub packet can only be 2^24 bytes)
+
+  SYNOPSIS
+    net_write_command()
+    net			NET handler
+    command		Command in MySQL server (enum enum_server_command)
+    header		Header to write after command
+    head_len		Length of header
+    packet		Query or parameter to query
+    len			Length of packet
+
+  DESCRIPTION
+    The reason for having both header and packet is so that libmysql
+    can easy add a header to a special command (like prepared statements)
+    without having to re-alloc the string.
+
+    As the command is part of the first data packet, we have to do some data
+    juggling to put the command in there, without having to create a new
+    packet.
+    This function will split big packets into sub-packets if needed.
+    (Each sub packet can only be 2^24 bytes)
+
+  RETURN VALUES
+    0	ok
+    1	error
 */
 
-int
-net_write_command(NET *net,uchar command,const char *packet,ulong len)
+my_bool
+net_write_command(NET *net,uchar command,
+		  const char *header, ulong head_len,
+		  const char *packet, ulong len)
 {
-  ulong length=len+1;				/* 1 extra byte for command */
+  ulong length=len+1+head_len;			/* 1 extra byte for command */
   uchar buff[NET_HEADER_SIZE+1];
   uint header_size=NET_HEADER_SIZE+1;
   DBUG_ENTER("net_write_command");
@@ -277,25 +431,28 @@ net_write_command(NET *net,uchar command,const char *packet,ulong len)
   if (length >= MAX_PACKET_LENGTH)
   {
     /* Take into account that we have the command in the first header */
-    len= MAX_PACKET_LENGTH -1;
+    len= MAX_PACKET_LENGTH - 1 - head_len;
     do
     {
       int3store(buff, MAX_PACKET_LENGTH);
       buff[3]= (uchar) net->pkt_nr++;
       if (net_write_buff(net,(char*) buff, header_size) ||
-	  net_write_buff(net,packet,len))
+	  net_write_buff(net, header, head_len) ||
+	  net_write_buff(net, packet, len))
 	DBUG_RETURN(1);
       packet+= len;
       length-= MAX_PACKET_LENGTH;
       len= MAX_PACKET_LENGTH;
+      head_len= 0;
       header_size= NET_HEADER_SIZE;
     } while (length >= MAX_PACKET_LENGTH);
     len=length;					/* Data left to be written */
   }
   int3store(buff,length);
   buff[3]= (uchar) net->pkt_nr++;
-  DBUG_RETURN(test(net_write_buff(net,(char*) buff,header_size) ||
-		   net_write_buff(net,packet,len) || net_flush(net)));
+  DBUG_RETURN(test(net_write_buff(net, (char*) buff, header_size) ||
+	      (head_len && net_write_buff(net, (char*) header, head_len)) ||
+	      net_write_buff(net, packet, len) || net_flush(net)));
 }
 
 /*
@@ -336,6 +493,9 @@ net_write_buff(NET *net,const char *packet,ulong len)
   else
     left_length= (ulong) (net->buff_end - net->write_pos);
 
+#ifdef DEBUG_DATA_PACKETS
+  DBUG_DUMP("data", packet, len);
+#endif
   if (len > left_length)
   {
     if (net->write_pos != net->buff)
@@ -392,7 +552,8 @@ net_real_write(NET *net,const char *packet,ulong len)
   my_bool net_blocking = vio_is_blocking(net->vio);
   DBUG_ENTER("net_real_write");
 
-#if defined(MYSQL_SERVER) && defined(HAVE_QUERY_CACHE)
+#if defined(MYSQL_SERVER) && defined(HAVE_QUERY_CACHE) \
+                          && !defined(MYSQL_INSTANCE_MANAGER)
   if (net->query_cache_query != 0)
     query_cache_insert(net, packet, len);
 #endif
@@ -411,10 +572,12 @@ net_real_write(NET *net,const char *packet,ulong len)
 			       COMP_HEADER_SIZE, MYF(MY_WME))))
     {
 #ifdef MYSQL_SERVER
-      net->last_errno=ER_OUT_OF_RESOURCES;
-      net->error=2;
+      net->last_errno= ER_OUT_OF_RESOURCES;
+      net->error= 2;
+      /* TODO is it needed to set this variable if we have no socket */
+      net->report_error= 1;
 #endif
-      net->reading_or_writing=0;
+      net->reading_or_writing= 0;
       DBUG_RETURN(1);
     }
     memcpy(b+header_length,packet,len);
@@ -436,7 +599,7 @@ net_real_write(NET *net,const char *packet,ulong len)
     thr_alarm(&alarmed,(uint) net->write_timeout,&alarm_buff);
 #else
   alarmed=0;
-  vio_timeout(net->vio, net->write_timeout);
+  vio_timeout(net->vio, 1, net->write_timeout);
 #endif /* NO_ALARM */
 
   pos=(char*) packet; end=pos+len;
@@ -461,9 +624,10 @@ net_real_write(NET *net,const char *packet,ulong len)
 		    my_progname,vio_errno(net->vio));
 #endif /* EXTRA_DEBUG */
 #ifdef MYSQL_SERVER	    
-	    net->last_errno=ER_NET_ERROR_ON_WRITE;
+	    net->last_errno= ER_NET_ERROR_ON_WRITE;
 #endif
-	    net->error=2;                     /* Close socket */
+	    net->error= 2;                     /* Close socket */
+            net->report_error= 1;
 	    goto end;
 	  }
 	  retry_count=0;
@@ -489,7 +653,8 @@ net_real_write(NET *net,const char *packet,ulong len)
 	continue;
       }
 #endif /* defined(THREAD_SAFE_CLIENT) && !defined(MYSQL_SERVER) */
-      net->error=2;				/* Close socket */
+      net->error= 2;				/* Close socket */
+      net->report_error= 1;
 #ifdef MYSQL_SERVER
       net->last_errno= (interrupted ? ER_NET_WRITE_INTERRUPTED :
 			ER_NET_ERROR_ON_WRITE);
@@ -497,7 +662,7 @@ net_real_write(NET *net,const char *packet,ulong len)
       break;
     }
     pos+=length;
-    statistic_add(bytes_sent,length,&LOCK_bytes_sent);
+    update_statistics(thd_increment_bytes_sent(length));
   }
 #ifndef __WIN__
  end:
@@ -568,7 +733,7 @@ static my_bool my_net_skip_rest(NET *net, uint32 remain, thr_alarm_t *alarmed,
   DBUG_PRINT("enter",("bytes_to_skip: %u", (uint) remain));
 
   /* The following is good for debugging */
-  statistic_increment(net_big_packet_count,&LOCK_bytes_received);
+  update_statistics(thd_increment_net_big_packet_count(1));
 
   if (!thr_alarm_in_use(alarmed))
   {
@@ -584,7 +749,7 @@ static my_bool my_net_skip_rest(NET *net, uint32 remain, thr_alarm_t *alarmed,
       uint length= min(remain, net->max_packet);
       if (net_safe_read(net, (char*) net->buff, length, alarmed))
 	DBUG_RETURN(1);
-      statistic_add(bytes_received, length, &LOCK_bytes_received);
+      update_statistics(thd_increment_bytes_received(length));
       remain -= (uint32) length;
     }
     if (old != MAX_PACKET_LENGTH)
@@ -627,7 +792,7 @@ my_real_read(NET *net, ulong *complen)
   if (net_blocking)
     thr_alarm(&alarmed,net->read_timeout,&alarm_buff);
 #else
-  vio_timeout(net->vio, net->read_timeout);
+  vio_timeout(net->vio, 0, net->read_timeout);
 #endif /* NO_ALARM */
 
     pos = net->buff + net->where_b;		/* net->packet -4 */
@@ -667,9 +832,10 @@ my_real_read(NET *net, ulong *complen)
 			my_progname,vio_errno(net->vio));
 #endif /* EXTRA_DEBUG */
 		len= packet_error;
-		net->error=2;                 /* Close socket */
+		net->error= 2;                 /* Close socket */
+	        net->report_error= 1;
 #ifdef MYSQL_SERVER
-		net->last_errno=ER_NET_FCNTL_ERROR;
+		net->last_errno= ER_NET_FCNTL_ERROR;
 #endif
 		goto end;
 	      }
@@ -698,7 +864,8 @@ my_real_read(NET *net, ulong *complen)
 	  DBUG_PRINT("error",("Couldn't read packet: remain: %u  errno: %d  length: %ld",
 			      remain, vio_errno(net->vio), length));
 	  len= packet_error;
-	  net->error=2;				/* Close socket */
+	  net->error= 2;				/* Close socket */
+	  net->report_error= 1;
 #ifdef MYSQL_SERVER
 	  net->last_errno= (vio_was_interrupted(net->vio) ? ER_NET_READ_INTERRUPTED :
 			    ER_NET_READ_ERROR);
@@ -707,11 +874,13 @@ my_real_read(NET *net, ulong *complen)
 	}
 	remain -= (uint32) length;
 	pos+= (ulong) length;
-	statistic_add(bytes_received,(ulong) length,&LOCK_bytes_received);
+	update_statistics(thd_increment_bytes_received(length));
       }
       if (i == 0)
       {					/* First parts is packet length */
 	ulong helping;
+        DBUG_DUMP("packet_header",(char*) net->buff+net->where_b,
+                  NET_HEADER_SIZE);
 	if (net->buff[net->where_b + 3] != (uchar) net->pkt_nr)
 	{
 	  if (net->buff[net->where_b] != (uchar) 255)
@@ -720,7 +889,6 @@ my_real_read(NET *net, ulong *complen)
 		       ("Packets out of order (Found: %d, expected %u)",
 			(int) net->buff[net->where_b + 3],
 			net->pkt_nr));
-	    DBUG_DUMP("packet_header",(char*) net->buff+net->where_b, 4);
 #ifdef EXTRA_DEBUG
 	    fprintf(stderr,"Packets out of order (Found: %d, expected %d)\n",
 		    (int) net->buff[net->where_b + 3],
@@ -728,6 +896,7 @@ my_real_read(NET *net, ulong *complen)
 #endif
 	  }
 	  len= packet_error;
+	  net->report_error= 1;
 #ifdef MYSQL_SERVER
 	  net->last_errno=ER_NET_PACKETS_OUT_OF_ORDER;
 #endif
@@ -776,6 +945,10 @@ end:
     vio_blocking(net->vio, net_blocking, &old_mode);
   }
   net->reading_or_writing=0;
+#ifdef DEBUG_DATA_PACKETS
+  if (len != packet_error)
+    DBUG_DUMP("data",(char*) net->buff+net->where_b, len);
+#endif
   return(len);
 }
 
@@ -908,7 +1081,8 @@ my_net_read(NET *net)
       if (my_uncompress((byte*) net->buff + net->where_b, &packet_len,
 			&complen))
       {
-	net->error=2;			/* caller will close socket */
+	net->error= 2;			/* caller will close socket */
+	net->report_error= 1;
 #ifdef MYSQL_SERVER
 	net->last_errno=ER_NET_UNCOMPRESS_ERROR;
 #endif
@@ -929,12 +1103,3 @@ my_net_read(NET *net)
   return len;
 }
 
-bool net_request_file(NET* net, const char* fname)
-{
-  char tmp [FN_REFLEN+1],*end;
-  DBUG_ENTER("net_request_file");
-  tmp[0] = (char) 251;				/* NULL_LENGTH */
-  end=strnmov(tmp+1,fname,sizeof(tmp)-2);
-  DBUG_RETURN(my_net_write(net,tmp,(uint) (end-tmp)) ||
-	      net_flush(net));
-}
