@@ -66,12 +66,10 @@ static int init_failsafe_rpl_thread(THD* thd)
     this thread has no other error reporting method).
   */
   thd->system_thread = thd->bootstrap = 1;
-  thd->host_or_ip= "";
+  thd->security_ctx->skip_grants();
   my_net_init(&thd->net, 0);
   thd->net.read_timeout = slave_net_timeout;
   thd->max_client_packet_length=thd->net.max_packet;
-  thd->master_access= ~(ulong)0;
-  thd->priv_user = 0;
   pthread_mutex_lock(&LOCK_thread_count);
   thd->thread_id = thread_id++;
   pthread_mutex_unlock(&LOCK_thread_count);
@@ -161,7 +159,7 @@ int register_slave(THD* thd, uchar* packet, uint packet_length)
   SLAVE_INFO *si;
   uchar *p= packet, *p_end= packet + packet_length;
 
-  if (check_access(thd, REPL_SLAVE_ACL, any_db,0,0,0))
+  if (check_access(thd, REPL_SLAVE_ACL, any_db,0,0,0,0))
     return 1;
   if (!(si = (SLAVE_INFO*)my_malloc(sizeof(SLAVE_INFO), MYF(MY_WME))))
     goto err2;
@@ -192,7 +190,6 @@ err:
   my_message(ER_UNKNOWN_ERROR, "Wrong parameters to function register_slave",
 	     MYF(0));
 err2:
-  send_error(thd);
   return 1;
 }
 
@@ -436,7 +433,7 @@ static Slave_log_event* find_slave_event(IO_CACHE* log,
    This function is broken now. See comment for translate_master().
  */
 
-int show_new_master(THD* thd)
+bool show_new_master(THD* thd)
 {
   Protocol *protocol= thd->protocol;
   DBUG_ENTER("show_new_master");
@@ -449,23 +446,24 @@ int show_new_master(THD* thd)
   {
     if (errmsg[0])
       my_error(ER_ERROR_WHEN_EXECUTING_COMMAND, MYF(0),
-	       "SHOW NEW MASTER", errmsg);
-    DBUG_RETURN(-1);
+               "SHOW NEW MASTER", errmsg);
+    DBUG_RETURN(TRUE);
   }
   else
   {
     field_list.push_back(new Item_empty_string("Log_name", 20));
     field_list.push_back(new Item_return_int("Log_pos", 10,
 					     MYSQL_TYPE_LONGLONG));
-    if (protocol->send_fields(&field_list, 1))
-      DBUG_RETURN(-1);
+    if (protocol->send_fields(&field_list,
+                              Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
+      DBUG_RETURN(TRUE);
     protocol->prepare_for_resend();
     protocol->store(lex_mi->log_file_name, &my_charset_bin);
     protocol->store((ulonglong) lex_mi->pos);
     if (protocol->write())
-      DBUG_RETURN(-1);
+      DBUG_RETURN(TRUE);
     send_eof(thd);
-    DBUG_RETURN(0);
+    DBUG_RETURN(FALSE);
   }
 }
 
@@ -504,7 +502,7 @@ int update_slave_list(MYSQL* mysql, MASTER_INFO* mi)
   int port_ind;
   DBUG_ENTER("update_slave_list");
 
-  if (mysql_real_query(mysql,"SHOW SLAVE HOSTS",16) ||
+  if (mysql_real_query(mysql, STRING_WITH_LEN("SHOW SLAVE HOSTS")) ||
       !(res = mysql_store_result(mysql)))
   {
     error= mysql_error(mysql);
@@ -579,7 +577,7 @@ int find_recovery_captain(THD* thd, MYSQL* mysql)
 }
 
 
-pthread_handler_decl(handle_failsafe_rpl,arg)
+pthread_handler_t handle_failsafe_rpl(void *arg)
 {
   DBUG_ENTER("handle_failsafe_rpl");
   THD *thd = new THD;
@@ -628,7 +626,7 @@ err:
 }
 
 
-int show_slave_hosts(THD* thd)
+bool show_slave_hosts(THD* thd)
 {
   List<Item> field_list;
   Protocol *protocol= thd->protocol;
@@ -648,8 +646,9 @@ int show_slave_hosts(THD* thd)
   field_list.push_back(new Item_return_int("Master_id", 10,
 					   MYSQL_TYPE_LONG));
 
-  if (protocol->send_fields(&field_list, 1))
-    DBUG_RETURN(-1);
+  if (protocol->send_fields(&field_list,
+                            Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
+    DBUG_RETURN(TRUE);
 
   pthread_mutex_lock(&LOCK_slave_list);
 
@@ -670,12 +669,12 @@ int show_slave_hosts(THD* thd)
     if (protocol->write())
     {
       pthread_mutex_unlock(&LOCK_slave_list);
-      DBUG_RETURN(-1);
+      DBUG_RETURN(TRUE);
     }
   }
   pthread_mutex_unlock(&LOCK_slave_list);
   send_eof(thd);
-  DBUG_RETURN(0);
+  DBUG_RETURN(FALSE);
 }
 
 
@@ -706,6 +705,7 @@ int connect_to_master(THD *thd, MYSQL* mysql, MASTER_INFO* mi)
   if (!mysql_real_connect(mysql, mi->host, mi->user, mi->password, 0,
 			mi->port, 0, 0))
     DBUG_RETURN(1);
+  mysql->reconnect= 1;
   DBUG_RETURN(0);
 }
 
@@ -736,7 +736,7 @@ static int fetch_db_tables(THD *thd, MYSQL *mysql, const char *db,
     {
       bzero((char*) &table, sizeof(table)); //just for safe
       table.db= (char*) db;
-      table.real_name= (char*) table_name;
+      table.table_name= (char*) table_name;
       table.updating= 1;
 
       if (!tables_ok(thd, &table))
@@ -756,7 +756,7 @@ static int fetch_db_tables(THD *thd, MYSQL *mysql, const char *db,
    - No active transaction (flush_relay_log_info would not work in this case)
 */
 
-int load_master_data(THD* thd)
+bool load_master_data(THD* thd)
 {
   MYSQL mysql;
   MYSQL_RES* master_status_res = 0;
@@ -778,16 +778,15 @@ int load_master_data(THD* thd)
       (error=terminate_slave_threads(active_mi,restart_thread_mask,
 				     1 /*skip lock*/)))
   {
-    send_error(thd,error);
+    my_message(error, ER(error), MYF(0));
     unlock_slave_threads(active_mi);
     pthread_mutex_unlock(&LOCK_active_mi);
-    return 1;
+    return TRUE;
   }
   
   if (connect_to_master(thd, &mysql, active_mi))
   {
-    net_printf(thd, error= ER_CONNECT_TO_MASTER,
-	       mysql_error(&mysql));
+    my_error(error= ER_CONNECT_TO_MASTER, MYF(0), mysql_error(&mysql));
     goto err;
   }
 
@@ -796,11 +795,10 @@ int load_master_data(THD* thd)
     MYSQL_RES *db_res, **table_res, **table_res_end, **cur_table_res;
     uint num_dbs;
 
-    if (mysql_real_query(&mysql, "SHOW DATABASES", 14) ||
+    if (mysql_real_query(&mysql, STRING_WITH_LEN("SHOW DATABASES")) ||
 	!(db_res = mysql_store_result(&mysql)))
     {
-      net_printf(thd, error = ER_QUERY_ON_MASTER,
-		 mysql_error(&mysql));
+      my_error(error= ER_QUERY_ON_MASTER, MYF(0), mysql_error(&mysql));
       goto err;
     }
 
@@ -813,7 +811,7 @@ int load_master_data(THD* thd)
 
     if (!(table_res = (MYSQL_RES**)thd->alloc(num_dbs * sizeof(MYSQL_RES*))))
     {
-      net_printf(thd, error = ER_OUTOFMEMORY);
+      my_message(error = ER_OUTOFMEMORY, ER(ER_OUTOFMEMORY), MYF(0));
       goto err;
     }
 
@@ -823,12 +821,12 @@ int load_master_data(THD* thd)
       we wait to issue FLUSH TABLES WITH READ LOCK for as long as we
       can to minimize the lock time.
     */
-    if (mysql_real_query(&mysql, "FLUSH TABLES WITH READ LOCK", 27) ||
-	mysql_real_query(&mysql, "SHOW MASTER STATUS",18) ||
+    if (mysql_real_query(&mysql,
+                         STRING_WITH_LEN("FLUSH TABLES WITH READ LOCK")) ||
+	mysql_real_query(&mysql, STRING_WITH_LEN("SHOW MASTER STATUS")) ||
 	!(master_status_res = mysql_store_result(&mysql)))
     {
-      net_printf(thd, error = ER_QUERY_ON_MASTER,
-		 mysql_error(&mysql));
+      my_error(error= ER_QUERY_ON_MASTER, MYF(0), mysql_error(&mysql));
       goto err;
     }
 
@@ -862,7 +860,8 @@ int load_master_data(THD* thd)
 
       if (!db_ok(db, replicate_do_db, replicate_ignore_db) ||
           !db_ok_with_wild_table(db) ||
-	  !strcmp(db,"mysql"))
+	  !strcmp(db,"mysql") ||
+          is_schema_db(db))
       {
 	*cur_table_res = 0;
 	continue;
@@ -873,17 +872,15 @@ int load_master_data(THD* thd)
 
       if (mysql_create_db(thd, db, &create_info, 1))
       {
-	send_error(thd, 0, 0);
 	cleanup_mysql_results(db_res, cur_table_res - 1, table_res);
 	goto err;
       }
 
       if (mysql_select_db(&mysql, db) ||
-	  mysql_real_query(&mysql, "SHOW TABLES", 11) ||
+	  mysql_real_query(&mysql, STRING_WITH_LEN("SHOW TABLES")) ||
 	  !(*cur_table_res = mysql_store_result(&mysql)))
       {
-	net_printf(thd, error = ER_QUERY_ON_MASTER,
-		   mysql_error(&mysql));
+	my_error(error= ER_QUERY_ON_MASTER, MYF(0), mysql_error(&mysql));
 	cleanup_mysql_results(db_res, cur_table_res - 1, table_res);
 	goto err;
       }
@@ -921,7 +918,7 @@ int load_master_data(THD* thd)
 
         if (init_master_info(active_mi, master_info_file, relay_log_info_file, 
 			     0, (SLAVE_IO | SLAVE_SQL)))
-          send_error(thd, ER_MASTER_INFO);
+          my_message(ER_MASTER_INFO, ER(ER_MASTER_INFO), MYF(0));
 	strmake(active_mi->master_log_name, row[0],
 		sizeof(active_mi->master_log_name));
 	active_mi->master_log_pos= my_strtoll10(row[1], (char**) 0, &error);
@@ -933,15 +930,15 @@ int load_master_data(THD* thd)
           host was specified; there could have been a problem when replication
           started, which led to relay log's IO_CACHE to not be inited.
         */
-	flush_master_info(active_mi, 0);
+        if (flush_master_info(active_mi, 0))
+          sql_print_error("Failed to flush master info file");
       }
       mysql_free_result(master_status_res);
     }
 
-    if (mysql_real_query(&mysql, "UNLOCK TABLES", 13))
+    if (mysql_real_query(&mysql, STRING_WITH_LEN("UNLOCK TABLES")))
     {
-      net_printf(thd, error = ER_QUERY_ON_MASTER,
-		 mysql_error(&mysql));
+      my_error(error= ER_QUERY_ON_MASTER, MYF(0), mysql_error(&mysql));
       goto err;
     }
   }
@@ -950,10 +947,10 @@ int load_master_data(THD* thd)
 		       0 /* not only reset, but also reinit */,
 		       &errmsg))
   {
-    send_error(thd, 0, "Failed purging old relay logs");
+    my_error(ER_RELAY_LOG_FAIL, MYF(0), errmsg);
     unlock_slave_threads(active_mi);
     pthread_mutex_unlock(&LOCK_active_mi);
-    return 1;
+    return TRUE;
   }
   pthread_mutex_lock(&active_mi->rli.data_lock);
   active_mi->rli.group_master_log_pos = active_mi->master_log_pos;
