@@ -56,7 +56,9 @@
 **
 ** Function 'myfunc_int' returns summary length of all its arguments.
 **
-** Function 'sequence' returns an sequence starting from a certain number
+** Function 'sequence' returns an sequence starting from a certain number.
+**
+** Function 'myfunc_argument_name' returns name of argument.
 **
 ** On the end is a couple of functions that converts hostnames to ip and
 ** vice versa.
@@ -82,6 +84,7 @@
 ** CREATE FUNCTION lookup RETURNS STRING SONAME "udf_example.so";
 ** CREATE FUNCTION reverse_lookup RETURNS STRING SONAME "udf_example.so";
 ** CREATE AGGREGATE FUNCTION avgcost RETURNS REAL SONAME "udf_example.so";
+** CREATE FUNCTION myfunc_argument_name RETURNS STRING SONAME "udf_example.so";
 **
 ** After this the functions will work exactly like native MySQL functions.
 ** Functions should be created only once.
@@ -94,6 +97,7 @@
 ** DROP FUNCTION lookup;
 ** DROP FUNCTION reverse_lookup;
 ** DROP FUNCTION avgcost;
+** DROP FUNCTION myfunc_argument_name;
 **
 ** The CREATE FUNCTION and DROP FUNCTION update the func@mysql table. All
 ** Active function will be reloaded on every restart of server
@@ -109,6 +113,8 @@
 */
 
 #ifdef STANDARD
+/* STANDARD is defined, don't use any mysql functions */
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #ifdef __WIN__
@@ -121,10 +127,12 @@ typedef long long longlong;
 #else
 #include <my_global.h>
 #include <my_sys.h>
+#include <m_string.h>		// To get strmov()
 #endif
 #include <mysql.h>
-#include <m_ctype.h>
-#include <m_string.h>		// To get strmov()
+#include <ctype.h>
+
+static pthread_mutex_t LOCK_hostname;
 
 #ifdef HAVE_DLOPEN
 
@@ -138,6 +146,7 @@ char *metaphon(UDF_INIT *initid, UDF_ARGS *args, char *result,
 my_bool myfunc_double_init(UDF_INIT *, UDF_ARGS *args, char *message);
 double myfunc_double(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
 		     char *error);
+my_bool myfunc_int_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
 longlong myfunc_int(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
 		    char *error);
 my_bool sequence_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
@@ -147,6 +156,7 @@ longlong sequence(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
 my_bool avgcost_init( UDF_INIT* initid, UDF_ARGS* args, char* message );
 void avgcost_deinit( UDF_INIT* initid );
 void avgcost_reset( UDF_INIT* initid, UDF_ARGS* args, char* is_null, char *error );
+void avgcost_clear( UDF_INIT* initid, char* is_null, char *error );
 void avgcost_add( UDF_INIT* initid, UDF_ARGS* args, char* is_null, char *error );
 double avgcost( UDF_INIT* initid, UDF_ARGS* args, char* is_null, char *error );
 }
@@ -334,7 +344,7 @@ char *metaphon(UDF_INIT *initid, UDF_ARGS *args, char *result,
   KSflag = 0; /* state flag for KS translation */
 
   for (metaph_end = result + MAXMETAPH, n_start = n;
-	n <= n_end && result < metaph_end; n++ )
+	n < n_end && result < metaph_end; n++ )
   {
 
     if ( KSflag )
@@ -487,7 +497,7 @@ char *metaphon(UDF_INIT *initid, UDF_ARGS *args, char *result,
 	}
     }
   }
-  *length= (ulong) (result - org_result);
+  *length= (unsigned long) (result - org_result);
   return org_result;
 }
 
@@ -511,7 +521,7 @@ my_bool myfunc_double_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 {
   if (!args->arg_count)
   {
-    strcpy(message,"myfunc_double must have at least on argument");
+    strcpy(message,"myfunc_double must have at least one argument");
     return 1;
   }
   /*
@@ -583,11 +593,21 @@ longlong myfunc_int(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
     case REAL_RESULT:			// Add numers as longlong
       val += (longlong) *((double*) args->args[i]);
       break;
+    default:
+      break;
     }
   }
   return val;
 }
 
+/*
+  At least one of _init/_deinit is needed unless the server is started
+  with --allow_suspicious_udfs.
+*/
+my_bool myfunc_int_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
+{
+  return 0;
+}
 
 /*
   Simple example of how to get a sequences starting from the first argument
@@ -610,10 +630,12 @@ my_bool sequence_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
     return 1;
   }
   bzero(initid->ptr,sizeof(longlong));
-  // Fool MySQL to think that this function is a constant
-  // This will ensure that MySQL only evalutes the function
-  // when the rows are sent to the client and not before any ORDER BY
-  // clauses
+  /*
+    Fool MySQL to think that this function is a constant
+    This will ensure that MySQL only evalutes the function
+    when the rows are sent to the client and not before any ORDER BY
+    clauses
+  */
   initid->const_item=1;
   return 0;
 }
@@ -630,9 +652,10 @@ longlong sequence(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
   ulonglong val=0;
   if (args->arg_count)
     val= *((longlong*) args->args[0]);
-  return ++ *((longlong*) initid->ptr) + val;
+  return ++*((longlong*) initid->ptr) + val;
 }
 
+
 /****************************************************************************
 ** Some functions that handles IP and hostname conversions
 ** The orignal function was from Zeev Suraski.
@@ -642,8 +665,6 @@ longlong sequence(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
 **
 ****************************************************************************/
 
-#if defined(HAVE_GETHOSTBYADDR_R) && defined(HAVE_SOLARIS_STYLE_GETHOST)
-
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -651,9 +672,11 @@ longlong sequence(UDF_INIT *initid, UDF_ARGS *args, char *is_null,
 
 extern "C" {
 my_bool lookup_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
+void lookup_deinit(UDF_INIT *initid);
 char *lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
 	     unsigned long *length, char *null_value, char *error);
 my_bool reverse_lookup_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
+void reverse_lookup_deinit(UDF_INIT *initid);
 char *reverse_lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
 		     unsigned long *length, char *null_value, char *error);
 }
@@ -676,9 +699,19 @@ my_bool lookup_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
   }
   initid->max_length=11;
   initid->maybe_null=1;
+#if !defined(HAVE_GETHOSTBYADDR_R) || !defined(HAVE_SOLARIS_STYLE_GETHOST)
+  (void) pthread_mutex_init(&LOCK_hostname,MY_MUTEX_INIT_SLOW);
+#endif
   return 0;
 }
 
+void lookup_deinit(UDF_INIT *initid)
+{
+#if !defined(HAVE_GETHOSTBYADDR_R) || !defined(HAVE_SOLARIS_STYLE_GETHOST)
+  (void) pthread_mutex_destroy(&LOCK_hostname);
+#endif
+}
+
 char *lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
 	     unsigned long *res_length, char *null_value, char *error)
 {
@@ -696,13 +729,23 @@ char *lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
     length=sizeof(name_buff)-1;
   memcpy(name_buff,args->args[0],length);
   name_buff[length]=0;
-
+#if defined(HAVE_GETHOSTBYADDR_R) && defined(HAVE_SOLARIS_STYLE_GETHOST)
   if (!(hostent=gethostbyname_r(name_buff,&tmp_hostent,hostname_buff,
 				sizeof(hostname_buff), &tmp_errno)))
   {
     *null_value=1;
     return 0;
   }
+#else
+  VOID(pthread_mutex_lock(&LOCK_hostname));
+  if (!(hostent= gethostbyname((char*) name_buff)))
+  {
+    VOID(pthread_mutex_unlock(&LOCK_hostname));
+    *null_value= 1;
+    return 0;
+  }
+  VOID(pthread_mutex_unlock(&LOCK_hostname));
+#endif
   struct in_addr in;
   memcpy_fixed((char*) &in,(char*) *hostent->h_addr_list, sizeof(in.s_addr));
   *res_length= (ulong) (strmov(result, inet_ntoa(in)) - result);
@@ -731,9 +774,18 @@ my_bool reverse_lookup_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
   }
   initid->max_length=32;
   initid->maybe_null=1;
+#if !defined(HAVE_GETHOSTBYADDR_R) || !defined(HAVE_SOLARIS_STYLE_GETHOST)
+  (void) pthread_mutex_init(&LOCK_hostname,MY_MUTEX_INIT_SLOW);
+#endif
   return 0;
 }
 
+void reverse_lookup_deinit(UDF_INIT *initid) 
+{
+#if !defined(HAVE_GETHOSTBYADDR_R) || !defined(HAVE_SOLARIS_STYLE_GETHOST)
+  (void) pthread_mutex_destroy(&LOCK_hostname);
+#endif
+}
 
 char *reverse_lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
 		     unsigned long *res_length, char *null_value, char *error)
@@ -776,6 +828,7 @@ char *reverse_lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
     return 0;
   }
   struct hostent *hp;
+#if defined(HAVE_GETHOSTBYADDR_R) && defined(HAVE_SOLARIS_STYLE_GETHOST)
   int tmp_errno;
   if (!(hp=gethostbyaddr_r((char*) &taddr,sizeof(taddr), AF_INET,
 			   &tmp_hostent, name_buff,sizeof(name_buff),
@@ -784,10 +837,19 @@ char *reverse_lookup(UDF_INIT *initid, UDF_ARGS *args, char *result,
     *null_value=1;
     return 0;
   }
+#else
+  VOID(pthread_mutex_lock(&LOCK_hostname));
+  if (!(hp= gethostbyaddr((char*) &taddr, sizeof(taddr), AF_INET)))
+  {
+    VOID(pthread_mutex_unlock(&LOCK_hostname));
+    *null_value= 1;
+    return 0;
+  }
+  VOID(pthread_mutex_unlock(&LOCK_hostname));
+#endif
   *res_length=(ulong) (strmov(result,hp->h_name) - result);
   return result;
 }
-#endif // defined(HAVE_GETHOSTBYADDR_R) && defined(HAVE_SOLARIS_STYLE_GETHOST)
 
 /*
 ** Syntax for the new aggregate commands are:
@@ -859,21 +921,29 @@ avgcost_deinit( UDF_INIT* initid )
   delete initid->ptr;
 }
 
+
+/* This is only for MySQL 4.0 compability */
 void
-avgcost_reset( UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* message )
+avgcost_reset(UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* message)
+{
+  avgcost_clear(initid, is_null, message);
+  avgcost_add(initid, args, is_null, message);
+}
+
+/* This is needed to get things to work in MySQL 4.1.1 and above */
+
+void
+avgcost_clear(UDF_INIT* initid, char* is_null, char* message)
 {
   struct avgcost_data* data = (struct avgcost_data*)initid->ptr;
-  data->totalprice	= 0.0;
-  data->totalquantity	= 0;
-  data->count			= 0;
-
-  *is_null = 0;
-  avgcost_add( initid, args, is_null, message );
+  data->totalprice=	0.0;
+  data->totalquantity=	0;
+  data->count=		0;
 }
 
 
 void
-avgcost_add( UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* message )
+avgcost_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* message)
 {
   if (args->args[0] && args->args[1])
   {
@@ -932,4 +1002,43 @@ avgcost( UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* error )
   return data->totalprice/double(data->totalquantity);
 }
 
+extern "C" {
+my_bool myfunc_argument_name_init(UDF_INIT *initid, UDF_ARGS *args,
+				  char *message);
+char *myfunc_argument_name(UDF_INIT *initid, UDF_ARGS *args, char *result,
+			   unsigned long *length, char *null_value,
+			   char *error);
+}
+
+my_bool myfunc_argument_name_init(UDF_INIT *initid, UDF_ARGS *args,
+				  char *message)
+{
+  if (args->arg_count != 1)
+  {
+    strmov(message,"myfunc_argument_name_init accepts only one argument");
+    return 1;
+  }
+  initid->max_length= args->attribute_lengths[0];
+  initid->maybe_null= 1;
+  initid->const_item= 1;
+  return 0;
+}
+
+char *myfunc_argument_name(UDF_INIT *initid, UDF_ARGS *args, char *result,
+			   unsigned long *length, char *null_value,
+			   char *error)
+{
+  if (!args->attributes[0])
+  {
+    null_value= 0;
+    return 0;
+  }
+  (*length)--; // space for ending \0 (for debugging purposes)
+  if (*length > args->attribute_lengths[0])
+    *length= args->attribute_lengths[0];
+  memcpy(result, args->attributes[0], *length);
+  result[*length]= 0;
+  return result;
+}
+
 #endif /* HAVE_DLOPEN */
