@@ -24,12 +24,12 @@
 void my_caseup_str_mb(CHARSET_INFO * cs, char *str)
 {
   register uint32 l;
-  register char *end=str+strlen(str); /* BAR TODO: remove strlen() call */
   register uchar *map=cs->to_upper;
   
   while (*str)
   {
-    if ((l=my_ismbchar(cs, str,end)))
+    /* Pointing after the '\0' is safe here. */
+    if ((l=my_ismbchar(cs, str, str + cs->mbmaxlen)))
       str+=l;
     else
     { 
@@ -42,12 +42,12 @@ void my_caseup_str_mb(CHARSET_INFO * cs, char *str)
 void my_casedn_str_mb(CHARSET_INFO * cs, char *str)
 {
   register uint32 l;
-  register char *end=str+strlen(str);
   register uchar *map=cs->to_lower;
   
   while (*str)
   {
-    if ((l=my_ismbchar(cs, str,end)))
+    /* Pointing after the '\0' is safe here. */
+    if ((l=my_ismbchar(cs, str, str + cs->mbmaxlen)))
       str+=l;
     else
     {
@@ -57,51 +57,62 @@ void my_casedn_str_mb(CHARSET_INFO * cs, char *str)
   }
 }
 
-void my_caseup_mb(CHARSET_INFO * cs, char *str, uint length)
+uint my_caseup_mb(CHARSET_INFO * cs, char *src, uint srclen,
+                  char *dst __attribute__((unused)),
+                  uint dstlen __attribute__((unused)))
 {
   register uint32 l;
-  register char *end=str+length;
-  register uchar *map=cs->to_upper;
-  
-  while (str<end)
+  register char *srcend= src + srclen;
+  register uchar *map= cs->to_upper;
+
+  DBUG_ASSERT(src == dst && srclen == dstlen);
+  while (src < srcend)
   {
-    if ((l=my_ismbchar(cs, str,end)))
-      str+=l;
+    if ((l=my_ismbchar(cs, src, srcend)))
+      src+= l;
     else 
     {
-      *str=(char) map[(uchar)*str];
-      str++;
+      *src=(char) map[(uchar) *src];
+      src++;
     }
   }
+  return srclen;
 }
 
-void my_casedn_mb(CHARSET_INFO * cs, char *str, uint length)
+uint my_casedn_mb(CHARSET_INFO * cs, char *src, uint srclen,
+                  char *dst __attribute__((unused)),
+                  uint dstlen __attribute__((unused)))
 {
   register uint32 l;
-  register char *end=str+length;
+  register char *srcend= src + srclen;
   register uchar *map=cs->to_lower;
-  
-  while (str<end)
+
+  DBUG_ASSERT(src == dst && srclen == dstlen);  
+  while (src < srcend)
   {
-    if ((l=my_ismbchar(cs, str,end)))
-      str+=l;
+    if ((l= my_ismbchar(cs, src, srcend)))
+      src+= l;
     else
     {
-      *str=(char) map[(uchar)*str];
-      str++;
+      *src= (char) map[(uchar)*src];
+      src++;
     }
   }
+  return srclen;
 }
 
+/*
+  my_strcasecmp_mb() returns 0 if strings are equal, non-zero otherwise.
+ */
 int my_strcasecmp_mb(CHARSET_INFO * cs,const char *s, const char *t)
 {
   register uint32 l;
-  register const char *end=s+strlen(s);
   register uchar *map=cs->to_upper;
   
-  while (s<end)
+  while (*s && *t)
   {
-    if ((l=my_ismbchar(cs, s,end)))
+    /* Pointing after the '\0' is safe here. */
+    if ((l=my_ismbchar(cs, s, s + cs->mbmaxlen)))
     {
       while (l--)
         if (*s++ != *t++) 
@@ -112,7 +123,8 @@ int my_strcasecmp_mb(CHARSET_INFO * cs,const char *s, const char *t)
     else if (map[(uchar) *s++] != map[(uchar) *t++])
       return 1;
   }
-  return *t;
+  /* At least one of '*s' and '*t' is zero here. */
+  return (*t != *s);
 }
 
 
@@ -260,7 +272,7 @@ uint my_charpos_mb(CHARSET_INFO *cs __attribute__((unused)),
     pos+= (mblen= my_ismbchar(cs, pos, end)) ? mblen : 1;
     length--;
   }
-  return length ? (uint) (end + 2 - start) : (uint) (pos - start);
+  return (uint) (length ? end+2-start : pos-start);
 }
 
 
@@ -321,7 +333,7 @@ uint my_instr_mb(CHARSET_INFO *cs,
         if (nmatch)
         {
           match[0].beg= 0;
-          match[0].end= b-b0;
+          match[0].end= (uint) (b-b0);
           match[0].mblen= res;
           if (nmatch > 1)
           {
@@ -365,6 +377,9 @@ static int my_strnncoll_mb_bin(CHARSET_INFO * cs __attribute__((unused)),
     slen		Length of 's'
     t			String to compare
     tlen		Length of 't'
+    diff_if_only_endspace_difference
+		        Set to 1 if the strings should be regarded as different
+                        if they only difference in end space
 
   NOTE
    This function is used for character strings with binary collations.
@@ -379,10 +394,16 @@ static int my_strnncoll_mb_bin(CHARSET_INFO * cs __attribute__((unused)),
 
 static int my_strnncollsp_mb_bin(CHARSET_INFO * cs __attribute__((unused)),
                                  const uchar *a, uint a_length, 
-                                 const uchar *b, uint b_length)
+                                 const uchar *b, uint b_length,
+                                 my_bool diff_if_only_endspace_difference)
 {
   const uchar *end;
   uint length;
+  int res;
+
+#ifndef VARCHAR_WITH_DIFF_ENDSPACE_ARE_DIFFERENT_FOR_UNIQUE
+  diff_if_only_endspace_difference= 0;
+#endif
   
   end= a + (length= min(a_length, b_length));
   while (a < end)
@@ -390,9 +411,12 @@ static int my_strnncollsp_mb_bin(CHARSET_INFO * cs __attribute__((unused)),
     if (*a++ != *b++)
       return ((int) a[-1] - (int) b[-1]);
   }
+  res= 0;
   if (a_length != b_length)
   {
     int swap= 1;
+    if (diff_if_only_endspace_difference)
+      res= 1;                                   /* Assume 'a' is bigger */
     /*
       Check the next not space character of the longer key. If it's < ' ',
       then it's smaller than the other key.
@@ -403,6 +427,7 @@ static int my_strnncollsp_mb_bin(CHARSET_INFO * cs __attribute__((unused)),
       a_length= b_length;
       a= b;
       swap= -1;					/* swap sign of result */
+      res= -res;
     }
     for (end= a + a_length-length; a < end ; a++)
     {
@@ -410,7 +435,7 @@ static int my_strnncollsp_mb_bin(CHARSET_INFO * cs __attribute__((unused)),
 	return (*a < ' ') ? -swap : swap;
     }
   }
-  return 0;
+  return res;
 }
 
 
@@ -502,17 +527,13 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
 			 char *min_str,char *max_str,
 			 uint *min_length,uint *max_length)
 {
-  const char *end;
+  const char *end= ptr + ptr_length;
   char *min_org= min_str;
   char *min_end= min_str + res_length;
   char *max_end= max_str + res_length;
-  uint charlen= my_charpos(cs, ptr, ptr+ptr_length, res_length/cs->mbmaxlen);
+  uint charlen= res_length / cs->mbmaxlen;
 
-  if (charlen < ptr_length)
-    ptr_length= charlen;
-  end= ptr + ptr_length;
-
-  for (; ptr != end && min_str != min_end ; ptr++)
+  for (; ptr != end && min_str != min_end && charlen > 0 ; ptr++, charlen--)
   {
     if (*ptr == escape && ptr+1 != end)
     {
@@ -527,9 +548,15 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
       if (charlen < (uint) (min_str - min_org))
         min_str= min_org + charlen;
       
-      /* Write min key  */
-      *min_length= (uint) (min_str - min_org);
+      /*
+        Calculate length of keys:
+        'a\0\0... is the smallest possible string when we have space expand
+        a\ff\ff... is the biggest possible string
+      */
+      *min_length= ((cs->state & MY_CS_BINSORT) ? (uint) (min_str - min_org) :
+                    res_length);
       *max_length= res_length;
+      /* Create min key  */
       do
       {
 	*min_str++= (char) cs->min_sort_char;
@@ -540,19 +567,20 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
         representation of the max_sort_char character,
         and copy it into max_str in a loop. 
       */
+      *max_length= res_length;
       pad_max_char(cs, max_str, max_end);
       return 0;
     }
     *min_str++= *max_str++ = *ptr;
   }
-  *min_length= *max_length = (uint) (min_str - min_org);
 
+  *min_length= *max_length = (uint) (min_str - min_org);
   while (min_str != min_end)
-    *min_str++= ' ';		/* Because if key compression */
-  pad_max_char(cs, max_str, max_end);
+    *min_str++= *max_str++= ' ';           /* Because if key compression */
   return 0;
 }
 
+
 static int my_wildcmp_mb_bin(CHARSET_INFO *cs,
 		  const char *str,const char *str_end,
 		  const char *wildstr,const char *wildend,
@@ -896,11 +924,13 @@ MY_COLLATION_HANDLER my_collation_mb_bin_handler =
     my_strnncoll_mb_bin,
     my_strnncollsp_mb_bin,
     my_strnxfrm_mb_bin,
+    my_strnxfrmlen_simple,
     my_like_range_simple,
     my_wildcmp_mb_bin,
     my_strcasecmp_mb_bin,
     my_instr_mb,
-    my_hash_sort_mb_bin
+    my_hash_sort_mb_bin,
+    my_propagate_simple
 };
 
 
