@@ -21,6 +21,8 @@
     Sasha Pachev <sasha@mysql.com>
 */
 
+#ifndef __NETWARE__
+
 #include <my_global.h>
 #include <my_pthread.h>
 #include <mysql.h>
@@ -89,6 +91,8 @@
 #define MAX_LAUNCHER_MSG 256
 #endif
 
+static CHARSET_INFO *cs= &my_charset_latin1;
+
 #define MAX_RETRY_COUNT 100
 
 /*
@@ -97,7 +101,7 @@
   set by the user
 */
 
-#if defined(__i386__) && defined(HAVE_LINUXTHREADS)
+#if defined(__i386__) && defined(TARGET_OS_LINUX)
 #define DO_STACKTRACE 1
 #endif
 
@@ -281,7 +285,7 @@ struct manager_cmd commands[] =
 static struct my_option my_long_options[] =
 {
 #ifndef DBUG_OFF
-  {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'",
+  {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.",
    0, 0, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #endif
   {"help", '?', "Display this help and exit.",
@@ -297,20 +301,20 @@ static struct my_option my_long_options[] =
   {"tcp-backlog", 'B', "Size of TCP/IP listen queue.",
    (gptr*) &manager_back_log, (gptr*) &manager_back_log, 0, GET_INT,
    REQUIRED_ARG, MANAGER_BACK_LOG, 0, 0, 0, 0, 0},
-  {"greeting", 'g', "Set greeting on connect", (gptr*) &manager_greeting,
+  {"greeting", 'g', "Set greeting on connect.", (gptr*) &manager_greeting,
    (gptr*) &manager_greeting, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"max-command-len", 'm', "Maximum command length",
+  {"max-command-len", 'm', "Maximum command length.",
    (gptr*) &manager_max_cmd_len, (gptr*) &manager_max_cmd_len, 0, GET_UINT,
    REQUIRED_ARG, MANAGER_MAX_CMD_LEN, 0, 0, 0, 0, 0},
-  {"one-thread", 'd', "Use one thread ( for debugging)", (gptr*) &one_thread,
+  {"one-thread", 'd', "Use one thread ( for debugging).", (gptr*) &one_thread,
    (gptr*) &one_thread, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"connect-retries", 'C', "Number of attempts to establish MySQL connection",
+  {"connect-retries", 'C', "Number of attempts to establish MySQL connection.",
    (gptr*) &manager_connect_retries, (gptr*) &manager_connect_retries, 0,
    GET_INT, REQUIRED_ARG, MANAGER_CONNECT_RETRIES, 0, 0, 0, 0, 0},
-  {"password-file", 'p', "Password file for manager",
+  {"password-file", 'p', "Password file for manager.",
    (gptr*) &manager_pw_file, (gptr*) &manager_pw_file, 0, GET_STR,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"pid-file", 'f', "Pid file to use", (gptr*) &pid_file, (gptr*) &pid_file,
+  {"pid-file", 'f', "Pid file to use.", (gptr*) &pid_file, (gptr*) &pid_file,
    0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"version", 'V', "Output version information and exit.", 0, 0, 0,
    GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
@@ -328,8 +332,8 @@ static int client_msg_raw(NET* net,int err_code,int pre,const char* fmt,
 static int authenticate(struct manager_thd* thd);
 /* returns pointer to end of line */
 static char* read_line(struct manager_thd* thd);
-static pthread_handler_decl(process_connection, arg);
-static pthread_handler_decl(process_launcher_messages, arg);
+pthread_handler_t process_connection(void *arg);
+pthread_handler_t process_launcher_messages(void *arg);
 static int exec_line(struct manager_thd* thd,char* buf,char* buf_end);
 
 #ifdef DO_STACKTRACE
@@ -353,7 +357,7 @@ LOG_MSG_FUNC(log_info,LOG_INFO)
 #ifndef DBUG_OFF
 LOG_MSG_FUNC(log_debug,LOG_DEBUG)
 #else
-void log_debug(const char* __attribute__((unused)) fmt,...) {}
+void log_debug(const char* fmt __attribute__((unused)),...) {}
 #endif
 
 static void handle_sigterm(int sig __attribute__((unused)))
@@ -428,8 +432,8 @@ static int exec_line(struct manager_thd* thd,char* buf,char* buf_end)
 {
   char* p=buf;
   struct manager_cmd* cmd;
-  for (;p<buf_end && !isspace(*p);p++)
-    *p=tolower(*p);
+  for (;p<buf_end && !my_isspace(cs,*p);p++)
+    *p=my_tolower(cs,*p);
   log_info("Command '%s'", buf);
   if (!(cmd=lookup_cmd(buf,(int)(p-buf))))
   {
@@ -439,7 +443,7 @@ static int exec_line(struct manager_thd* thd,char* buf,char* buf_end)
       thd->fatal=1;
     return 1;
   }
-  for (;p<buf_end && isspace(*p);p++);
+  for (;p<buf_end && my_isspace(cs,*p);p++);
   return cmd->handler_func(thd,p,buf_end);
 }
 
@@ -553,14 +557,13 @@ HANDLE_DECL(handle_set_exec_stderr)
 static int set_exec_param(struct manager_thd* thd, char* args_start,
 			  char* args_end, PARAM_TYPE param_type)
 {
-  int num_args;
   const char* error=0;
   struct manager_exec* e;
   char* arg_p;
   char* param;
   int param_size;
 
-  if ((num_args=tokenize_args(args_start,&args_end))<2)
+  if (tokenize_args(args_start,&args_end) < 2)
   {
     error="Too few arguments";
     goto err;
@@ -603,12 +606,11 @@ err:
 
 HANDLE_DECL(handle_start_exec)
 {
-  int num_args;
   struct manager_exec* e;
   int ident_len;
   const char* error=0;
   struct timespec t;
-  if ((num_args=tokenize_args(args_start,&args_end))<1)
+  if (tokenize_args(args_start,&args_end) < 1)
   {
     error="Too few arguments";
     goto err;
@@ -652,12 +654,11 @@ err:
 
 HANDLE_DECL(handle_stop_exec)
 {
-  int num_args;
   struct timespec abstime;
   struct manager_exec* e;
   int ident_len;
   const char* error=0;
-  if ((num_args=tokenize_args(args_start,&args_end))<2)
+  if (tokenize_args(args_start,&args_end) <2)
   {
     error="Too few arguments";
     goto err;
@@ -683,7 +684,7 @@ HANDLE_DECL(handle_stop_exec)
     error="Process not running";
     goto err;
   }
-  if (mysql_shutdown(&e->mysql))
+  if (mysql_shutdown(&e->mysql, SHUTDOWN_DEFAULT))
   {
     /* e->th=0; */	/* th may be a struct */
     pthread_mutex_unlock(&e->lock);
@@ -716,7 +717,7 @@ HANDLE_DECL(handle_query)
   int num_fields,i,ident_len;
   char* ident,*query;
   query=ident=args_start;
-  while (!isspace(*query))
+  while (!my_isspace(cs,*query))
     query++;
   if (query == ident)
   {
@@ -724,7 +725,7 @@ HANDLE_DECL(handle_query)
     goto err;
   }
   ident_len=(int)(query-ident);
-  while (query<args_end && isspace(*query))
+  while (query<args_end && my_isspace(cs,*query))
     query++;
   if (query == args_end)
   {
@@ -816,7 +817,7 @@ HANDLE_DECL(handle_def_exec)
     update_req_len(e);
     hash_delete(&exec_hash,(byte*)old_e);
   }
-  hash_insert(&exec_hash,(byte*)e);
+  my_hash_insert(&exec_hash,(byte*)e);
   pthread_mutex_unlock(&lock_exec_hash);
   client_msg(&thd->net,MANAGER_OK,"Exec definition created");
   return 0;
@@ -873,7 +874,10 @@ static void manager_exec_connect(struct manager_exec* e)
   {
     if (mysql_real_connect(&e->mysql,e->con_host,e->con_user,e->con_pass,0,
 			   e->con_port,e->con_sock,0))
+    {
+      e->mysql.reconnect= 1;
       return;
+    }
     sleep(1);
   }
   e->error="Could not connect to MySQL server withing the number of tries";
@@ -1000,7 +1004,7 @@ static int authenticate(struct manager_thd* thd)
   for (buf=thd->cmd_buf,p=thd->user,p_end=p+MAX_USER_NAME;
        buf<buf_end && (c=*buf) && p<p_end; buf++,p++)
   {
-    if (isspace(c))
+    if (my_isspace(cs,c))
     {
       *p=0;
       break;
@@ -1013,7 +1017,7 @@ static int authenticate(struct manager_thd* thd)
   if (!(u=(struct manager_user*)hash_search(&user_hash,thd->user,
 					    (uint)(p-thd->user))))
     return 1;
-  for (;isspace(*buf) && buf<buf_end;buf++) /* empty */;
+  for (;my_isspace(cs,*buf) && buf<buf_end;buf++) /* empty */;
 
   my_MD5Init(&context);
   my_MD5Update(&context,(uchar*) buf,(uint)(buf_end-buf));
@@ -1085,8 +1089,7 @@ static void log_msg(const char* fmt, int msg_type, va_list args)
   pthread_mutex_unlock(&lock_log);
 }
 
-static pthread_handler_decl(process_launcher_messages,
-			    args __attribute__((unused)))
+pthread_handler_t process_launcher_messages(void *arg __attribute__((unused)))
 {
   my_thread_init();
   for (;!in_shutdown;)
@@ -1142,7 +1145,7 @@ static pthread_handler_decl(process_launcher_messages,
   return 0;
 }
 
-static pthread_handler_decl(process_connection,arg)
+pthread_handler_t process_connection(void *arg)
 {
   struct manager_thd* thd = (struct manager_thd*)arg;
   my_thread_init();
@@ -1582,9 +1585,9 @@ static void manager_exec_free(void* e)
 
 static int hex_val(char c)
 {
-  if (isdigit(c))
+  if (my_isdigit(cs,c))
     return c-'0';
-  c=tolower(c);
+  c=my_tolower(cs,c);
   return c-'a'+10;
 }
 
@@ -1641,7 +1644,8 @@ static void init_user_hash()
   FILE* f;
   char buf[80];
   int line_num=1;
-  if (hash_init(&user_hash,1024,0,0,get_user_key,manager_user_free,MYF(0)))
+  if (hash_init(&user_hash,cs,1024,0,0,
+                get_user_key,manager_user_free,MYF(0)))
     die("Could not initialize user hash");
   if (!(f=my_fopen(manager_pw_file, O_RDONLY | O_BINARY, MYF(MY_WME))))
   {
@@ -1663,7 +1667,7 @@ static void init_user_hash()
     }
     else
     {
-      hash_insert(&user_hash,(gptr)u);
+      my_hash_insert(&user_hash,(gptr)u);
     }
   }
   my_fclose(f, MYF(0));
@@ -1687,7 +1691,8 @@ static void init_pid_file()
 static void init_globals()
 {
   pthread_attr_t thr_attr;
-  if (hash_init(&exec_hash,1024,0,0,get_exec_key,manager_exec_free,MYF(0)))
+  if (hash_init(&exec_hash,cs,1024,0,0,
+                get_exec_key,manager_exec_free,MYF(0)))
     die("Exec hash initialization failed");
   if (!one_thread)
   {
@@ -1846,3 +1851,16 @@ int main(int argc, char** argv)
   else
     return daemonize();
 }
+
+#else
+
+#include <stdio.h>
+
+int main(void)
+{
+  fprintf(stderr,"This tool has not been ported to NetWare\n");
+  return 0;
+}
+
+#endif /* __NETWARE__ */
+
