@@ -57,8 +57,8 @@ main(int argc, char**	argv)
   char*	cipher=0;
   int	child_pid,sv[2];
   my_bool unused;
-  struct st_VioSSLAcceptorFd* ssl_acceptor=0;
-  struct st_VioSSLConnectorFd* ssl_connector=0; 
+  struct st_VioSSLFd* ssl_acceptor= 0;
+  struct st_VioSSLFd* ssl_connector= 0;
   Vio* client_vio=0, *server_vio=0;
   MY_INIT(argv[0]);
   DBUG_PROCESS(argv[0]);
@@ -115,7 +115,7 @@ main(int argc, char**	argv)
   {
     /* child, therefore, client */
     char	xbuf[100];
-    int	r = client_vio->read(client_vio,xbuf, sizeof(xbuf));
+    int	r = vio_read(client_vio,xbuf, sizeof(xbuf));
     if (r<=0) {
       my_free((gptr)ssl_acceptor,MYF(0));
       my_free((gptr)ssl_connector,MYF(0));
@@ -130,7 +130,7 @@ main(int argc, char**	argv)
   else
   {
     const char*	s = "Huhuhuh";
-    int		r = server_vio->write(server_vio,(gptr)s, strlen(s));
+    int		r = vio_write(server_vio,(gptr)s, strlen(s));
     if (r<=0) {
       my_free((gptr)ssl_acceptor,MYF(0));
       my_free((gptr)ssl_connector,MYF(0));
